#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "audio.h"
#include "game.h"

extern Game* game;

//-------------------------------------------------------------------------------------------------
Audio::Audio(char *datapath_dir)
{
    MusicVolume = 64;
    SoundVolume = 64;
    CurrentlySelectedMusicTrack = 0;

    for (int soundIndex = 0; soundIndex < SoundTotal; soundIndex++)  SoundFX[soundIndex] = NULL;
    for (int musicIndex = 0; musicIndex < MusicTotal; musicIndex++)  MusicTrack[musicIndex] = NULL;

    AudioWorking = false;

    strcpy(datapath, datapath_dir);
    printf("Audio: %s\n", datapath);
}

//-------------------------------------------------------------------------------------------------
Audio::~Audio(void)
{
    if (AudioWorking == false)  return;

    Mix_HaltChannel(-1);
    for (int soundIndex = 0; soundIndex < SoundTotal; soundIndex++)
    {
        if (SoundFX[soundIndex] != NULL)  Mix_FreeChunk(SoundFX[soundIndex]);
        SoundFX[soundIndex] = NULL;
    }
    Mix_AllocateChannels(0);
    printf("Unloaded all music from memory\n");

    Mix_HaltMusic();
    for (int musicIndex = 0; musicIndex < MusicTotal; musicIndex++)
    {
        if (MusicTrack[musicIndex] != NULL)  Mix_FreeMusic(MusicTrack[musicIndex]);
        MusicTrack[musicIndex] = NULL;
    }
    printf("Unloaded all sounds from memory\n");

    Mix_CloseAudio();
    printf("SDL_mixer closed\n");

}

//-------------------------------------------------------------------------------------------------
bool Audio::SetupAudio(void)
{
    char filePath[256];

    if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 1/*MIX_DEFAULT_CHANNELS*/, 1024)==-1)
    {
        printf("ERROR: SDL_Mixer init failed: %s\n", Mix_GetError());
        return false;
    }
    else printf("SDL_mixer initialized\n");

    Mix_AllocateChannels(16);
    for (int channel = 0; channel < 16; channel++)
    {
        Mix_Volume(channel, SoundVolume);
    }

    for (int index = 0; index < MusicTotal; index++)
    {
        MusicTrack[index] = NULL;
	strcpy(filePath, datapath);

	switch(index)
	{
	    case 0:
		strcat(filePath, "music/Title-BGM2.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

	    case 1:
		strcat(filePath, "music/Title-BGM2.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 2:
		strcat(filePath, "music/New-High-Score.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 3:
		strcat(filePath, "music/Crisis-I-Mode-Level7.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 4:
		strcat(filePath, "music/Track-1-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 5:
		strcat(filePath, "music/Track-2-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 6:
		strcat(filePath, "music/Track-3-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 7:
		strcat(filePath, "music/Track-4-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 8:
		strcat(filePath, "music/Track-5-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 9:
		strcat(filePath, "music/Track-6-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 10:
		strcat(filePath, "music/Track-7-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 11:
		strcat(filePath, "music/Track-8-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 12:
		strcat(filePath, "music/Track-9-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 13:
		strcat(filePath, "music/Track-10-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 14:
		strcat(filePath, "music/Track-11-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 15:
		strcat(filePath, "music/Track-12-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 16:
		strcat(filePath, "music/Track-13-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
		break;

            case 17:
		strcat(filePath, "music/Track-14-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 18:
		strcat(filePath, "music/Track-15-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 19:
		strcat(filePath, "music/Track-16-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 20:
		strcat(filePath, "music/Track-17-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 21:
		strcat(filePath, "music/Track-18-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 22:
		strcat(filePath, "music/Track-19-BGM.mod");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            case 24:
		strcat(filePath, "music/Win.ogg");
		MusicTrack[index] = Mix_LoadMUS(filePath);
                break;

            default:
                break;
        }

        if(!MusicTrack[index] && MusicTotal != 0)
        {
            printf("ERROR: Couldn't load music %s\n", Mix_GetError());
        }
        Mix_VolumeMusic(MusicVolume);
    }
    printf("Loaded all music into memory\n");


    for (int index = 0; index < SoundTotal; index++)
    {
        SoundFX[index] = NULL;
	strcpy(filePath, datapath);

        switch(index)
        {
            case 0:
		strcat(filePath, "sound/Click.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

            case 1:
		strcat(filePath, "sound/Click.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 2:
		strcat(filePath, "sound/Move-Piece.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 3:
		strcat(filePath, "sound/Piece-Collision.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 4:
		strcat(filePath, "sound/Piece-Drop.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 5:
		strcat(filePath, "sound/Piece-Rotate.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 6:
		strcat(filePath, "sound/Line-Cleared.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 7:
		strcat(filePath, "sound/Tetri-Cleared.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 8:
		strcat(filePath, "sound/Level-Up.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 9:
		strcat(filePath, "sound/Must-Think-In-Russian.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 10:
		strcat(filePath, "sound/Incoming-Line.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 11:
		strcat(filePath, "sound/Game-Over.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 12:
		strcat(filePath, "sound/Crack.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

	    case 13:
		strcat(filePath, "sound/Shall-We-Play-A-Game.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);
		break;

            case 14:
		strcat(filePath, "sound/Sword.wav");
		SoundFX[index] = Mix_LoadWAV(filePath);

            default:
                break;
        }

        if(!SoundFX[index])
        {
            printf("ERROR: Couldn't load sound %s\n", Mix_GetError());
            return false;
        }
        Mix_VolumeChunk(SoundFX[index], MIX_MAX_VOLUME);
    }
    printf("Loaded all sounds into memory\n");

    AudioWorking = true;
    return true;
}

//-------------------------------------------------------------------------------------------------
void Audio::PlayMusic(Uint8 musicIndex, int loop)
{
    if (AudioWorking == false)  return;

    if (MusicTrack[musicIndex] == NULL)  return;

    if (musicIndex >= MusicTotal)  return;

    if (Mix_PlayingMusic() == 1)  Mix_HaltMusic();

    if (MusicVolume == 0)  return;

    Mix_VolumeMusic(MusicVolume);

    CurrentlySelectedMusicTrack = musicIndex;

    if(Mix_PlayMusic(MusicTrack[musicIndex], loop)==-1)
    {
        printf("Mix_PlayMusic: %s\n", Mix_GetError());
    }
}

//-------------------------------------------------------------------------------------------------
void Audio::PlayDigitalSoundFX(Uint8 soundIndex, int loops)
{
if (game->Debug == 2)  return;

    if (AudioWorking == false)  return;

    if (SoundFX[soundIndex] == NULL)  return;

    if (soundIndex >= SoundTotal)  return;

    if (SoundVolume == 0)  return;

    for (int index = 0; index < 16; index++)
        Mix_Volume( index, SoundVolume );

    for (int indexTwo = 0; indexTwo < SoundTotal; indexTwo++)
        Mix_VolumeChunk(SoundFX[indexTwo], SoundVolume);

    if(Mix_PlayChannel(-1, SoundFX[soundIndex], loops)==-1)
    {
        printf("Mix_PlayChannel: %s\n",Mix_GetError());
    }
}
//-------------------------------------------------------------------------------------------------
