#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "game.h"

#include "visuals.h"
#include "input.h"
#include "logic.h"

extern Visuals* visuals;
extern Input* input;
extern Logic* logic;

//-------------------------------------------------------------------------------------------------
Game::Game(void)
{
    CoreFailure = false;
    ExitGame = false;
    Debug = false;

    NetBookMode = false;

    FullScreenMode = false;

    BoxTileSheet = 0;

    AIlogic = 0;

    CPUPlayerEnabled = 1;

    FrameLock = 16;
    SystemTicks = SDL_GetTicks();
    NextFrameTicks = SystemTicks + FrameLock;
    NumberOfFrames = 0;

    CurrentFramePerSecond = 0;

    for (int index = 0; index < 10; index++)
    {
        FramesPerSecondArray[index] = 0;
    }

    NextSecondTick = 0;
    AverageFPS = 0;
}

//-------------------------------------------------------------------------------------------------
Game::~Game(void)
{

}

//-------------------------------------------------------------------------------------------------
void Game::CalculateFramerate(void)
{
    SystemTicks = SDL_GetTicks();
    NextFrameTicks = SystemTicks + FrameLock;

    NumberOfFrames++;

    if (SystemTicks > NextSecondTick)
    {
        NextSecondTick = SystemTicks + 1000;

        FramesPerSecondArray[CurrentFramePerSecond] = NumberOfFrames;
        NumberOfFrames = 0;

        if (CurrentFramePerSecond < 9)  CurrentFramePerSecond++;
        else  CurrentFramePerSecond = 0;

        Uint32 frameTotal = 0;
        for (int index = 0; index < 10; index++)
        {
            frameTotal += FramesPerSecondArray[index];
        }

        AverageFPS = frameTotal / 10;
    }
}

//-------------------------------------------------------------------------------------------------
void Game::DisplayDebuggingInfo(void)
{
    if (Debug == 2)
    {
        visuals->DrawTextOntoScreenBuffer("Fast Unseen Game...", visuals->FontDefault[2], 3, 413
                                          , JustifyLeft, 255, 255, 0, 0, 0, 0);
        CPUPlayerEnabled = 4;
    }

    if (Debug == 2 || Debug == 3)
    {
        int averageCPULinesPerGame = 0;
        if (logic->NumberofCPUGames > 0) averageCPULinesPerGame = (logic->TotalCPUPlayerLines / logic->NumberofCPUGames);

        //DEBUG info...
        char temp[64];
        char tempTwo[64];
        sprintf(temp, "%d", averageCPULinesPerGame);
        strcpy(visuals->VariableText, "CPU AVG Lines:");
        strcat(visuals->VariableText, temp);
        strcat(visuals->VariableText, " #Games:");
        sprintf(tempTwo, "%d", logic->NumberofCPUGames);
        strcat(visuals->VariableText, tempTwo);

        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], 0, 0
                                          , JustifyCenter, 0, 0, 0, 255, 255, 255);
    }

    if (Debug != 4)
    {
        sprintf(visuals->VariableText, "%i", AverageFPS);

        visuals->DrawTextOntoScreenBuffer("0000", visuals->FontDefault[2], 3, 429
                                          , JustifyLeft, 255, 255, 255, 255, 255, 255);

        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[2], 3, 429
                                          , JustifyLeft, 1, 1, 1, 255, 255, 255);

        sprintf(visuals->VariableText, "%i", input->MouseX);

        visuals->DrawTextOntoScreenBuffer("0000", visuals->FontDefault[2], 3, 445
                                          , JustifyLeft, 255, 255, 255, 255, 255, 255);

        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[2], 3, 445
                                          , JustifyLeft, 1, 1, 1, 255, 255, 255);

        sprintf(visuals->VariableText, "%i", input->MouseY);

        visuals->DrawTextOntoScreenBuffer("0000", visuals->FontDefault[2], 3, 461
                                          , JustifyLeft, 255, 255, 255, 255, 255, 255);

        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[2], 3, 461
                                          , JustifyLeft, 1, 1, 1, 255, 255, 255);
    }
}

//-------------------------------------------------------------------------------------------------
void Game::ProcessFramerate(void)
{
    SystemTicks = SDL_GetTicks();
    if (NextFrameTicks > SystemTicks)  SDL_Delay(NextFrameTicks - SystemTicks);
}
//-------------------------------------------------------------------------------------------------
