#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "interface.h"

#include "visuals.h"
#include "screens.h"
#include "input.h"
#include "audio.h"

extern Visuals* visuals;
extern Screens* screens;
extern Input* input;
extern Audio* audio;

//-------------------------------------------------------------------------------------------------
Interface::Interface(char *datapath_dir)
{
    for (int index = 0; index < NumberOfGUIs; index++)
    {
        GUIs[index].OGL_Texture = (GLuint)NULL;
    }
    strcpy(datapath, datapath_dir);
    printf("Interface: %s\n", datapath);
}

//-------------------------------------------------------------------------------------------------
Interface::~Interface(void)
{
    if (ButtonsFont != NULL)  TTF_CloseFont(ButtonsFont);

    for (int index = 0; index < NumberOfGUIs; index++)
    {
        if (GUIs[index].OGL_Texture)  glDeleteTextures( 1, &GUIs[index].OGL_Texture );
    }
    printf("Unloaded all button/GUI images from memory\n");
}

//-------------------------------------------------------------------------------------------------
bool Interface::CreateAllGUIAndInitialize(bool initialize)
{
SDL_Surface *tempSurface;
bool loadedImageIsOK = true;
GLenum textureFormat = 0;
GLint  numberOfColors;
bool imageToTextureOK = true;
char filePath[256];
SDL_Surface *tempSurfaceFixed = NULL;
Uint32 saved_flags;
Uint8  saved_alpha;
char buttonText[64];

    ButtonsFont = NULL;
    strcpy(filePath, datapath);
    strcat(filePath, "interface/Default.ttf");
    if ( !(ButtonsFont = TTF_OpenFont(filePath, 27)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    for (int indexGUI = 0; indexGUI < NumberOfGUIs; indexGUI++)
    {
        strcpy(filePath, datapath);
        if      (indexGUI == 0)  strcat(filePath, "interface/Button-Selector-Left.png");
        else if (indexGUI == 1)  strcat(filePath, "interface/Button-Selector-Right.png");
        else if (indexGUI >  1 && indexGUI < 10)  strcat(filePath, "interface/Button.png");
        else if (indexGUI == 10)  strcat(filePath, "interface/Selector-Line.png");
        else if (indexGUI % 2 != 0)  strcat(filePath, "interface/Button-Selector-Right.png");
        else   strcat(filePath, "interface/Button-Selector-Left.png");

        strcat(filePath, "\0");

        tempSurface = NULL;
        if ( ((tempSurface = IMG_Load(filePath)) != NULL) )
        {
            if ( (tempSurface->w & (tempSurface->w - 1)) != 0 )
            {
                printf("ERROR: Image width is not a power of 2\n");
                loadedImageIsOK = false;
            }

            if ( (tempSurface->h & (tempSurface->h - 1)) != 0 )
            {
                printf("ERROR: Image height is not a power of 2\n");
                loadedImageIsOK = false;
            }

            if (loadedImageIsOK == true)
            {
                tempSurfaceFixed = SDL_CreateRGBSurface(SDL_SWSURFACE, tempSurface->w, tempSurface->h, 32
                                                        , visuals->R_Mask, visuals->G_Mask, visuals->B_Mask, visuals->A_Mask);

                saved_flags = tempSurface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
                saved_alpha = tempSurface->format->alpha;
                if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
                {
                    SDL_SetAlpha(tempSurface, 0, 0);
                }

                SDL_BlitSurface(tempSurface, NULL, tempSurfaceFixed, NULL);

                if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
                {
                    SDL_SetAlpha(tempSurfaceFixed, saved_flags, saved_alpha);
                }

				numberOfColors = tempSurface->format->BytesPerPixel;
				if (numberOfColors == 4)
				{
				    textureFormat = GL_RGBA;
				}
				else if (numberOfColors == 3)
				{
					textureFormat = GL_RGB;
				}
				else
				{
					printf("ERROR: The image is not truecolor\n");
					imageToTextureOK = false;
				}

                if (initialize == true)
                {
                    GUIs[indexGUI].OnScreenIndex = -1;
                    GUIs[indexGUI].ScreenX = 320.0f;
                    GUIs[indexGUI].ScreenY = 240.0f;
                    GUIs[indexGUI].ScaleX = 1.0f;
                    GUIs[indexGUI].ScaleY = 1.0f;
                    GUIs[indexGUI].RotationDegree = 0.0f;
                    GUIs[indexGUI].Transparency = 1.0f;
                    GUIs[indexGUI].RedHue = 1.0f;
                    GUIs[indexGUI].GreenHue = 1.0f;
                    GUIs[indexGUI].BlueHue = 1.0f;
                    GUIs[indexGUI].Smooth = true;
                    GUIs[indexGUI].FlipX = false;
                    GUIs[indexGUI].FlipY = false;
                    GUIs[indexGUI].Width = tempSurface->w;
                    GUIs[indexGUI].Height = tempSurface->h;
                    GUIs[indexGUI].OffsetX = 3;
                    GUIs[indexGUI].OffsetY = 12;
                    GUIs[indexGUI].CenterX = tempSurface->w / 2;
                    GUIs[indexGUI].CenterY = tempSurface->h / 2;
                    GUIs[indexGUI].AnimationTimer = -1;
                }

                if (indexGUI > 1 && indexGUI < 10)
                {
                    SDL_Color textColor;
                    SDL_Color outlineColor;
                    SDL_Surface *text;
                    SDL_Surface *textOutline;
                    SDL_Surface *textAndOutline;
                    int w = tempSurface->w;
                    int h = tempSurface->h;
                    SDL_Rect rect;

                    textColor.r = 0; textColor.g = 0; textColor.b = 0;
                    outlineColor.r = 128; outlineColor.g = 128; outlineColor.b = 128;

                    switch(indexGUI)
                    {
                        case 2:
                            strcpy(buttonText, "Start!\0");
                            break;

                        case 3:
                            strcpy(buttonText, "Options\0");
                            break;

                        case 4:
                            strcpy(buttonText, "How To Play\0");
                            break;

                        case 5:
                            strcpy(buttonText, "High Scores\0");
                            break;

                        case 6:
                            strcpy(buttonText, "About\0");
                            break;

                        case 7:
                            strcpy(buttonText, "Exit\0");
                            break;

                        case 8:
                            strcpy(buttonText, "Back\0");
                            break;

                        default:
                            strcpy(buttonText, " \0");
                            break;
                    }

                    text = NULL;
                    textOutline = NULL;

                    text = TTF_RenderText_Blended(ButtonsFont, buttonText, textColor);
                    textOutline = TTF_RenderText_Solid(ButtonsFont, buttonText, outlineColor);

                    textAndOutline = SDL_CreateRGBSurface(0, w, h, 32, visuals->R_Mask, visuals->G_Mask, visuals->B_Mask, visuals->A_Mask);

                    Sint16 posX = (w / 2) - (text->w / 2);
                    Sint16 posY = (h / 2 + 1) - (text->h / 2);

                    for (Sint16 yOffSet = -1; yOffSet < 2; yOffSet++)
                        for (Sint16 xOffSet = -1; xOffSet < 2; xOffSet++)
                            {
                                rect.x = posX+xOffSet; rect.y = posY+yOffSet;
                                SDL_BlitSurface(textOutline, 0, tempSurfaceFixed, &rect);
                            }

                    rect.x = posX;
                    rect.y = posY;

                    SDL_BlitSurface(text, 0, tempSurfaceFixed, &rect);

                    SDL_FreeSurface(text);
                    SDL_FreeSurface(textOutline);
                    SDL_FreeSurface(textAndOutline);
                }

                GUIs[indexGUI].OGL_Texture = (GLuint)NULL;
                if (imageToTextureOK == true)
                {
                    glGenTextures( 1, &GUIs[indexGUI].OGL_Texture );

                    glBindTexture( GL_TEXTURE_2D, GUIs[indexGUI].OGL_Texture );

                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
                    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

                    glTexImage2D( GL_TEXTURE_2D, 0, numberOfColors, tempSurface->w, tempSurface->h, 0,
                                  textureFormat, GL_UNSIGNED_BYTE, tempSurfaceFixed->pixels );

                    if ( tempSurface )
                    {
                        SDL_FreeSurface( tempSurface );
                        SDL_FreeSurface(tempSurfaceFixed);
                    }
                }
            }
        }
        else
        {
            printf("ERROR: SDL could not load image %s\n", SDL_GetError());
            loadedImageIsOK = false;
            imageToTextureOK = false;
        }

        if (loadedImageIsOK == false || imageToTextureOK == false)  return false;
    }

    if (loadedImageIsOK == true && imageToTextureOK == true)
    {
        if (initialize == true  )printf("Loaded and initialized all button/GUI images into memory\n");
        else  printf("Loaded all button/GUI images into memory\n");

        return true;
    }

    return true;
}

//-------------------------------------------------------------------------------------------------
void Interface::CreateButton(int imageIndex, int screenIndex, GLfloat screenX, GLfloat screenY)
{
    GUIs[imageIndex].OnScreenIndex = screenIndex;
    GUIs[imageIndex].ScreenX = screenX;
    GUIs[imageIndex].ScreenY = screenY;
    GUIs[imageIndex].ScaleX = 1.0f;
    GUIs[imageIndex].ScaleY = 1.0f;
    GUIs[imageIndex].RotationDegree = 0.0f;
    GUIs[imageIndex].Transparency = 0.8f;
    GUIs[imageIndex].RedHue = 1.0f;
    GUIs[imageIndex].GreenHue = 1.0f;
    GUIs[imageIndex].BlueHue = 1.0f;
    GUIs[imageIndex].Smooth = true;
    GUIs[imageIndex].FlipX = false;
    GUIs[imageIndex].FlipY = false;
    GUIs[imageIndex].AnimationTimer = -1;
}

//-------------------------------------------------------------------------------------------------
void Interface::DisplayAllButtonsOntoScreenBuffer(void)
{
    for (int index = 0; index < 10; index++)
    {
        if (GUIs[index].OnScreenIndex > -1)
        {
            glPushMatrix();

            glBindTexture(GL_TEXTURE_2D, GUIs[index].OGL_Texture);
            glEnable(GL_TEXTURE_2D);

            if (GUIs[index].AnimationTimer > 0)  { GUIs[index].ScaleX = 0.85f; GUIs[index].ScaleY = 0.85f; }
            else  { GUIs[index].ScaleX = 1.0f; GUIs[index].ScaleY = 1.0f; }

            glTranslatef(GUIs[index].ScreenX, GUIs[index].ScreenY, 0.0f);

            glScalef(GUIs[index].ScaleX, GUIs[index].ScaleY, 1.0f);

            glRotatef(GUIs[index].RotationDegree, 0.0f, 0.0f, 1.0f);

            glColor4f(GUIs[index].RedHue, GUIs[index].GreenHue, GUIs[index].BlueHue
                      , GUIs[index].Transparency);

            if (GUIs[index].Smooth == false)
            {

            }
            else
            {
                glEnable(GL_BLEND);
                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            }

            GLfloat zeroWidth = 0, oneWidth = 0;
            GLfloat zeroHeight = 0, oneHeight = 0;

            if (GUIs[index].FlipX == false)
            {
                zeroWidth = -(GUIs[index].Width / 2); oneWidth = (GUIs[index].Width / 2);
            }
            else if (GUIs[index].FlipX == true)
            {
                zeroWidth = (GUIs[index].Width); oneWidth = 0;
            }

            if (GUIs[index].FlipY == false)
            {
                zeroHeight = -(GUIs[index].Height / 2); oneHeight = (GUIs[index].Height / 2);
            }
            else if (GUIs[index].FlipY == true)
            {
                zeroHeight = (GUIs[index].Height); oneHeight = 0;
            }

            int numvertices = 4;
            glEnableClientState( GL_VERTEX_ARRAY );
            glEnableClientState( GL_TEXTURE_COORD_ARRAY );

            GLfloat *vertices = new float[3*numvertices];
            GLfloat *textcoord = new float[2*numvertices];

            textcoord[0] = 0;
            textcoord[1] = 0;
            textcoord[2] = 1;
            textcoord[3] = 0;
            textcoord[4] = 1;
            textcoord[5] = 1;
            textcoord[6] = 0;
            textcoord[7] = 1;

            vertices[0] = zeroWidth;
            vertices[1] = zeroHeight;
            vertices[2] = 0;
            vertices[3] = oneWidth;
            vertices[4] = zeroHeight;
            vertices[5] = 0;
            vertices[6] = oneWidth;
            vertices[7] = oneHeight;
            vertices[8] = 0;
            vertices[9] = zeroWidth;
            vertices[10] = oneHeight;
            vertices[11] = 0;

            glVertexPointer( 3, GL_FLOAT, 0, vertices );
            glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

            glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
            delete textcoord;
            delete vertices;

            glPopMatrix();
        }
    }

    for (int index = 2; index < 10; index++)
    {
        if (GUIs[index].OnScreenIndex == ButtonSelected)
        {
            GUIs[0].ScreenX = (GUIs[index].ScreenX - 150);
            GUIs[0].ScreenY = GUIs[index].ScreenY;
            GUIs[1].ScreenX = (GUIs[index].ScreenX + 150);
            GUIs[1].ScreenY = GUIs[index].ScreenY;
        }
    }

    for (int index = 0; index < 2; index++)
    {
        glPushMatrix();

        glBindTexture(GL_TEXTURE_2D, GUIs[index].OGL_Texture);
        glEnable(GL_TEXTURE_2D);

        glTranslatef(GUIs[index].ScreenX, GUIs[index].ScreenY, 0.0f);

        if (GUIs[index].AnimationTimer > 0)  { GUIs[index].ScaleX = 0.85f; GUIs[index].ScaleY = 0.85f; }
        else  { GUIs[index].ScaleX = 1.0f; GUIs[index].ScaleY = 1.0f; }
        glScalef(GUIs[index].ScaleX, GUIs[index].ScaleY, 1.0f);

        glRotatef(GUIs[index].RotationDegree, 0.0f, 0.0f, 1.0f);

        glColor4f(GUIs[index].RedHue, GUIs[index].GreenHue, GUIs[index].BlueHue
                  , GUIs[index].Transparency);

        if (GUIs[index].Smooth == false)
        {

        }
        else
        {
            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        }

        GLfloat zeroWidth = 0, oneWidth = 0;
        GLfloat zeroHeight = 0, oneHeight = 0;

        if (GUIs[index].FlipX == false)
        {
            zeroWidth = -(GUIs[index].Width / 2); oneWidth = (GUIs[index].Width / 2);
        }
        else if (GUIs[index].FlipX == true)
        {
            zeroWidth = (GUIs[index].Width); oneWidth = 0;
        }

        if (GUIs[index].FlipY == false)
        {
            zeroHeight = -(GUIs[index].Height / 2); oneHeight = (GUIs[index].Height / 2);
        }
        else if (GUIs[index].FlipY == true)
        {
            zeroHeight = (GUIs[index].Height); oneHeight = 0;
        }

        int numvertices = 4;
        glEnableClientState( GL_VERTEX_ARRAY );
        glEnableClientState( GL_TEXTURE_COORD_ARRAY );

        GLfloat *vertices = new float[3*numvertices];
        GLfloat *textcoord = new float[2*numvertices];

        textcoord[0] = 0;
        textcoord[1] = 0;
        textcoord[2] = 1;
        textcoord[3] = 0;
        textcoord[4] = 1;
        textcoord[5] = 1;
        textcoord[6] = 0;
        textcoord[7] = 1;

        vertices[0] = zeroWidth;
        vertices[1] = zeroHeight;
        vertices[2] = 0;
        vertices[3] = oneWidth;
        vertices[4] = zeroHeight;
        vertices[5] = 0;
        vertices[6] = oneWidth;
        vertices[7] = oneHeight;
        vertices[8] = 0;
        vertices[9] = zeroWidth;
        vertices[10] = oneHeight;
        vertices[11] = 0;

        glVertexPointer( 3, GL_FLOAT, 0, vertices );
        glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

        glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
        delete textcoord;
        delete vertices;

        glPopMatrix();

        screens->ScreenIsDirty = true;
    }
}

//-------------------------------------------------------------------------------------------------
void Interface::ProcessAllButtonsOnScreen(void)
{
int minButtonScreenIndex = 0;
int maxButtonScreenIndex = 0;

    for (int index = 0; index < 10; index++)
    {
        if (GUIs[index].OnScreenIndex > -1)
        {
            if (GUIs[index].AnimationTimer == 10 || GUIs[index].AnimationTimer == 0)
                screens->ScreenMustBeDrawn = true;

            if (GUIs[index].AnimationTimer > -1)  GUIs[index].AnimationTimer--;

            if (GUIs[index].AnimationTimer == 0)  screens->ScreenTransitionStatus = FadeOut;

            if (input->MouseButtonPressed[0] == true)
            {
                if (   (  input->MouseX > ( GUIs[index].ScreenX - ((GUIs[index].Width / 2) - GUIs[index].OffsetX) )
                   && input->MouseX < ( GUIs[index].ScreenX + ((GUIs[index].Width / 2) - GUIs[index].OffsetX) )  )
                   && (  input->MouseY > ( GUIs[index].ScreenY - ((GUIs[index].Height / 2) - GUIs[index].OffsetY) )
                   && input->MouseY < ( GUIs[index].ScreenY + ((GUIs[index].Height / 2) - GUIs[index].OffsetY) )  )   )
                {
                    GUIs[index].AnimationTimer = 10;
                    ButtonSelected = GUIs[index].OnScreenIndex;
                    input->DelayAllUserInput = 22;
                    audio->PlayDigitalSoundFX(0, 0);

                }
            }
            else if ( (input->JoystickButtonOne[Any] == ON)
                     || (input->KeyOnKeyboardPressedByUser == '_' && screens->ScreenToDisplay != NameInputKeyboardScreen)
                     || input->KeyOnKeyboardPressedByUser == '/')
            {

                if (screens->ScreenToDisplay != NameInputJoystickScreen)
                {
                    if (ButtonSelected == GUIs[index].OnScreenIndex)
                    {
                        GUIs[index].AnimationTimer = 10;
                        index = NumberOfGUIs;
                        input->DelayAllUserInput = 22;
                        audio->PlayDigitalSoundFX(0, 0);
                    }
                }
            }
        }
    }

    if (input->JoystickDirection[Any] != CENTER)
    {
        minButtonScreenIndex = 0;
        for (int index = 9; index > 1; index--)
        {
            if (GUIs[index].OnScreenIndex > -1)
            {
                minButtonScreenIndex = GUIs[index].OnScreenIndex;
            }
        }

        maxButtonScreenIndex = 0;
        for (int index = 2; index < 10; index++)
        {
            if (GUIs[index].OnScreenIndex > -1)
            {
                maxButtonScreenIndex = GUIs[index].OnScreenIndex;
            }
        }
    }

    if (input->JoystickDirection[Any] == UP)
    {
        if (ButtonSelected > 0 && ButtonSelected > minButtonScreenIndex)  ButtonSelected--;
        else
        {
            for (int index = 2; index < 10; index++)
            {
                if (GUIs[index].OnScreenIndex > -1)
                {
                    ButtonSelected = GUIs[index].OnScreenIndex;
                }
            }
        }

        if (maxButtonScreenIndex != 0)
        {
            input->DelayAllUserInput = 10;
            screens->ScreenMustBeDrawn = true;
            audio->PlayDigitalSoundFX(0, 0);
        }
    }
    else if (input->JoystickDirection[Any] == DOWN)
    {
        if (ButtonSelected < maxButtonScreenIndex)  ButtonSelected++;
        else  ButtonSelected = 0;

        if (maxButtonScreenIndex != 0)
        {
            input->DelayAllUserInput = 10;
            screens->ScreenMustBeDrawn = true;
            audio->PlayDigitalSoundFX(0, 0);
        }
    }
}

//-------------------------------------------------------------------------------------------------
void Interface::DestroyAllButtons(void)
{
    for (int index = 0; index < 10; index++)
    {
        GUIs[index].OnScreenIndex = -1;
    }
}

//-------------------------------------------------------------------------------------------------
void Interface::CreateArrowSet(int leftArrowScreenIndex, int rightArrowScreenIndex, GLfloat positionY)
{
    GUIs[11+leftArrowScreenIndex].OnScreenIndex = leftArrowScreenIndex;
    GUIs[11+leftArrowScreenIndex].ScreenX = 30;
    GUIs[11+leftArrowScreenIndex].ScreenY = positionY+22;
    GUIs[11+leftArrowScreenIndex].ScaleX = 1.0f;
    GUIs[11+leftArrowScreenIndex].ScaleY = 1.0f;
    GUIs[11+leftArrowScreenIndex].RotationDegree = 0.0f;
    GUIs[11+leftArrowScreenIndex].Transparency = 1.0f;
    GUIs[11+leftArrowScreenIndex].RedHue = 1.0f;
    GUIs[11+leftArrowScreenIndex].GreenHue = 1.0f;
    GUIs[11+leftArrowScreenIndex].BlueHue = 1.0f;
    GUIs[11+leftArrowScreenIndex].Smooth = true;
    GUIs[11+leftArrowScreenIndex].FlipX = false;
    GUIs[11+leftArrowScreenIndex].FlipY = false;
    GUIs[11+leftArrowScreenIndex].OffsetX = 10;
    GUIs[11+leftArrowScreenIndex].OffsetY = 13;
    GUIs[11+leftArrowScreenIndex].AnimationTimer = -1;

    GUIs[11+rightArrowScreenIndex].OnScreenIndex = rightArrowScreenIndex;
    GUIs[11+rightArrowScreenIndex].ScreenX = 610;
    GUIs[11+rightArrowScreenIndex].ScreenY = positionY+22;
    GUIs[11+rightArrowScreenIndex].ScaleX = 1.0f;
    GUIs[11+rightArrowScreenIndex].ScaleY = 1.0f;
    GUIs[11+rightArrowScreenIndex].RotationDegree = 0.0f;
    GUIs[11+rightArrowScreenIndex].Transparency = 1.0f;
    GUIs[11+rightArrowScreenIndex].RedHue = 1.0f;
    GUIs[11+rightArrowScreenIndex].GreenHue = 1.0f;
    GUIs[11+rightArrowScreenIndex].BlueHue = 1.0f;
    GUIs[11+rightArrowScreenIndex].Smooth = true;
    GUIs[11+rightArrowScreenIndex].FlipX = false;
    GUIs[11+rightArrowScreenIndex].FlipY = false;
    GUIs[11+rightArrowScreenIndex].OffsetX = 10;
    GUIs[11+rightArrowScreenIndex].OffsetY = 13;
    GUIs[11+rightArrowScreenIndex].AnimationTimer = -1;
}

//-------------------------------------------------------------------------------------------------
void Interface::DisplayAllArrowSetsOntoScreenBuffer(void)
{
    glPushMatrix();

    glBindTexture(GL_TEXTURE_2D, GUIs[10].OGL_Texture);
    glEnable(GL_TEXTURE_2D);

    GUIs[10].ScreenY = GUIs[11+ArrowSetSelected].ScreenY;

    glTranslatef(GUIs[10].ScreenX, GUIs[10].ScreenY, 0.0f);

    if (GUIs[10].AnimationTimer > 0)  { GUIs[10].ScaleX = 0.85f; GUIs[10].ScaleY = 0.85f; }
    else  { GUIs[10].ScaleX = 1.0f; GUIs[10].ScaleY = 1.0f; }
    glScalef(GUIs[10].ScaleX, GUIs[10].ScaleY, 1.0f);

    glRotatef(GUIs[10].RotationDegree, 0.0f, 0.0f, 1.0f);

    GUIs[10].RedHue = 0.0f;
    GUIs[10].GreenHue = 0.0f;
    GUIs[10].BlueHue = 0.0f;
    GUIs[10].Transparency = 0.5f;
    glColor4f(GUIs[10].RedHue, GUIs[10].GreenHue, GUIs[10].BlueHue
              , GUIs[10].Transparency);

    if (GUIs[10].Smooth == false)
    {

    }
    else
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    }

    GLfloat zeroWidth = 0, oneWidth = 0;
    GLfloat zeroHeight = 0, oneHeight = 0;

    if (GUIs[10].FlipX == false)
    {
        zeroWidth = -(GUIs[10].Width / 2); oneWidth = (GUIs[10].Width / 2);
    }
    else if (GUIs[10].FlipX == true)
    {
        zeroWidth = (GUIs[10].Width); oneWidth = 0;
    }

    if (GUIs[10].FlipY == false)
    {
        zeroHeight = -(GUIs[10].Height / 2); oneHeight = (GUIs[10].Height / 2);
    }
    else if (GUIs[10].FlipY == true)
    {
        zeroHeight = (GUIs[10].Height); oneHeight = 0;
    }

    int numvertices = 4;
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );

    GLfloat *vertices = new float[3*numvertices];
    GLfloat *textcoord = new float[2*numvertices];

    textcoord[0] = 0;
    textcoord[1] = 0;
    textcoord[2] = 1;
    textcoord[3] = 0;
    textcoord[4] = 1;
    textcoord[5] = 1;
    textcoord[6] = 0;
    textcoord[7] = 1;

    vertices[0] = zeroWidth;
    vertices[1] = zeroHeight;
    vertices[2] = 0;
    vertices[3] = oneWidth;
    vertices[4] = zeroHeight;
    vertices[5] = 0;
    vertices[6] = oneWidth;
    vertices[7] = oneHeight;
    vertices[8] = 0;
    vertices[9] = zeroWidth;
    vertices[10] = oneHeight;
    vertices[11] = 0;

    glVertexPointer( 3, GL_FLOAT, 0, vertices );
    glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

    glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
    delete textcoord;
    delete vertices;

    glPopMatrix();

    for (int index = 11; index < NumberOfGUIs; index++)
    {
        if (GUIs[index].OnScreenIndex > -1)
        {
            glPushMatrix();

            glBindTexture(GL_TEXTURE_2D, GUIs[index].OGL_Texture);
            glEnable(GL_TEXTURE_2D);

            if (GUIs[index].AnimationTimer > 0)  { GUIs[index].ScaleX = 0.85f; GUIs[index].ScaleY = 0.85f; }
            else  { GUIs[index].ScaleX = 1.0f; GUIs[index].ScaleY = 1.0f; }

            glTranslatef(GUIs[index].ScreenX, GUIs[index].ScreenY, 0.0f);

            glScalef(GUIs[index].ScaleX, GUIs[index].ScaleY, 1.0f);

            glRotatef(GUIs[index].RotationDegree, 0.0f, 0.0f, 1.0f);

            glColor4f(GUIs[index].RedHue, GUIs[index].GreenHue, GUIs[index].BlueHue
                      , GUIs[index].Transparency);

            if (GUIs[index].Smooth == false)
            {

            }
            else
            {
                glEnable(GL_BLEND);
                glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
            }

            GLfloat zeroWidth = 0, oneWidth = 0;
            GLfloat zeroHeight = 0, oneHeight = 0;

            if (GUIs[index].FlipX == false)
            {
                zeroWidth = -(GUIs[index].Width / 2); oneWidth = (GUIs[index].Width / 2);
            }
            else if (GUIs[index].FlipX == true)
            {
                zeroWidth = (GUIs[index].Width); oneWidth = 0;
            }

            if (GUIs[index].FlipY == false)
            {
                zeroHeight = -(GUIs[index].Height / 2); oneHeight = (GUIs[index].Height / 2);
            }
            else if (GUIs[index].FlipY == true)
            {
                zeroHeight = (GUIs[index].Height); oneHeight = 0;
            }

            int numvertices = 4;
            glEnableClientState( GL_VERTEX_ARRAY );
            glEnableClientState( GL_TEXTURE_COORD_ARRAY );

            GLfloat *vertices = new float[3*numvertices];
            GLfloat *textcoord = new float[2*numvertices];

            textcoord[0] = 0;
            textcoord[1] = 0;
            textcoord[2] = 1;
            textcoord[3] = 0;
            textcoord[4] = 1;
            textcoord[5] = 1;
            textcoord[6] = 0;
            textcoord[7] = 1;

            vertices[0] = zeroWidth;
            vertices[1] = zeroHeight;
            vertices[2] = 0;
            vertices[3] = oneWidth;
            vertices[4] = zeroHeight;
            vertices[5] = 0;
            vertices[6] = oneWidth;
            vertices[7] = oneHeight;
            vertices[8] = 0;
            vertices[9] = zeroWidth;
            vertices[10] = oneHeight;
            vertices[11] = 0;

            glVertexPointer( 3, GL_FLOAT, 0, vertices );
            glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

            glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
            delete textcoord;
            delete vertices;

            glPopMatrix();
        }
    }

    screens->ScreenIsDirty = true;
}

//-------------------------------------------------------------------------------------------------
void Interface::ProcessAllArrowSetsOnScreen(void)
{
int minArrowSetScreenIndex = 0;
int maxArrowSetScreenIndex = 0;

    for (int index = 11; index < NumberOfGUIs; index++)
    {
        if (GUIs[index].OnScreenIndex > -1)
        {
            if (GUIs[index].AnimationTimer == 10 || GUIs[index].AnimationTimer == 0)
                screens->ScreenMustBeDrawn = true;

            if (GUIs[index].AnimationTimer > -1)  GUIs[index].AnimationTimer--;

            if (input->MouseButtonPressed[0] == true)
            {
                if (   (  input->MouseX > ( GUIs[index].ScreenX - ((GUIs[index].Width / 2) - GUIs[index].OffsetX) )
                   && input->MouseX < ( GUIs[index].ScreenX + ((GUIs[index].Width / 2) - GUIs[index].OffsetX) )  )
                   && (  input->MouseY > ( GUIs[index].ScreenY - ((GUIs[index].Height / 2) - GUIs[index].OffsetY) )
                   && input->MouseY < ( GUIs[index].ScreenY + ((GUIs[index].Height / 2) - GUIs[index].OffsetY) )  )   )
                {
                    ArrowSetSelected = GUIs[index].OnScreenIndex;
                    if (ArrowSetSelected % 2 != 0)  ArrowSetSelected--;

                    GUIs[index].AnimationTimer = 10;
                    ArrowSelected = GUIs[index].OnScreenIndex;
                    input->DelayAllUserInput = 15;
                    audio->PlayDigitalSoundFX(0, 0);
                }
            }
            else if (input->JoystickDirection[Any] == LEFT && ArrowSetSelected == index-11)
            {
                GUIs[index].AnimationTimer = 10;
                ArrowSelected = GUIs[index].OnScreenIndex;
                input->DelayAllUserInput = 15;
                audio->PlayDigitalSoundFX(0, 0);
            }
            else if (input->JoystickDirection[Any] == RIGHT && ArrowSetSelected == index-12)
            {
                GUIs[index].AnimationTimer = 10;
                ArrowSelected = GUIs[index].OnScreenIndex;
                input->DelayAllUserInput = 15;
                audio->PlayDigitalSoundFX(0, 0);
            }
        }
    }

    if (input->JoystickDirection[Any] != CENTER)
    {
        minArrowSetScreenIndex = 0;
        for (int index = NumberOfGUIs-1; index > 10; index-=2)
        {
            if (GUIs[index].OnScreenIndex > -1)
            {
                minArrowSetScreenIndex = GUIs[index].OnScreenIndex;
            }
        }

        maxArrowSetScreenIndex = 0;
        for (int index = 11; index < NumberOfGUIs; index+=2)
        {
            if (GUIs[index].OnScreenIndex > -1)
            {
                maxArrowSetScreenIndex = GUIs[index].OnScreenIndex;
            }
        }
    }

    if (input->JoystickDirection[Any] == UP)
    {
        if (ArrowSetSelected > 0 && ArrowSetSelected > minArrowSetScreenIndex)  ArrowSetSelected-=2;
        else
        {
            for (int index = 11; index < NumberOfGUIs; index+=2)
            {
                if (GUIs[index].OnScreenIndex > -1)
                {
                    ArrowSetSelected = GUIs[index].OnScreenIndex;
                }
            }
        }

        if (maxArrowSetScreenIndex != 0)
        {
            input->DelayAllUserInput = 10;
            screens->ScreenMustBeDrawn = true;
            audio->PlayDigitalSoundFX(0, 0);
        }
    }
    else if (input->JoystickDirection[Any] == DOWN)
    {
        if (ArrowSetSelected < maxArrowSetScreenIndex)  ArrowSetSelected+=2;
        else  ArrowSetSelected = 0;

        if (maxArrowSetScreenIndex != 0)
        {
            input->DelayAllUserInput = 10;
            screens->ScreenMustBeDrawn = true;
            audio->PlayDigitalSoundFX(0, 0);
        }
    }
}

//-------------------------------------------------------------------------------------------------
void Interface::DestroyAllArrowSets(void)
{
    for (int index = 11; index < NumberOfGUIs; index++)
    {
        GUIs[index].OnScreenIndex = -1;
    }
}

//-------------------------------------------------------------------------------------------------
void Interface::UnloadGUIFromMemory(void)
{
    for (int index = 0; index < NumberOfGUIs; index++)
    {
        if (GUIs[index].OGL_Texture)  glDeleteTextures( 1, &GUIs[index].OGL_Texture );
    }
    printf("Unloaded all button/GUI images from memory\n");

}
