#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "logic.h"

#include "audio.h"
#include "screens.h"
#include "game.h"
#include "input.h"
#include "visuals.h"

extern Audio* audio;
extern Screens* screens;
extern Game* game;
extern Input* input;
extern Visuals* visuals;

//-------------------------------------------------------------------------------------------------
Logic::Logic(void)
{
    GameMode = OriginalMode;

    SelectedBackground = 0;
    SelectedMusicTrack = 1;

    DisplayNextPiece = true;
    DisplayDropShadow = true;

    PressingUPAction = Rotate;

    for (int player = 0; player < NumberOfPlayers; player++)
    {
        PlayerData[player].UPActionTaken = false;
        PlayerData[player].RotateDirection = 0;

        PlayerData[player].Score = 0;
        PlayerData[player].DropBonus = 0;
        PlayerData[player].Level = 0;
        PlayerData[player].Lines = 0;
    }

    TileSet = 1;

    GameWasJustPlayed = false;
    CrisisWon = false;

    ThinkRussianTimer = 0;

    TotalCPUPlayerLines = 0;
    NumberofCPUGames = 0;

    MaxRotationArray[0] = 0;
    MaxRotationArray[1] = 2;
    MaxRotationArray[2] = 2;
    MaxRotationArray[3] = 4;
    MaxRotationArray[4] = 4;
    MaxRotationArray[5] = 4;
    MaxRotationArray[6] = 1;
    MaxRotationArray[7] = 2;
}

//-------------------------------------------------------------------------------------------------
Logic::~Logic(void)
{

}

//-------------------------------------------------------------------------------------------------

/*  __________________
    \___  __  __  ___/
       / / / / / /                  TM
  T+B / / / / / / 100% PerfecT+EnginE   Version 9.0  [MultI+PlayeR A.I.]
   __/ /_/ /_/ /__
 /________________\

*/

//-------------------------------------------------------------------------------------------------
void Logic::InitializePieceData(void)
{
Uint8 piece, rotation, box;

    for (piece = 0; piece < 8; piece++)
        for (rotation = 0; rotation < 5; rotation++)
            for (box = 0; box < 17; box++)
                PieceData [piece] [rotation] [box] = 0;

    //RED "S Piece"...
    PieceData [1] [1] [10] = 1; // 01 02 03 04
    PieceData [1] [1] [11] = 1; // 05 06 07 08
    PieceData [1] [1] [13] = 1; // 09 [] [] 12
    PieceData [1] [1] [14] = 1; // [] [] 15 16

    PieceData [1] [2] [ 5] = 1;
    PieceData [1] [2] [ 9] = 1;
    PieceData [1] [2] [10] = 1;
    PieceData [1] [2] [14] = 1;

    PieceData [1] [3] [10] = 1;
    PieceData [1] [3] [11] = 1;
    PieceData [1] [3] [13] = 1;
    PieceData [1] [3] [14] = 1;

    PieceData [1] [4] [ 5] = 1;
    PieceData [1] [4] [ 9] = 1;
    PieceData [1] [4] [10] = 1;
    PieceData [1] [4] [14] = 1;

    //ORANGE "Z Piece"...
    PieceData [2] [1] [ 9] = 1;
    PieceData [2] [1] [10] = 1;
    PieceData [2] [1] [14] = 1;
    PieceData [2] [1] [15] = 1;

    PieceData [2] [2] [ 6] = 1;
    PieceData [2] [2] [ 9] = 1;
    PieceData [2] [2] [10] = 1;
    PieceData [2] [2] [13] = 1;

    PieceData [2] [3] [ 9] = 1;
    PieceData [2] [3] [10] = 1;
    PieceData [2] [3] [14] = 1;
    PieceData [2] [3] [15] = 1;

    PieceData [2] [4] [ 6] = 1;
    PieceData [2] [4] [ 9] = 1;
    PieceData [2] [4] [10] = 1;
    PieceData [2] [4] [13] = 1;

    //AQUA "T Piece"...
    PieceData [3] [1] [ 9] = 1;
    PieceData [3] [1] [10] = 1;
    PieceData [3] [1] [11] = 1;
    PieceData [3] [1] [14] = 1;

    PieceData [3] [2] [ 6] = 1;
    PieceData [3] [2] [ 9] = 1;
    PieceData [3] [2] [10] = 1;
    PieceData [3] [2] [14] = 1;

    PieceData [3] [3] [ 6] = 1;
    PieceData [3] [3] [ 9] = 1;
    PieceData [3] [3] [10] = 1;
    PieceData [3] [3] [11] = 1;

    PieceData [3] [4] [ 6] = 1;
    PieceData [3] [4] [10] = 1;
    PieceData [3] [4] [11] = 1;
    PieceData [3] [4] [14] = 1;

    //YELLOW "L Piece"...
    PieceData [4] [1] [ 9] = 1;
    PieceData [4] [1] [10] = 1;
    PieceData [4] [1] [11] = 1;
    PieceData [4] [1] [13] = 1;

    PieceData [4] [2] [ 5] = 1;
    PieceData [4] [2] [ 6] = 1;
    PieceData [4] [2] [10] = 1;
    PieceData [4] [2] [14] = 1;

    PieceData [4] [3] [ 7] = 1;
    PieceData [4] [3] [ 9] = 1;
    PieceData [4] [3] [10] = 1;
    PieceData [4] [3] [11] = 1;

    PieceData [4] [4] [ 6] = 1;
    PieceData [4] [4] [10] = 1;
    PieceData [4] [4] [14] = 1;
    PieceData [4] [4] [15] = 1;

    //GREEN "Backwards L Piece"...
    PieceData [5] [1] [ 9] = 1;
    PieceData [5] [1] [10] = 1;
    PieceData [5] [1] [11] = 1;
    PieceData [5] [1] [15] = 1;

    PieceData [5] [2] [ 6] = 1;
    PieceData [5] [2] [10] = 1;
    PieceData [5] [2] [13] = 1;
    PieceData [5] [2] [14] = 1;

    PieceData [5] [3] [ 5] = 1;
    PieceData [5] [3] [ 9] = 1;
    PieceData [5] [3] [10] = 1;
    PieceData [5] [3] [11] = 1;

    PieceData [5] [4] [ 6] = 1;
    PieceData [5] [4] [ 7] = 1;
    PieceData [5] [4] [10] = 1;
    PieceData [5] [4] [14] = 1;

    //BLUE "Box Piece"...
    PieceData [6] [1] [10] = 1;
    PieceData [6] [1] [11] = 1;
    PieceData [6] [1] [14] = 1;
    PieceData [6] [1] [15] = 1;

    PieceData [6] [2] [10] = 1;
    PieceData [6] [2] [11] = 1;
    PieceData [6] [2] [14] = 1;
    PieceData [6] [2] [15] = 1;

    PieceData [6] [3] [10] = 1;
    PieceData [6] [3] [11] = 1;
    PieceData [6] [3] [14] = 1;
    PieceData [6] [3] [15] = 1;

    PieceData [6] [4] [10] = 1;
    PieceData [6] [4] [11] = 1;
    PieceData [6] [4] [14] = 1;
    PieceData [6] [4] [15] = 1;

    //PURPLE "Line Piece"...
    PieceData [7] [1] [ 9] = 1;
    PieceData [7] [1] [10] = 1;
    PieceData [7] [1] [11] = 1;
    PieceData [7] [1] [12] = 1;

    PieceData [7] [2] [ 2] = 1;
    PieceData [7] [2] [ 6] = 1;
    PieceData [7] [2] [10] = 1;
    PieceData [7] [2] [14] = 1;

    PieceData [7] [3] [ 9] = 1;
    PieceData [7] [3] [10] = 1;
    PieceData [7] [3] [11] = 1;
    PieceData [7] [3] [12] = 1;

    PieceData [7] [4] [ 2] = 1;
    PieceData [7] [4] [ 6] = 1;
    PieceData [7] [4] [10] = 1;
    PieceData [7] [4] [14] = 1;
}

//-------------------------------------------------------------------------------------------------
void Logic::ClearPlayfieldsWithCollisionDetection(void)
{
	for (int player = 0; player < NumberOfPlayers; player++)
	{
		for (int y = 0; y < 26; y++)
			for (int x = 0; x < 15; x++)
				PlayerData[player].Playfield[x][y] = 255; // Collision detection value

        for (int y = 2; y < 5; y++)
            for (int x = 5; x < 9; x++)
                PlayerData[player].Playfield[x][y] = 0;

        for (int y = 5; y < 24; y++)
            for (int x = 2; x < 12; x++)
                PlayerData[player].Playfield[x][y] = 0;
	}
}

//-------------------------------------------------------------------------------------------------
Uint8 Logic::GetRandomPiece(void)
{
Uint32 random = rand()%7;

	if      (random < 1)  return(1);
    else if (random < 2)  return(2);
    else if (random < 3)  return(3);
    else if (random < 4)  return(4);
    else if (random < 5)  return(5);
    else if (random < 6)  return(6);
	else  return(7);
/*
Uint32 random = rand()%100;

	if      (random < 20)  return(1);
    else if (random < 40)  return(2);
    else if (random < 54)  return(3);
    else if (random < 67)  return(4);
    else if (random < 80)  return(5);
    else if (random < 92)  return(6);
	else  return(7);
*/
}

//-------------------------------------------------------------------------------------------------
int Logic::PieceCollision(void)
{
Sint8 x, y;
Uint8 box = 1;
int returnValue = CollisionNotTrue;

    for (y = 0; y < 4; y++)
        for (x = 0; x < 4; x++)
        {
            if ( (PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX + x ][ PlayerData[Player].PiecePlayfieldY + y ] > 1)
                &&(PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [box] > 0) )
                    returnValue = CollisionWithPlayfield;

            box++;
        }

    return(returnValue);
}

//-------------------------------------------------------------------------------------------------
int Logic::PieceCollisionDown(void)
{
Sint8 x, y;
Uint8 box = 1;
int returnValue = CollisionNotTrue;

    for (y = 1; y < 5; y++)
        for (x = 0; x < 4; x++)
        {
            if ( (PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX + x ][ PlayerData[Player].PiecePlayfieldY + y ] > 1)
                &&(PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [box] > 0) )
                    returnValue = CollisionWithPlayfield;

            box++;
        }

    return(returnValue);
}

//-------------------------------------------------------------------------------------------------
void Logic::AddPieceToPlayfieldMemory(int TempOrCurrentOrNextOrDropShadow)
{
    if ( (DisplayDropShadow == false)&&(TempOrCurrentOrNextOrDropShadow == DropShadow) )
        return;

	Uint8 TEMP_Piece = PlayerData[Player].Piece;
	Uint8 TEMP_PieceRotation = PlayerData[Player].PieceRotation;
	Uint8 TEMP_PiecePlayfieldX = PlayerData[Player].PiecePlayfieldX;
	Uint8 TEMP_PiecePlayfieldY = PlayerData[Player].PiecePlayfieldY;

	int value = PlayerData[Player].Piece+10;

    if (TempOrCurrentOrNextOrDropShadow == Next)
	{
		PlayerData[Player].Piece = PlayerData[Player].NextPiece;
		value = PlayerData[Player].NextPiece+10;
		PlayerData[Player].PieceRotation = 1;

        if      (Player == 0)  PlayerData[Player].PiecePlayfieldX = 5;
        else if (Player == 1)  PlayerData[Player].PiecePlayfieldX = 5;
        else if (Player == 2)  PlayerData[Player].PiecePlayfieldX = 5;

		PlayerData[Player].PiecePlayfieldY = 0;
	}
	else if (TempOrCurrentOrNextOrDropShadow == DropShadow)
	{
		for (int y = PlayerData[Player].PiecePlayfieldY; y < 23; y++)
		{
			PlayerData[Player].PiecePlayfieldY = y;
			if (PieceCollision() != CollisionNotTrue)
			{
				if (y - TEMP_PiecePlayfieldY > 4)
				{
					value = 1;
					PlayerData[Player].PiecePlayfieldY = y-1;
					y = 100;
				}
				else
				{
					PlayerData[Player].Piece = TEMP_Piece;
					PlayerData[Player].PieceRotation = TEMP_PieceRotation;
					PlayerData[Player].PiecePlayfieldX = TEMP_PiecePlayfieldX;
					PlayerData[Player].PiecePlayfieldY = TEMP_PiecePlayfieldY;
					return;
				}
			}
		}
	}

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 1] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 2] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 3] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 4] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY ] = value;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 5] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+1 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 6] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+1 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 7] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+1 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 8] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+1 ] = value;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 9] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+2 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [10] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+2 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [11] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+2 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [12] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+2 ] = value;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [13] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+3 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [14] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+3 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [15] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+3 ] = value;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [16] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+3 ] = value;

	PlayerData[Player].Piece = TEMP_Piece;
	PlayerData[Player].PieceRotation = TEMP_PieceRotation;
	PlayerData[Player].PiecePlayfieldX = TEMP_PiecePlayfieldX;
	PlayerData[Player].PiecePlayfieldY = TEMP_PiecePlayfieldY;
}

//-------------------------------------------------------------------------------------------------
void Logic::DeletePieceFromPlayfieldMemory(int CurrentOrDropShadow)
{
    if ( (DisplayDropShadow == false)&&(CurrentOrDropShadow == DropShadow) )
        return;

    if (PlayerData[Player].PlayerStatus == FlashingCompletedLines || PlayerData[Player].PlayerStatus == ClearingCompletedLines)
        return;

	Uint8 TEMP_Piece = PlayerData[Player].Piece;
	Uint8 TEMP_PieceRotation = PlayerData[Player].PieceRotation;
	Uint8 TEMP_PiecePlayfieldX = PlayerData[Player].PiecePlayfieldX;
	Uint8 TEMP_PiecePlayfieldY = PlayerData[Player].PiecePlayfieldY;

	if (CurrentOrDropShadow == DropShadow)
	{
		for (int y = PlayerData[Player].PiecePlayfieldY; y < 23; y++)
		{
			PlayerData[Player].PiecePlayfieldY = y;
			if (PieceCollision() != CollisionNotTrue)
			{
				if (y - TEMP_PiecePlayfieldY > 4)
				{
					PlayerData[Player].PiecePlayfieldY = y-1;
					y = 100;
				}
				else
				{
					PlayerData[Player].Piece = TEMP_Piece;
					PlayerData[Player].PieceRotation = TEMP_PieceRotation;
					PlayerData[Player].PiecePlayfieldX = TEMP_PiecePlayfieldX;
					PlayerData[Player].PiecePlayfieldY = TEMP_PiecePlayfieldY;
					return;
				}
			}
		}
	}

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 1] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 2] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 3] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 4] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY ] = 0;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 5] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 6] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 7] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 8] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 9] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [10] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [11] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [12] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;

	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [13] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [14] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [15] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
	if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [16] == 1)
        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;

	PlayerData[Player].Piece = TEMP_Piece;
	PlayerData[Player].PieceRotation = TEMP_PieceRotation;
	PlayerData[Player].PiecePlayfieldX = TEMP_PiecePlayfieldX;
	PlayerData[Player].PiecePlayfieldY = TEMP_PiecePlayfieldY;
}

//-------------------------------------------------------------------------------------------------
void Logic::SetupNewPiece(void)
{
	PlayerData[Player].Piece = PlayerData[Player].NextPiece;
	PlayerData[Player].PieceMovementDelay = 0;
	PlayerData[Player].PieceRotation = 1;

    if (Player == 0)
        PlayerData[0].PiecePlayfieldX = 5;
    else if (Player == 1)
        PlayerData[1].PiecePlayfieldX = 5;
    else if (Player == 2)
        PlayerData[2].PiecePlayfieldX = 5;

	PlayerData[Player].PiecePlayfieldY = 0;

	PlayerData[Player].NextPiece = GetRandomPiece();

	PlayerData[Player].PlayerStatus = NewPieceDropping;

	PlayerData[Player].PieceDropTimer = 0;

	PlayerData[Player].PieceRotated1 = false;
	PlayerData[Player].PieceRotated2 = false;

	PlayerData[Player].FlashCompletedLinesTimer = 0;
	PlayerData[Player].ClearCompletedLinesTimer = 0;

	PlayerData[Player].DropBonus = 0;

    for (int x = 0; x < 12; x++)
    {
        for (int rotation = 1; rotation < 5; rotation++)
        {
            PlayerData[Player].NewTrappedHoles[x][rotation] = 0;

            PlayerData[Player].MoveCompletesLines[x][rotation] = 0;

            PlayerData[Player].NewPlayfieldHeight[x][rotation] = 0;
            PlayerData[Player].FallenPieceHeight[x][rotation] = 0;

            PlayerData[Player].NewNumberOfOneBlockCavernHoles[x][rotation] = 0;
        }
    }

    PlayerData[Player].BestMoveX = -1;
    PlayerData[Player].BestRotation = -1;
    PlayerData[Player].BestMoveCalculated = false;
    PlayerData[Player].MovedToBestMove = false;

    PlayerData[Player].OldTrappedHoles = 0;

    GameDisplayChanged = true;
}

//-------------------------------------------------------------------------------------------------
void Logic::CheckForCompletedLines(void)
{
int numberOfCompletedLines = 0;

    AddPieceToPlayfieldMemory(Current);

	for (int y = 5; y < 24; y++)
	{
		int boxTotal = 0;

		for (int x = 2; x < 12; x++)
		{
			if ( (PlayerData[Player].Playfield[x][y] > 9)&&(PlayerData[Player].Playfield[x][y] < 20) )
				boxTotal++;
		}

		if (boxTotal == 10)
            numberOfCompletedLines++;
	}

	if (numberOfCompletedLines > 0)
	{
		if (numberOfCompletedLines == 1)
		{
			PlayerData[Player].Score += (40 * (PlayerData[Player].Level+1) );
		}
		else if (numberOfCompletedLines == 2)
		{
			PlayerData[Player].Score += (100 * (PlayerData[Player].Level+1) );
		}
		else if (numberOfCompletedLines == 3)
		{
			PlayerData[Player].Score += (300 * (PlayerData[Player].Level+1) );
		}
		else if (numberOfCompletedLines == 4)
		{
			PlayerData[Player].Score += (1200 * (PlayerData[Player].Level+1) );
			audio->PlayDigitalSoundFX(7, 0);
		}

		PlayerData[Player].PlayerStatus = FlashingCompletedLines;
		PlayerData[Player].FlashCompletedLinesTimer = 1;
	}
	else  SetupNewPiece();
}

//-------------------------------------------------------------------------------------------------
void Logic::MovePieceDown(bool Force)
{
    if (Force == false)
    {
        if (PlayerData[Player].PlayerInput == CPU)
        {
            if (game->CPUPlayerEnabled == 1)
            {
                PlayerData[Player].CPUFrame++;
                if (PlayerData[Player].CPUFrame < 3)
                {
                    return;
                }
                else  PlayerData[Player].CPUFrame = 0;
            }
            else if (game->CPUPlayerEnabled == 2)
            {
                PlayerData[Player].CPUFrame++;
                if (PlayerData[Player].CPUFrame < 2)
                {
                    return;
                }
                else  PlayerData[Player].CPUFrame = 0;
            }
        }
    }

    DeletePieceFromPlayfieldMemory(Current);

	PlayerData[Player].PiecePlayfieldY++;

	if (PieceCollision() == CollisionWithPlayfield)
	{
        if (PlayersCanJoin == true)
        {
            if (PlayerData[0].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[0].PlayerInput = CPU;
                PlayerData[0].PlayerStatus = NewPieceDropping;
            }

            if (PlayerData[1].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[1].PlayerInput = CPU;
                PlayerData[1].PlayerStatus = NewPieceDropping;
            }

            if (PlayerData[2].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[2].PlayerInput = CPU;
                PlayerData[2].PlayerStatus = NewPieceDropping;
            }

            if ( game->CPUPlayerEnabled > 0
                 && (PlayerData[0].PlayerInput == CPU || PlayerData[2].PlayerInput == CPU) )  audio->PlayDigitalSoundFX(13, 0);

            PlayersCanJoin = false;
        }

		PlayerData[Player].PiecePlayfieldY--;

		if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(3, 0);

		PlayerData[Player].Score += PlayerData[Player].DropBonus;

		PlayerData[Player].DropBonus = 0;
		PlayerData[Player].PieceDropTimer = 0;

		if (PlayerData[Player].PlayerStatus == NewPieceDropping)
        {
            PlayerData[Player].PlayerStatus = GameOver;
            audio->PlayDigitalSoundFX(11, 0);
        }
		else  CheckForCompletedLines();
	}
	else
	{
    	GameDisplayChanged = true;
	}
}

//-------------------------------------------------------------------------------------------------
void Logic::MovePieceDownFast(void)
{
	while (PieceCollision() == CollisionNotTrue)
    {
        PlayerData[Player].PiecePlayfieldY++;
        PlayerData[Player].DropBonus++;
    }

	if (PieceCollision() == CollisionWithPlayfield)
	{
        if (PlayersCanJoin == true)
        {
            if (PlayerData[0].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[0].PlayerInput = CPU;
                PlayerData[0].PlayerStatus = NewPieceDropping;
            }

            if (PlayerData[1].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[1].PlayerInput = CPU;
                PlayerData[1].PlayerStatus = NewPieceDropping;
            }

            if (PlayerData[2].PlayerStatus == GameOver && game->CPUPlayerEnabled > 0)
            {
                PlayerData[2].PlayerInput = CPU;
                PlayerData[2].PlayerStatus = NewPieceDropping;
            }

            if ( game->CPUPlayerEnabled > 0
                 && (PlayerData[0].PlayerInput == CPU || PlayerData[2].PlayerInput == CPU) )  audio->PlayDigitalSoundFX(13, 0);

            PlayersCanJoin = false;
        }

		PlayerData[Player].PiecePlayfieldY--;

		if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(3, 0);

		PlayerData[Player].Score += PlayerData[Player].DropBonus;

		PlayerData[Player].DropBonus = 0;
		PlayerData[Player].PieceDropTimer = 0;

		if (PlayerData[Player].PlayerStatus == NewPieceDropping)
        {
            PlayerData[Player].PlayerStatus = GameOver;
            audio->PlayDigitalSoundFX(11, 0);
        }
		else  CheckForCompletedLines();
	}
	else
	{
		GameDisplayChanged = true;
	}
}

//-------------------------------------------------------------------------------------------------
bool Logic::RotatePieceCounterClockwise(void)
{
	if (PlayerData[Player].PieceRotation > 1)  PlayerData[Player].PieceRotation--;
	else  PlayerData[Player].PieceRotation = 4;

	if (PieceCollision() == CollisionNotTrue)
	{
		if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(5, 0);

		GameDisplayChanged = true;

        return(true);
	}
	else
	{
		if (PlayerData[Player].PieceRotation < 4)  PlayerData[Player].PieceRotation++;
		else  PlayerData[Player].PieceRotation = 1;

        if (PlayerData[Player].RotateDirection == 0)  PlayerData[Player].RotateDirection = 1;
        else  PlayerData[Player].RotateDirection = 0;
    }

    return(false);
}

//-------------------------------------------------------------------------------------------------
bool Logic::RotatePieceClockwise(void)
{
	if (PlayerData[Player].PieceRotation < 4)  PlayerData[Player].PieceRotation++;
	else  PlayerData[Player].PieceRotation = 1;

	if (PieceCollision() == CollisionNotTrue)
	{
		if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(5, 0);

		GameDisplayChanged = true;

        return(true);
	}
	else
	{
		if (PlayerData[Player].PieceRotation > 1)  PlayerData[Player].PieceRotation--;
		else  PlayerData[Player].PieceRotation = 4;

        if (PlayerData[Player].RotateDirection == 0)  PlayerData[Player].RotateDirection = 1;
        else  PlayerData[Player].RotateDirection = 0;
	}

    return(false);
}

//-------------------------------------------------------------------------------------------------
void Logic::MovePieceLeft(void)
{
    if (PlayerData[Player].PieceMovementDelay > -15)  PlayerData[Player].PieceMovementDelay--;
    if ( (PlayerData[Player].PieceMovementDelay == -1)
       ||(PlayerData[Player].PieceMovementDelay == -5)
       ||(PlayerData[Player].PieceMovementDelay == -9)
       ||(PlayerData[Player].PieceMovementDelay < -14) )
    {
        PlayerData[Player].PiecePlayfieldX--;

        if (PieceCollision() == CollisionNotTrue)
        {
            GameDisplayChanged = true;

            if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(2, 0);
        }
        else
        {
            PlayerData[Player].PiecePlayfieldX++;
        }
    }
}

//-------------------------------------------------------------------------------------------------
void Logic::MovePieceRight(void)
{
    if (PlayerData[Player].PieceMovementDelay < 15)  PlayerData[Player].PieceMovementDelay++;
    if ( (PlayerData[Player].PieceMovementDelay == 1)
       ||(PlayerData[Player].PieceMovementDelay == 5)
       ||(PlayerData[Player].PieceMovementDelay == 9)
       ||(PlayerData[Player].PieceMovementDelay > 14) )
    {
        PlayerData[Player].PiecePlayfieldX++;

        if (PieceCollision() == CollisionNotTrue)
        {
            GameDisplayChanged = true;

            if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(2, 0);
        }
        else
        {
            PlayerData[Player].PiecePlayfieldX--;
        }
    }
}

//-------------------------------------------------------------------------------------------------
void Logic::SetupForNewGame(void)
{
    PlayersCanJoin = true;
    GameWasJustPlayed = true;

    if (SelectedBackground == 1 && GameMode == CrisisMode)
    {
        audio->PlayDigitalSoundFX(9, 0);
        ThinkRussianTimer = 320;
    }
    else  ThinkRussianTimer = 1;

	InitializePieceData();
	ClearPlayfieldsWithCollisionDetection();

    PlayerData[0].PlayersPlayfieldScreenX = 100;
    PlayerData[0].PlayersPlayfieldScreenY = 255;

    PlayerData[1].PlayersPlayfieldScreenX = 320;
    PlayerData[1].PlayersPlayfieldScreenY = 255;

    PlayerData[2].PlayersPlayfieldScreenX = 540;
    PlayerData[2].PlayersPlayfieldScreenY = 255;

    PlayerData[0].PiecePlayfieldX = 5;
    PlayerData[0].PiecePlayfieldY = 0;

    PlayerData[1].PiecePlayfieldX = 5;
    PlayerData[1].PiecePlayfieldY = 0;

    PlayerData[2].PiecePlayfieldX = 5;
    PlayerData[2].PiecePlayfieldY = 0;

	for (int player = 0; player < NumberOfPlayers; player++)
	{
		PlayerData[player].Piece = GetRandomPiece();
		PlayerData[player].PieceMovementDelay = 0;
		PlayerData[player].PieceRotation = 1;

		Player = player;

		PlayerData[player].NextPiece = GetRandomPiece();

		PlayerData[player].PlayerStatus = NewPieceDropping;

		PlayerData[player].PieceDropTimer = 0;
		PlayerData[player].TimeToDropPiece = 47;

		PlayerData[player].PieceRotated1 = false;
		PlayerData[player].PieceRotated2 = false;

		PlayerData[player].FlashCompletedLinesTimer = 0;
		PlayerData[player].ClearCompletedLinesTimer = 0;

		if (game->Debug == 1) PlayerData[player].Score = 3210;
        else  PlayerData[player].Score = 0;

		PlayerData[player].DropBonus = 0;
		PlayerData[player].Level = 0;

		if (game->Debug == 1) PlayerData[player].Lines = 9;
        else  PlayerData[player].Lines = 0;

		for (int y = 0; y < 12; y++)
			for (int x = 0; x < 10; x++)
				PlayerData[player].AttackLines[x][y] = 0;

        if (GameMode == TwentyLineChallengeMode)  PlayerData[player].TwentyLineCounter = 20;

        for (int x = 0; x < 12; x++)
        {
            for (int rotation = 1; rotation < 5; rotation++)
            {
                PlayerData[Player].NewTrappedHoles[x][rotation] = 0;

                PlayerData[Player].MoveCompletesLines[x][rotation] = 0;

                PlayerData[Player].NewPlayfieldHeight[x][rotation] = 0;
                PlayerData[Player].FallenPieceHeight[x][rotation] = 0;

                PlayerData[Player].NewNumberOfOneBlockCavernHoles[x][rotation] = 0;
            }
        }

        PlayerData[Player].BestMoveX = -1;
        PlayerData[Player].BestRotation = -1;
        PlayerData[Player].BestMoveCalculated = false;
        PlayerData[Player].MovedToBestMove = false;

        PlayerData[Player].OldTrappedHoles = 0;

        PlayerData[Player].CPUFrame = 0;
	}

    if (PlayerData[0].PlayerInput == -1)  PlayerData[0].PlayerStatus = GameOver;
    if (PlayerData[2].PlayerInput == -1)  PlayerData[2].PlayerStatus = GameOver;

    if (PlayerData[1].PlayerInput == Keyboard)
    {
        if (input->JoystickDeviceOne != NULL)  PlayerData[0].PlayerInput = JoystickOne;
        else
        {
            PlayerData[0].PlayerInput = CPU;
            if (game->CPUPlayerEnabled != false)  PlayerData[0].PlayerStatus = NewPieceDropping;
            else  PlayerData[0].PlayerStatus = GameOver;
        }

    	if (input->JoystickDeviceTwo != NULL)  PlayerData[2].PlayerInput = JoystickTwo;
        else
        {
            PlayerData[2].PlayerInput = CPU;
            if (game->CPUPlayerEnabled != false)  PlayerData[2].PlayerStatus = NewPieceDropping;
            else  PlayerData[2].PlayerStatus = GameOver;
        }
    }
    else if (PlayerData[1].PlayerInput == JoystickOne)
    {
        PlayerData[0].PlayerInput = Keyboard;

    	if (input->JoystickDeviceTwo != NULL)  PlayerData[2].PlayerInput = JoystickTwo;
        else
        {
            PlayerData[2].PlayerInput = CPU;
            if (game->CPUPlayerEnabled != false)  PlayerData[2].PlayerStatus = NewPieceDropping;
            else  PlayerData[2].PlayerStatus = GameOver;
        }
    }
    if (PlayerData[1].PlayerInput == JoystickTwo)
    {
        PlayerData[0].PlayerInput = Keyboard;

        if (input->JoystickDeviceOne != NULL)  PlayerData[2].PlayerInput = JoystickOne;
        else
        {
            PlayerData[2].PlayerInput = CPU;
            if (game->CPUPlayerEnabled != false)  PlayerData[2].PlayerStatus = NewPieceDropping;
            else  PlayerData[2].PlayerStatus = GameOver;
        }
    }

	Player = 0;

	PAUSEgame = false;

	if (GameMode == TimeAttack30Mode)  TimeAttackTimer = 6000;
	else if (GameMode == TimeAttack60Mode)  TimeAttackTimer = 12000;
	else if (GameMode == TimeAttack120Mode)  TimeAttackTimer = 24000;
    else  TimeAttackTimer = 0;

    CrisisModeTimer = 0;
    Crisis7BGMPlayed = false;

    CrisisWon = false;
    CrisisModeOnePlayerLeftPlayfieldCleared = false;

    GameDisplayChanged = true;

    if (game->Debug == 2)
    {
        PlayerData[0].PlayerInput = CPU;
        PlayerData[0].PlayerStatus = NewPieceDropping;

        PlayerData[1].PlayerInput = CPU;
        PlayerData[1].PlayerStatus = NewPieceDropping;

        PlayerData[2].PlayerInput = CPU;
        PlayerData[2].PlayerStatus = NewPieceDropping;
    }

    if (game->Debug == 4) // Screenshot mode...
    {
        PlayerData[0].Score = 121094;
        PlayerData[0].Level = 43;
        PlayerData[0].Lines = 436;
        PlayerData[1].Score = 116043;
        PlayerData[1].Level = 39;
        PlayerData[1].Lines = 392;
        PlayerData[2].Score = 131092;
        PlayerData[2].Level = 42;
        PlayerData[2].Lines = 411;
    }
}

//-------------------------------------------------------------------------------------------------
void Logic::FlashCompletedLines(void)
{
int numberOfCompletedLines = 0;

	if (PlayerData[Player].FlashCompletedLinesTimer < 21)  PlayerData[Player].FlashCompletedLinesTimer++;

	for (int y = 5; y < 24; y++)
	{
		int boxTotal = 0;

		for (int x = 2; x < 12; x++)
		{
			if ( (PlayerData[Player].Playfield[x][y] > 9)&&(PlayerData[Player].Playfield[x][y] < 30) )
				boxTotal++;
		}

		if (boxTotal == 10)
		{
            numberOfCompletedLines++;

			if (PlayerData[Player].FlashCompletedLinesTimer % 2 == 0)
			{
				for (int xTwo = 2; xTwo < 12; xTwo++)
					PlayerData[Player].Playfield[xTwo][y] = PlayerData[Player].Playfield[xTwo][y] + 10;
			}
			else
			{
				for (int xTwo = 2; xTwo < 12; xTwo++)
					PlayerData[Player].Playfield[xTwo][y] = PlayerData[Player].Playfield[xTwo][y] - 10;
			}
		}
	}

	if (PlayerData[Player].FlashCompletedLinesTimer == 21)
	{
		PlayerData[Player].PlayerStatus = ClearingCompletedLines;

        if (CrisisModeOnePlayerLeftPlayfieldCleared == false)
        {
            for (int y = 5; y < 24; y++)
            {
                int boxTotal = 0;

                for (int x = 2; x < 12; x++)
                {
                    if ( (PlayerData[Player].Playfield[x][y] > 9)&&(PlayerData[Player].Playfield[x][y] < 30) )
                        boxTotal++;
                }

                if (boxTotal == 10 && numberOfCompletedLines > 0)
                {
                    for (int attackY = 1; attackY < 12; attackY++)
                        for (int attackX = 0; attackX < 10; attackX++)
                            PlayerData[Player].AttackLines[attackX][attackY-1] = PlayerData[Player].AttackLines[attackX][attackY];

                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 1] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 2] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 3] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 4] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY ] = 0;

                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 5] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 6] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 7] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 8] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+1 ] = 0;

                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [ 9] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [10] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [11] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [12] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+2 ] = 0;

                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [13] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [14] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+1 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [15] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+2 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;
                    if (PieceData [PlayerData[Player].Piece] [PlayerData[Player].PieceRotation] [16] == 1)
                        PlayerData[Player].Playfield[ PlayerData[Player].PiecePlayfieldX+3 ][ PlayerData[Player].PiecePlayfieldY+3 ] = 0;

                    int attackX = 0;
                    for (int xThree = 2; xThree < 12; xThree++)
                    {
                        PlayerData[Player].AttackLines[attackX][11] = PlayerData[Player].Playfield[xThree][y];
                        attackX++;
                    }
                }

                AddPieceToPlayfieldMemory(Current);
            }
        }
	}

	GameDisplayChanged = true;
}

//-------------------------------------------------------------------------------------------------
void Logic::ClearCompletedLines(void)
{
bool thereWasACompletedLine = false;

	for (int y = 5; y < 24; y++)
	{
		int boxTotal = 0;

		for (int x = 2; x < 12; x++)
		{
			if ( (PlayerData[Player].Playfield[x][y] > 9)&&(PlayerData[Player].Playfield[x][y] < 20) )
				boxTotal++;
		}

		if (boxTotal == 10)
		{
			thereWasACompletedLine = true;

			if (PlayerData[Player].ClearCompletedLinesTimer < 40)  PlayerData[Player].ClearCompletedLinesTimer++;

			if (PlayerData[Player].ClearCompletedLinesTimer % 10 == 0)
			{
				for (int yTwo = y; yTwo > 5; yTwo--)
		            for (int xTwo = 2; xTwo < 12; xTwo++)
						PlayerData[Player].Playfield[xTwo][yTwo] = PlayerData[Player].Playfield[xTwo][yTwo-1];

                for (int xTwo = 2; xTwo < 12; xTwo++)
					PlayerData[Player].Playfield[xTwo][5] = 0;

				PlayerData[Player].Lines++;

                if (game->Debug == 1 && GameMode == CrisisMode)
                {
                    if (PlayerData[Player].Level > 0)  PlayerData[Player].Lines = (PlayerData[Player].Level+1) * 10;
                    else PlayerData[Player].Lines = 10;
                }

                if (PlayerData[Player].TwentyLineCounter > 0)  PlayerData[Player].TwentyLineCounter--;

				if (PlayerData[Player].Lines % 10 == 0)
				{
                    if (GameMode == CrisisMode)
                    {
                        Uint8 playersAlive = 0;
                        if (PlayerData[0].PlayerStatus != GameOver)  playersAlive++;
                        if (PlayerData[1].PlayerStatus != GameOver)  playersAlive++;
                        if (PlayerData[2].PlayerStatus != GameOver)  playersAlive++;

                        if (PlayerData[Player].Level < 9 && playersAlive == 1)
                        {
                            PlayerData[Player].Level++;

                            if (PlayerData[Player].Level == 7 && Crisis7BGMPlayed == false)
                            {
                                audio->CurrentlySelectedMusicTrack = 2;
                                audio->PlayMusic(3, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                audio->PlayDigitalSoundFX(12, 0);
                                Crisis7BGMPlayed = true;
                            }

                            PlayerData[Player].TimeToDropPiece-=5;
                            audio->PlayDigitalSoundFX(8, 0);
                        }
                        else if (PlayerData[Player].Level > 8)
                        {
                            PlayerData[0].PlayerStatus = GameOver;
                            PlayerData[1].PlayerStatus = GameOver;
                            PlayerData[2].PlayerStatus = GameOver;

                            PlayerData[Player].Level++;

                            CrisisWon = true;
                        }
                    }
                    else
                    {
                        if (PlayerData[Player].Level < 10000)
                        {
                            PlayerData[Player].Level++;
                            if (PlayerData[Player].TimeToDropPiece > 3)  PlayerData[Player].TimeToDropPiece-=2;
                            audio->PlayDigitalSoundFX(8, 0);
                        }
                    }
				}

				audio->PlayDigitalSoundFX(6, 0);
				GameDisplayChanged = true;
			}
		}
	}

	if (thereWasACompletedLine == false)
	{
		SetupNewPiece();
		PlayerData[Player].PlayerStatus = NewPieceDropping;
	}
}

//-------------------------------------------------------------------------------------------------
void Logic::AddAnAttackLineToEnemiesPlayfield(void)
{
int TEMP_Player = Player;

    for (Player = 0; Player < NumberOfPlayers; Player++)
    {
        if (Player != TEMP_Player && PlayerData[Player].PlayerStatus != GameOver)
        {
            if (PlayerData[Player].PlayerStatus != FlashingCompletedLines && PlayerData[Player].PlayerStatus != ClearingCompletedLines)
            {
                for (int x = 2; x < 12; x++)
                {
                    if (PlayerData[Player].Playfield[x][5] > 10 && PlayerData[Player].Playfield[x][5] < 20)
                    {
                        PlayerData[Player].PlayerStatus = GameOver;
                        return;
                    }
                }

                if (PieceCollisionDown() == true)  MovePieceDown(true);

                if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(10, 0);

                for (int y = 5; y < 23; y++)
                {
                    for (int x = 2; x < 12; x++)
                    {
                        PlayerData[Player].Playfield[x][y] = PlayerData[Player].Playfield[x][y+1];
                    }
                }

                for (int x = 2; x < 12; x++)
                    PlayerData[Player].Playfield[x][23] = 0;

                int attackX = 0;
                for (int x = 2; x < 12; x++)
                {
                    PlayerData[Player].Playfield[x][23] = PlayerData[TEMP_Player].AttackLines[attackX][11];
                    attackX++;
                }
            }
            else  audio->PlayDigitalSoundFX(14,0);
        }
    }

    for (int y = 11; y > 0; y--)
        for (int x = 0; x < 10; x++)
            PlayerData[TEMP_Player].AttackLines[x][y] = PlayerData[TEMP_Player].AttackLines[x][y-1];

	GameDisplayChanged = true;
    Player = TEMP_Player;
}

//-------------------------------------------------------------------------------------------------
bool Logic::AddAnIncompleteLineToPlayfieldCrisisMode(void)
{
    if (PlayerData[Player].PlayerStatus == FlashingCompletedLines || PlayerData[Player].PlayerStatus == ClearingCompletedLines)
    {
        return(false);
    }

    if (ThinkRussianTimer == 0)  audio->PlayDigitalSoundFX(10, 0);

    for (int x = 2; x < 12; x++)
    {
        if (PlayerData[Player].Playfield[x][5] > 10 && PlayerData[Player].Playfield[x][5] < 20)
        {
            PlayerData[Player].PlayerStatus = GameOver;
            return(true);
        }
    }

    if (PieceCollisionDown() == true)  MovePieceDown(true);

    for (int y = 5; y < 23; y++)
	    for (int x = 2; x < 12; x++)
            PlayerData[Player].Playfield[x][y] = PlayerData[Player].Playfield[x][y+1];

    for (int x = 2; x < 12; x++)
        PlayerData[Player].Playfield[x][23] = 0;

    int boxTotal = 0;
    for (int x = 2; x < 12; x++)
    {
        Uint32 box = rand()%8;
        if (box > 0)  boxTotal++;

        if (boxTotal < 10)
        {
            if (box != 0)  PlayerData[Player].Playfield[x][23] = (int)box+10;
            else  PlayerData[Player].Playfield[x][23] = 0;
        }
        else  PlayerData[Player].Playfield[x][23] = 0;
    }

    GameDisplayChanged = true;
    return(true);
}

//-------------------------------------------------------------------------------------------------
void Logic::RunTetriGameEngine(void)
{
	if (input->KeyOnKeyboardPressedByUser == '_')
	{
		if (PAUSEgame == false)
        {
            PAUSEgame = true;
            Mix_PauseMusic();
        }
		else
        {
            PAUSEgame = false;
            Mix_ResumeMusic();
        }

		GameDisplayChanged = true;
		audio->PlayDigitalSoundFX(0, 0);

        GameDisplayChanged = true;
	}

    if (ThinkRussianTimer > 0)  ThinkRussianTimer--;
    if (ThinkRussianTimer == 1)
    {
        audio->PlayMusic(SelectedMusicTrack+2, 1);
        Mix_ResumeMusic();
    }

	if (PAUSEgame == false)
	{
		for (Player = 0; Player < NumberOfPlayers; Player++)
		{
            if (PlayerData[Player].PlayerStatus != GameOver)
            {
                if (game->Debug != 1 && game->Debug != 4)  PlayerData[Player].PieceDropTimer++;

                if (input->JoystickDirection[PlayerData[Player].PlayerInput] == DOWN)
                {
                    PlayerData[Player].PieceDropTimer = 1+PlayerData[Player].TimeToDropPiece;
                }
                else  PlayerData[Player].DropBonus = 0;

                if (PlayerData[Player].PlayerStatus == NewPieceDropping)
                {
                    PlayerData[Player].UPActionTaken = true;
                    PlayerData[Player].DropBonus = 0;

                    if (PlayerData[Player].PiecePlayfieldY < 5)  MovePieceDown(true);
                    else
                    {
                        if (DisplayNextPiece == true)  AddPieceToPlayfieldMemory(Next);
                        PlayerData[Player].PlayerStatus = PieceFalling;
                    }

                    GameDisplayChanged = true;
                }
                else if (PlayerData[Player].PlayerStatus == PieceFalling)
                {
                    if (GameMode == TwentyLineChallengeMode && PlayerData[Player].TwentyLineCounter == 0)
                    {
                        PlayerData[Player].PlayerStatus = GameOver;
                    }

                    if (PlayerData[Player].PieceDropTimer > PlayerData[Player].TimeToDropPiece)
                    {
                        MovePieceDown(false);
                    }

                    if (PlayerData[Player].PlayerInput == CPU)
                    {
                        ComputeComputerPlayerMove();
                    }
                    else
                    {
                        if (PressingUPAction == Rotate)
                        {
                            if (input->JoystickDirection[PlayerData[Player].PlayerInput] == UP)
                            {
                                if (PlayerData[Player].RotateDirection == 0)
                                {
                                    if (PlayerData[Player].UPActionTaken == false)  RotatePieceCounterClockwise();
                                    PlayerData[Player].UPActionTaken = true;
                                }
                                else if (PlayerData[Player].RotateDirection == 1)
                                {
                                    if (PlayerData[Player].UPActionTaken == false)  RotatePieceClockwise();
                                    PlayerData[Player].UPActionTaken = true;
                                }
                            }
                            else  PlayerData[Player].UPActionTaken = false;
                        }
                        else if (PressingUPAction == Fall)
                        {
                            if (input->JoystickDirection[PlayerData[Player].PlayerInput] == UP)
                            {
                                if (PlayerData[Player].UPActionTaken == false)
                                {
                                    MovePieceDownFast();
                                }

                                PlayerData[Player].UPActionTaken = true;
                            }
                            else  PlayerData[Player].UPActionTaken = false;
                        }

                        if (PlayerData[Player].PieceDropTimer > PlayerData[Player].TimeToDropPiece)
                        {
                            if (input->JoystickDirection[PlayerData[Player].PlayerInput] != DOWN)
                            {
                                if (ThinkRussianTimer == 0) audio->PlayDigitalSoundFX(2, 0);
                                PlayerData[Player].DropBonus = 0;
                            }
                            else  PlayerData[Player].DropBonus++;
                        }

                        if (input->JoystickButtonOne[PlayerData[Player].PlayerInput] == ON)
                        {
                            if (PlayerData[Player].PieceRotated1 == false)
                            {
                                RotatePieceCounterClockwise();
                                PlayerData[Player].PieceRotated1 = true;
                            }
                        }
                        else PlayerData[Player].PieceRotated1 = false;

                        if (input->JoystickButtonTwo[PlayerData[Player].PlayerInput] == ON)
                        {
                            if (PlayerData[Player].PieceRotated2 == false)
                            {
                                RotatePieceClockwise();
                                PlayerData[Player].PieceRotated2 = true;
                            }
                        }
                        else PlayerData[Player].PieceRotated2 = false;

                        if (input->JoystickDirection[PlayerData[Player].PlayerInput] == LEFT)  MovePieceLeft();
                        else if (input->JoystickDirection[PlayerData[Player].PlayerInput] == RIGHT)  MovePieceRight();
                        else  PlayerData[Player].PieceMovementDelay = 0;
                    }

                    if (GameMode == CrisisMode)
                    {
                        for (int x = 0; x < 10; x++)
                        {
                            if (PlayerData[Player].AttackLines[x][11] > 0)
                            {
                                x = 100;
                                AddAnAttackLineToEnemiesPlayfield();
                            }
                        }

                        if ( (Player == 0 && PlayerData[1].PlayerStatus == GameOver && PlayerData[2].PlayerStatus == GameOver)
                           ||(Player == 1 && PlayerData[0].PlayerStatus == GameOver && PlayerData[2].PlayerStatus == GameOver)
                           ||(Player == 2 && PlayerData[0].PlayerStatus == GameOver && PlayerData[1].PlayerStatus == GameOver) )
                        {
                            if (CrisisModeOnePlayerLeftPlayfieldCleared == false
                                && PlayersCanJoin == false && PlayerData[Player].PlayerStatus == PieceFalling)
                            {
                                DeletePieceFromPlayfieldMemory(Current);
                                DeletePieceFromPlayfieldMemory(DropShadow);

                            	PlayerData[Player].PlayerStatus = ClearingPlayfield;
                            }

                            if (CrisisModeTimer < 300)  CrisisModeTimer++;
                            else
                            {
                                if (AddAnIncompleteLineToPlayfieldCrisisMode() == true)  CrisisModeTimer = 0;
                            }
                        }
                    }
                }
                else if (PlayerData[Player].PlayerStatus == FlashingCompletedLines)  FlashCompletedLines();
                else if (PlayerData[Player].PlayerStatus == ClearingCompletedLines)  ClearCompletedLines();
                else if (PlayerData[Player].PlayerStatus == ClearingPlayfield)
                {
                    if ( CrisisModeClearPlayfield() == false )
                    {
                        PlayerData[Player].PlayerStatus = PieceFalling;
                        CrisisModeOnePlayerLeftPlayfieldCleared = true;
                    }
                }

                if (PlayerData[Player].PieceDropTimer > PlayerData[Player].TimeToDropPiece)  PlayerData[Player].PieceDropTimer = 0;

                if (GameMode == TimeAttack30Mode || GameMode == TimeAttack60Mode || GameMode == TimeAttack120Mode)
                {
                    TimeAttackTimer--;

                    if (TimeAttackTimer == 0)
                    {
                        PlayerData[0].PlayerStatus = GameOver;
                        PlayerData[1].PlayerStatus = GameOver;
                        PlayerData[2].PlayerStatus = GameOver;
                    }
                }
            }
		}

        bool clearPF = false;
        if      (PlayerData[0].PlayerStatus == ClearingPlayfield)  {clearPF = true; Player = 0;}
        else if (PlayerData[1].PlayerStatus == ClearingPlayfield)  {clearPF = true; Player = 1;}
        else if (PlayerData[2].PlayerStatus == ClearingPlayfield)  {clearPF = true; Player = 2;}
	}
}

//-------------------------------------------------------------------------------------------------
bool Logic::CrisisModeClearPlayfield(void)
{
bool returnValue = false;

	for (int y = 23; y > 5; y--)
	{
		for (int x = 2; x < 12; x++)
		{
            PlayerData[Player].Playfield[x][y] = PlayerData[Player].Playfield[x][y-1];
		}
    }

    for (int x = 2; x < 12; x++)
        PlayerData[Player].Playfield[x][5] = 0;

	for (int y = 5; y < 24; y++)
	{
		for (int x = 2; x < 12; x++)
		{
            if (PlayerData[Player].Playfield[x][y] != 0)  returnValue = true;
		}
    }

    if (returnValue == true)  GameDisplayChanged = true;

    return(returnValue);
}

//-------------------------------------------------------------------------------------------------
void Logic::ComputeComputerPlayerMove(void)
{
    if (PlayerData[Player].PlayerStatus != PieceFalling)  return;

    if (game->AIlogic == 1)// "GrandMaster" A.I. by "mattmatteh" - XXX line average!
    {

    }
    else if (game->AIlogic == 0)// "Gift of Sight" A.I. by "JeZ+Lee" - 135 line average!
    {
        if (PlayerData[Player].BestMoveCalculated == false)//Calculate best position/rotation if best not found yet
        {
            DeletePieceFromPlayfieldMemory(Current);//Delete current piece from Playfield memory

            //-Calculate old Playfield height--------------------------------------------
            PlayerData[Player].OldPlayfieldHeight = 23;
            for (int y = 23; y > 5; y--)
            {
                for (int x = 2; x < 12; x++)
                {
                    if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                    {
                        PlayerData[Player].OldPlayfieldHeight = y;
                    }
                }
            }
            //--------------------------------------------Calculate old Playfield height-

            //-Calculate old Playfield trapped holes-------------------------------------
            PlayerData[Player].OldTrappedHoles = 0;
            for (int y = 5; y < 24; y++)
            {
                for (int x = 2; x < 12; x++)
                {
                    if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                    {
                        for (int yTwo = y+1; yTwo < 24; yTwo++)
                        {
                            if (PlayerData[Player].Playfield[x][yTwo] == 0)  PlayerData[Player].OldTrappedHoles++;
                            else if (PlayerData[Player].Playfield[x][yTwo] > 10 && PlayerData[Player].Playfield[x][yTwo] < 20)  yTwo = 999;
                        }
                    }
                }
            }
            //-------------------------------------Calculate old Playfield trapped holes-

            //-Calculate old Playfield one empty block cavern holes----------------------
            PlayerData[Player].OldNumberOfOneBlockCavernHoles = 0;
            for (int y = 23; y > 5; y--)
            {
                for (int x = 2; x < 12; x++)
                {
                    if (PlayerData[Player].Playfield[x][y] == 0 && PlayerData[Player].Playfield[x-1][y] != 0
                     && PlayerData[Player].Playfield[x+1][y] != 0)
                        PlayerData[Player].OldNumberOfOneBlockCavernHoles++;
                }
            }
            //----------------------Calculate old Playfield one empty block cavern holes-

//-Cycle through all Playfield X positions & optimized rotations & drop piece & calculate criteria---------------------
            for (Uint8 pieceTestX = 1; pieceTestX < 11; pieceTestX++)
            {
                for (Uint8 rotationTest = 1; rotationTest <= MaxRotationArray[ PlayerData[Player].Piece ]; rotationTest++)
                {
                    Uint8 TEMP_PieceRotation = PlayerData[Player].PieceRotation; //Backup current piece rotation
                    Uint8 TEMP_PiecePlayfieldX = PlayerData[Player].PiecePlayfieldX; //Backup current piece Playfield X position
                    Uint8 TEMP_PiecePlayfieldY = PlayerData[Player].PiecePlayfieldY; //Backup current piece Playfield Y position

                    PlayerData[Player].PiecePlayfieldX = pieceTestX; //Set test piece Playfield X position
                    PlayerData[Player].PieceRotation = rotationTest; //Set test piece rotation value

                    PlayerData[Player].FallenPieceHeight[pieceTestX][rotationTest] = 0;
                    if (PieceCollision() == CollisionNotTrue)
                    {
                        //-Drop piece until it hits bottom-------------------------------
                        for (int y = PlayerData[Player].PiecePlayfieldY; y < 23; y++)
                        {
                            PlayerData[Player].PiecePlayfieldY = y;
                            if (PieceCollision() != CollisionNotTrue)
                            {
                                PlayerData[Player].PiecePlayfieldY = y-1;
                                PlayerData[Player].FallenPieceHeight[pieceTestX][rotationTest] = PlayerData[Player].PiecePlayfieldY;
                                y = 100;
                            }
                        }
                        //-------------------------------Drop piece until it hits bottom-

                        AddPieceToPlayfieldMemory(Current); //Add dropped piece temporarily to Playfield

                        //-Calculate new Playfield height--------------------------------
                        PlayerData[Player].NewPlayfieldHeight[pieceTestX][rotationTest] = 23;
                        for (int y = 23; y > 5; y--)
                        {
                            for (int x = 2; x < 12; x++)
                            {
                                if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                                    PlayerData[Player].NewPlayfieldHeight[pieceTestX][rotationTest] = y;
                            }
                        }
                        //--------------------------------Calculate new Playfield height-

                        //-Calculate new Playfield trapped holes-------------------------
                        PlayerData[Player].NewTrappedHoles[pieceTestX][rotationTest] = 0;
                        for (int y = 5; y < 24; y++)
                        {
                            for (int x = 2; x < 12; x++)
                            {
                               if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                                {
                                    for (int yTwo = y+1; yTwo < 24; yTwo++)
                                    {
                                        if (PlayerData[Player].Playfield[x][yTwo] == 0)  PlayerData[Player].NewTrappedHoles[pieceTestX][rotationTest]++;
                                        else if (PlayerData[Player].Playfield[x][yTwo] > 10 && PlayerData[Player].Playfield[x][yTwo] < 20)  yTwo = 999;
                                    }
                                }
                            }
                        }
                        //-------------------------Calculate new Playfield trapped holes-

                        //-Calculate number of completed lines---------------------------
                        PlayerData[Player].MoveCompletesLines[pieceTestX][rotationTest] = 0;
                        PlayerData[Player].NewNumberOfOneBlockCavernHoles[pieceTestX][rotationTest] = 0;
                        for (int y = 23; y > 5; y--)
                        {
                            int boxTotal = 0;

                            for (int x = 2; x < 12; x++)
                            {
                                if (PlayerData[Player].Playfield[x][y] == 0 && PlayerData[Player].Playfield[x-1][y] != 0
                                 && PlayerData[Player].Playfield[x+1][y] != 0)
                                    PlayerData[Player].NewNumberOfOneBlockCavernHoles[pieceTestX][rotationTest]++;

                                if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)  boxTotal++;
                            }

                            if (boxTotal == 10)  PlayerData[Player].MoveCompletesLines[pieceTestX][rotationTest]++;
                        }
                        //---------------------------Calculate number of completed lines-

                        //-If there are completed lines, recalculate search criteria-----
                        if (PlayerData[Player].MoveCompletesLines[pieceTestX][rotationTest] > 0)
                        {
                            for (int y = 5; y < 24; y++)
                                for (int x = 2; x < 12; x++)
                                    PlayerData[Player].PlayfieldBackup[x][y] = PlayerData[Player].Playfield[x][y];

                            for (int y = 5; y < 24; y++)
                            {
                                int boxTotal = 0;
                                for (int x = 2; x < 12; x++)
                                {
                                    if ( (PlayerData[Player].Playfield[x] [y] > 9)&&(PlayerData[Player].Playfield[x][y] < 20) )
                                        boxTotal++;
                                }

                                if (boxTotal == 10)
                                {
                                    for (int yTwo = y; yTwo > 5; yTwo--)
                                        for (int xTwo = 2; xTwo < 12; xTwo++)
                                            PlayerData[Player].Playfield[xTwo][yTwo] = PlayerData[Player].Playfield[xTwo][yTwo-1];

                                    for (int xTwo = 2; xTwo < 12; xTwo++)
                                        PlayerData[Player].Playfield[xTwo][5] = 0;
                                }
                            }

                            //-Recalculate Playfield height--------------------------------------------------
                            for (int y = 23; y > 5; y--)
                            {
                                for (int x = 2; x < 12; x++)
                                {
                                    if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                                        PlayerData[Player].NewPlayfieldHeight[pieceTestX][rotationTest] = y;
                                }
                            }
                            //--------------------------------------------------Recalculate Playfield height-

                            //-Recalculate trapped holes in Playfield----------------------------------------
                            PlayerData[Player].NewTrappedHoles[pieceTestX][rotationTest] = 0;
                            for (int y = 5; y < 24; y++)
                            {
                                for (int x = 2; x < 12; x++)
                                {
                                   if (PlayerData[Player].Playfield[x][y] > 10 && PlayerData[Player].Playfield[x][y] < 20)
                                    {
                                        for (int yTwo = y+1; yTwo < 24; yTwo++)
                                        {
                                            if (PlayerData[Player].Playfield[x][yTwo] == 0)  PlayerData[Player].NewTrappedHoles[pieceTestX][rotationTest]++;
                                            else if (PlayerData[Player].Playfield[x][yTwo] > 10 && PlayerData[Player].Playfield[x][yTwo] < 20)  yTwo = 999;
                                        }
                                    }
                                }
                            }
                            //----------------------------------------Recalculate trapped holes in Playfield-

                            //-Recalculate Playfield one empty block cavern holes----------------------------
                            PlayerData[Player].NewNumberOfOneBlockCavernHoles[pieceTestX][rotationTest] = 0;
                            for (int y = 23; y > 5; y--)
                            {
                                for (int x = 2; x < 12; x++)
                                {
                                    if (PlayerData[Player].Playfield[x][y] == 0 && PlayerData[Player].Playfield[x-1][y] != 0
                                     && PlayerData[Player].Playfield[x+1][y] != 0)
                                        PlayerData[Player].NewNumberOfOneBlockCavernHoles[pieceTestX][rotationTest]++;
                                }
                            }
                            //----------------------------Recalculate Playfield one empty block cavern holes-

                            for (int y = 5; y < 24; y++)
                                for (int x = 2; x < 12; x++)
                                    PlayerData[Player].Playfield[x][y] = PlayerData[Player].PlayfieldBackup[x][y];
                        }
                        //-----If there are completed lines, recalculate search criteria-

                        DeletePieceFromPlayfieldMemory(Current); //Delete piece from Playfield memory
                    }
                    else //Piece is in collision at this Playfield X position
                    {
                        PlayerData[Player].NewTrappedHoles[pieceTestX][rotationTest] = 9999999;
                    }

                    PlayerData[Player].PieceRotation = TEMP_PieceRotation; //Restore original piece rotation
                    PlayerData[Player].PiecePlayfieldX = TEMP_PiecePlayfieldX; //Restore original piece Playfield X position
                    PlayerData[Player].PiecePlayfieldY = TEMP_PiecePlayfieldY; //Restore original piece Playfield Y position
                }

            }
//---------------------Cycle through all Playfield X positions & optimized rotations & drop piece & calculate criteria-

            PlayerData[Player].BestMoveX = -1; //Set best piece Playfield X position to no value(-1)
            PlayerData[Player].BestRotation = -1; //Set best piece rotation to no valye(-1)
            int bestValue = 9999999; //Set best value variable to insanely high value(lower means better)

            //-If a move/rotation exists with 3 or more completed lines then do it-------
            if (PlayerData[Player].Piece == 4 || PlayerData[Player].Piece == 5 || PlayerData[Player].Piece == 7)
            {
                for (int movePosX = 1; movePosX < 11; movePosX++)
                {
                    for (Uint8 rotationValue = 1; rotationValue <= MaxRotationArray[ PlayerData[Player].Piece ]; rotationValue++)
                    {
                        if ( PlayerData[Player].MoveCompletesLines[movePosX][rotationValue] > 2
                            && (4 - PlayerData[Player].MoveCompletesLines[movePosX][rotationValue]) < bestValue )
                        {
                            bestValue = (4 - PlayerData[Player].MoveCompletesLines[movePosX][rotationValue]);

                            PlayerData[Player].BestMoveX = movePosX;
                            PlayerData[Player].BestRotation = rotationValue;
                        }
                    }
                }
            }
            //-------If a move/rotation exists with 2 or more completed lines then do it-

            //-If best not found yet, look for a move where there are no additional trapped holes------------
            if (bestValue == 9999999)
            {
                for (int movePosX = 1; movePosX < 11; movePosX++)
                {
                    for (Uint8 rotationValue = 1; rotationValue <= MaxRotationArray[ PlayerData[Player].Piece ]; rotationValue++)
                    {
                        if ( (PlayerData[Player].NewTrappedHoles[movePosX][rotationValue] - PlayerData[Player].OldTrappedHoles < 1)
                            && (PlayerData[Player].FallenPieceHeight[movePosX][rotationValue] > 14) )
                        {
                            if ( (PlayerData[Player].NewNumberOfOneBlockCavernHoles[movePosX][rotationValue] - PlayerData[Player].OldNumberOfOneBlockCavernHoles)
                                + (4 - PlayerData[Player].MoveCompletesLines[movePosX][rotationValue])
                                + (PlayerData[Player].OldPlayfieldHeight - PlayerData[Player].NewPlayfieldHeight[movePosX][rotationValue])
                                < bestValue )
                            {
                                bestValue = (PlayerData[Player].NewNumberOfOneBlockCavernHoles[movePosX][rotationValue] - PlayerData[Player].OldNumberOfOneBlockCavernHoles)
                                             + (4 - PlayerData[Player].MoveCompletesLines[movePosX][rotationValue])
                                             + (PlayerData[Player].OldPlayfieldHeight - PlayerData[Player].NewPlayfieldHeight[movePosX][rotationValue]);

                                PlayerData[Player].BestMoveX = movePosX;
                                PlayerData[Player].BestRotation = rotationValue;
                            }
                        }
                    }
                }
            }
            //------------If best not found yet, look for a move where there are no additional trapped holes-

            //-If best not found yet, look for move with best value possible---------------------------------
            if (bestValue == 9999999)
            {
                for (int movePosX = 1; movePosX < 11; movePosX++)
                {
                    for (Uint8 rotationValue = 1; rotationValue <= MaxRotationArray[ PlayerData[Player].Piece ]; rotationValue++)
                    {
                        if ( (2*(PlayerData[Player].NewTrappedHoles[movePosX][rotationValue] - PlayerData[Player].OldTrappedHoles))
                               + ((PlayerData[Player].NewNumberOfOneBlockCavernHoles[movePosX][rotationValue] - PlayerData[Player].OldNumberOfOneBlockCavernHoles))
                               + (21 - PlayerData[Player].FallenPieceHeight[movePosX][rotationValue])
                               + (4-PlayerData[Player].MoveCompletesLines[movePosX][rotationValue])
                               + (PlayerData[Player].OldPlayfieldHeight - PlayerData[Player].NewPlayfieldHeight[movePosX][rotationValue])
                               < bestValue )
                        {
                            bestValue = (2*(PlayerData[Player].NewTrappedHoles[movePosX][rotationValue] - PlayerData[Player].OldTrappedHoles))
                                         + ((PlayerData[Player].NewNumberOfOneBlockCavernHoles[movePosX][rotationValue] - PlayerData[Player].OldNumberOfOneBlockCavernHoles))
                                         + (21-PlayerData[Player].FallenPieceHeight[movePosX][rotationValue])
                                         + (4-PlayerData[Player].MoveCompletesLines[movePosX][rotationValue])
                                         + (PlayerData[Player].OldPlayfieldHeight - PlayerData[Player].NewPlayfieldHeight[movePosX][rotationValue]);

                            PlayerData[Player].BestMoveX = movePosX;
                            PlayerData[Player].BestRotation = rotationValue;
                        }
                    }
                }
            }
            //---------------------------------If best not found yet, look for move with best value possible-

            PlayerData[Player].BestMoveCalculated = true;
        }

        //-Move/rotate piece to best-------------------------------------------------------------------------
        if (PlayerData[Player].MovedToBestMove == false && PlayerData[Player].BestMoveX != -1 && PlayerData[Player].BestRotation != -1)
        {
            if (PlayerData[Player].PieceRotation < PlayerData[Player].BestRotation)  RotatePieceClockwise();

            else if (PlayerData[Player].PieceRotation > PlayerData[Player].BestRotation)  RotatePieceCounterClockwise();

            if (PlayerData[Player].BestMoveX < PlayerData[Player].PiecePlayfieldX)
                MovePieceLeft();
            else if (PlayerData[Player].BestMoveX > PlayerData[Player].PiecePlayfieldX)
                MovePieceRight();
            else if (PlayerData[Player].PieceRotation == PlayerData[Player].BestRotation)
            {
                if (game->CPUPlayerEnabled != 4)  MovePieceDown(true);
                PlayerData[Player].MovedToBestMove = true;
            }
        }
        else
        {
            if (game->CPUPlayerEnabled != 4)  MovePieceDown(false);
            else  MovePieceDownFast();
        }
        //-------------------------------------------------------------------------Move/rotate piece to best-
    }
}
