/*                                       JeZ+Lee's
                                Cross-Platform Open-Source
                                  S.D.L. 1.2 + OpenGL(R)
   _______ _________   _______     _______  _        _______ _________ _        _______ TM
  (  ____ \\__   __/  (  ____ )   (  ____ \( (    /|(  ____ \\__   __/( (    /|(  ____ \
  | (    \/   ) (     | (    )|   | (    \/|  \  ( || (    \/   ) (   |  \  ( || (    \/
  | |         | |  __ | (____)|   | (__    |   \ | || |         | |   |   \ | || (__
  | | ____    | | |__||     __)   |  __)   | (\ \) || | ____    | |   | (\ \) ||  __)
  | | \_  )   | |     | (\ (      | (      | | \   || | \_  )   | |   | | \   || (
  | (___) |   | |     | ) \ \__   | (____/\| )  \  || (___) |___) (___| )  \  || (____/\
  (_______)   |_|     |/   \__/   (_______/|/    )_)(_______)\_______/|/    )_)(_______/

 = T  W  I  N  +  T  U  R  B  O =========================================================

                                   V E R S I O N   2 . 0
  --------------------------------------------------------------------------------------
                                                       TM
                                    T-Crisis 3 100% A.I.
                                           3.5.12
                             (C)opyright 2011, By 16BitSoft(R)
                                     www.16BitSoft.com
  -------------------------------------------------------------------------------------- */

#include <time.h> // Needed to randomize random number generator.

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "game.h"
#include "visuals.h"
#include "input.h"
#include "screens.h"
#include "interface.h"
#include "audio.h"
#include "settings.h"
#include "data.h"
#include "logic.h"
#include "runtime.h"

Game *game;
Visuals *visuals;
Input *input;
Screens *screens;
Interface *interface;
Audio *audio;
Settings *settings;
Data *data;
Logic *logic;
Runtime *runtime;

//-------------------------------------------------------------------------------------------------
int main( int argc, char* args[] )
{
    game = new Game();

    settings = new Settings(argc, args);

    visuals = new Visuals(settings->GetDataPath());

    runtime = new Runtime(argc, args);

    if ( visuals->SetupGameWindow( runtime->getWidth(), runtime->getHeight() ) != true )  game->CoreFailure = true;

    if ( visuals->LoadSpritesIntoMemoryAndInitialize() != true )  game->CoreFailure = true;
    if ( visuals->LoadFontsIntoMemory() != true )  game->CoreFailure = true;
    if ( visuals->LoadAllTextsIntoMemory() != true )  game->CoreFailure = true;

    input = new Input();

    screens = new Screens();

    interface = new Interface(settings->GetDataPath());
    if ( interface->CreateAllGUIAndInitialize(true) != true )  game->CoreFailure = true;

    data = new Data();
    data->ClearHighScores();

    logic = new Logic();

    srand( (unsigned)time(NULL) ); // Place unique time seed into random number generator.

    audio = new Audio(settings->GetDataPath());

    data->LoadHighScoresAndOptions();

    audio->SetupAudio();

    if (game->FullScreenMode == true)
    {
        visuals->SetupGameWindowAgain();
    }

    audio->PlayMusic(0, 0);

    printf("Main loop...\n");
//-MAIN+LOOP--------------------------------------------------------------
    while (game->ExitGame == false && game->CoreFailure == false)
    {
        if (input->KeyOnKeyboardPressedByUser != '~')  input->GetAllUserInput();
        if (input->KeyOnKeyboardPressedByUser == '~')  game->ExitGame = true;

        game->CalculateFramerate();

        switch(screens->ScreenToDisplay)
        {
            case cnetScreen:
                screens->DisplaycnetScreen();
                break;

            case SixteenBitSoftScreen:
                screens->DisplaySixteenBitSoftScreen();
                break;

            case TitleScreen:
                screens->DisplayTitleScreen();
                break;

            case AboutScreen:
                screens->DisplayAboutScreen();
                break;

            case HowToPlayScreen:
                screens->DisplayHowToPlayScreen();
                break;

            case HighScoresScreen:
                screens->DisplayHighScoresScreen();
                break;

            case OptionsScreen:
                screens->DisplayOptionsScreen();
                break;

            case NewGameScreen:
                screens->DisplayNewGameScreen();
                break;

            case PlayingGameScreen:
                screens->DisplayPlayingGameScreen();
                break;

            case NameInputKeyboardScreen:
                screens->DisplayNameInputKeyboardScreen();
                break;

            case NameInputJoystickScreen:
                screens->DisplayNameInputJoystickScreen();
                break;

            default:
                break;
        }

        if (Mix_PlayingMusic() == false && logic->ThinkRussianTimer == 0)
            audio->PlayMusic(audio->CurrentlySelectedMusicTrack, 0); ///BUG - Band-aid...

        screens->ApplyScreenFadeTransition();

        if (game->Debug != false)  game->DisplayDebuggingInfo();

        game->ProcessFramerate();

        if (screens->ScreenIsDirty == true)
        {
            if (game->Debug == 2 && screens->ScreenToDisplay == PlayingGameScreen){}
            else  visuals->DisplayScreenBufferOntoDisplay();
            if (game->Debug != 1)  screens->ScreenIsDirty = false;
        }
    }
//--------------------------------------------------------------MAIN+LOOP-
    printf("...Quit game\n");

    data->SaveHighScoresAndOptions();

    if (game->FullScreenMode == true)
    {
        game->FullScreenMode = false;
        visuals->SetupGameWindowAgain();
    }

    delete runtime;
    delete logic;
    delete data;
    delete settings;
    delete audio;
    delete interface;
    delete screens;
    delete input;
    delete visuals;
    delete game;

    return(0);
}

//                   TM
// "A 100% By JeZ+Lee"...
