#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "screens.h"

#include "visuals.h"
#include "input.h"
#include "game.h"
#include "audio.h"
#include "interface.h"
#include "logic.h"
#include "data.h"

extern Visuals* visuals;
extern Input* input;
extern Game* game;
extern Audio* audio;
extern Interface* interface;
extern Logic* logic;
extern Data* data;

//-------------------------------------------------------------------------------------------------
Screens::Screens(void)
{
    ScreenFadeTransparency = 1.0f;
    ScreenTransitionStatus = FadeIn;

    ScreenToDisplay = cnetScreen;

    ExitToTitleScreen = false;

    ScreenMustBeDrawn = true;

    DisplayScreenTimer = 450;
}

//-------------------------------------------------------------------------------------------------
Screens::~Screens(void)
{

}

//-------------------------------------------------------------------------------------------------
void Screens::ApplyScreenFadeTransition(void)
{
    if (ScreenTransitionStatus == FadeNone)
    {
        if (DisplayScreenTimer > 0)  DisplayScreenTimer--;

        if ( (input->MouseButtonPressed[0] == true || input->JoystickButtonOne[Any] == ON
              || input->KeyOnKeyboardPressedByUser == '_') && DisplayScreenTimer > 0 )
        {
            DisplayScreenTimer = 0;
            audio->PlayDigitalSoundFX(0, 0);
        }

        if (DisplayScreenTimer == 0)
        {
            ScreenTransitionStatus = FadeOut;
            DisplayScreenTimer = -1;
        }

        return;
    }

    if (ScreenTransitionStatus == FadeIn)
    {
        if (ScreenFadeTransparency > 0.0f)  ScreenFadeTransparency -= 0.2f;
        else
        {
            ScreenFadeTransparency = 0.0f;
            ScreenTransitionStatus = FadeNone;
        }
    }
    else if (ScreenTransitionStatus == FadeOut)
    {
        if (ScreenFadeTransparency < 1.0f)  ScreenFadeTransparency += 0.2f;
        else
        {
            ScreenFadeTransparency = 1.0f;
            ScreenTransitionStatus = FadeAll;
        }
    }

    for (GLfloat y = 0; y < 480+64; y+=64)
    {
        for (GLfloat x = 0; x < 640+64; x+=64)
        {
            glPushMatrix();

            glEnable(GL_TEXTURE_2D);
            glBindTexture(GL_TEXTURE_2D, visuals->Sprites[0].OGL_Texture);

            glTranslatef(x, y, 0.0f);
            glColor4f(1.0f, 1.0f, 1.0f, ScreenFadeTransparency);

            glEnable(GL_BLEND);
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

            glBegin(GL_QUADS);
                glTexCoord2i(0, 0);
                glVertex3i(0-32, 0-32, 0);

                glTexCoord2i(1, 0);
                glVertex3i(0+32, 0-32, 0);

                glTexCoord2i(1, 1);
                glVertex3i(0+32, 0+32, 0);

                glTexCoord2i(0, 1);
                glVertex3i(0-32, 0+32, 0);
            glEnd();

            glPopMatrix();
        }
    }

    logic->GameDisplayChanged = true;;
    ScreenMustBeDrawn = true;
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplaycnetScreen(void)
{
    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(255, 255, 255, 255);

        visuals->Sprites[8].ScreenX = 320;
        visuals->Sprites[8].ScreenY = 240;
        visuals->DrawSpriteOntoScreenBuffer(8);

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenToDisplay = SixteenBitSoftScreen;
        DisplayScreenTimer = 450;
        ScreenMustBeDrawn = true;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplaySixteenBitSoftScreen(void)
{
    if (DisplayScreenTimer == 450)
    {
        ScreenTransitionStatus = FadeIn;
        DisplayScreenTimer = 449;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[10].ScreenX = 320;
        visuals->Sprites[10].ScreenY = 240;

        visuals->Sprites[10].ScaleX = 5.0;
        visuals->Sprites[10].ScaleY = 8.0;
        visuals->Sprites[10].Smooth = false;
        visuals->DrawSpriteOntoScreenBuffer(10);

        visuals->DrawTextOntoScreenBuffer("TM", visuals->FontDefault[2]
                                          , 55, 295, JustifyRight, 1, 255, 1, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Bringing back old memories from the 16bit era!", visuals->FontDefault[1]
                                          , 0, 307, JustifyCenter, 1, 255, 1, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("www.16BitSoft.com", visuals->FontDefault[0]
                                          , 0, 447, JustifyCenter, 1, 255, 1, 1, 90, 1);

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    if (input->KeyOnKeyboardPressedByUser == 'd')
    {
        game->Debug++;
        audio->PlayDigitalSoundFX(12, 0);
        input->DelayAllUserInput = 20;
        ScreenIsDirty = true;
        ScreenMustBeDrawn = true;
    }

    if (game->Debug > 4)  game->Debug = 0;

    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenToDisplay = TitleScreen;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayTitleScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(2, 0, 320, 196);
        interface->CreateButton(3, 1, 320, 238);
        interface->CreateButton(4, 2, 320, 280);
        interface->CreateButton(5, 3, 320, 322);
        interface->CreateButton(6, 4, 320, 364);
        interface->CreateButton(7, 5, 320, 406);

        interface->ButtonSelected = 0;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->Sprites[25].ScreenX = 320;
        visuals->Sprites[25].ScreenY = 74;
        visuals->Sprites[25].ScaleX = 0.95f;
        visuals->Sprites[25].ScaleY = 0.95f;
        visuals->DrawSpriteOntoScreenBuffer(25);

        visuals->DrawTextOntoScreenBuffer("JeZ+Lee's", visuals->FontDefault[1]
                                          , 0, 1, JustifyCenter, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("A.I.350Avg", visuals->FontDefault[5]
                                          , 0, 110, JustifyRight, 0, 255, 0, 0, 100, 0);

        char temp[256];
        strcpy(visuals->VariableText, "''");
        strcat(visuals->VariableText, data->HighScoresName[logic->GameMode][0]);
        strcat(visuals->VariableText, "'' Scored: ");
		sprintf(temp, "%d", data->HighScoresScore[logic->GameMode][0]);
        strcat(visuals->VariableText, temp);

        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], 0, 140, JustifyCenter
                                          , 255, 255, 0, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Ver. 3.5.12", visuals->FontDefault[2]
                                          , 2, 457, JustifyRight, 255, 255, 255, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        char *copy = new char[2];
        char *reg = new char[2];
        sprintf(copy, "%c", 0xA9);
        sprintf(reg, "%c", 0xAE);
        strcpy(visuals->VariableText, copy);
        strcat(visuals->VariableText, "opyright 2011, By 16BitSoft");
        strcat(visuals->VariableText, reg);
        visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0]
                                          , 0, 420, JustifyCenter, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("www.16BitSoft.com", visuals->FontDefault[0]
                                          , 0, 447, JustifyCenter, 255, 255, 255, 0, 0, 0);

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();

    if (game->Debug == 2 && input->JoyButton1[Any] == ON)
        ScreenTransitionStatus = FadeOut;

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (interface->ButtonSelected == 5)
        {
            input->KeyOnKeyboardPressedByUser = '~';
        }
        else if (interface->ButtonSelected == 4)
        {
            ScreenToDisplay = AboutScreen;
            input->DelayAllUserInput = 20;
        }
        else if (interface->ButtonSelected == 2)
        {
            ScreenToDisplay = HowToPlayScreen;
            input->DelayAllUserInput = 20;
        }
        else if (interface->ButtonSelected == 3)
        {
            ScreenToDisplay = HighScoresScreen;
            input->DelayAllUserInput = 20;
        }
        else if (interface->ButtonSelected == 1)
        {
            ScreenToDisplay = OptionsScreen;
            input->DelayAllUserInput = 20;
        }
        else if (interface->ButtonSelected == 0)
        {
            ScreenToDisplay = NewGameScreen;
            input->DelayAllUserInput = 20;
        }

        interface->DestroyAllButtons();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayAboutScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        for (int index = 0; index < (visuals->TotalNumberOfLoadedTexts+1); index++)
        {
            if (index == 0)
            {
                visuals->LoadedTexts[0].ScreenX = 465;
                visuals->LoadedTexts[0].ScreenY = 491;
            }
            else if (index == 1)
            {
                visuals->LoadedTexts[1].ScreenX = 320;
                visuals->LoadedTexts[1].ScreenY = 510;
            }
            else if (index == 2)
            {
                visuals->LoadedTexts[2].ScreenX = 320;
                visuals->LoadedTexts[2].ScreenY = 540;
            }
            else if (index < visuals->TotalNumberOfLoadedTexts)
            {
                if (visuals->LoadedTexts[index-1].blueHue == 1.0f && visuals->LoadedTexts[index].blueHue == 0)
                {
                    visuals->LoadedTexts[index].ScreenX = 320;
                    visuals->LoadedTexts[index].ScreenY = visuals->LoadedTexts[index-1].ScreenY+60;
                }
                else
                {
                    visuals->LoadedTexts[index].ScreenX = 320;
                    visuals->LoadedTexts[index].ScreenY = visuals->LoadedTexts[index-1].ScreenY+30;
                }
            }
            else if (index == visuals->TotalNumberOfLoadedTexts)
            {
                visuals->LoadedTexts[index].ScreenX = 320;
                visuals->LoadedTexts[index].ScreenY = visuals->LoadedTexts[index-1].ScreenY+240;
            }
        }

        interface->ButtonSelected = 0;

        if (logic->CrisisWon == false) audio->PlayMusic(0, 0);
        else if (logic->GameMode == CrisisMode)  audio->PlayMusic(24, 0);

        StaffNotWinBG1Zoom = 1.7f;

        StaffNotWinSpr1Transparency = 0.0f;
    }

    if (ScreenMustBeDrawn == true)
    {
        if (StaffNotWinBG1Zoom > 0.1f && StaffNotWinBG1Zoom < 0.2f)
            visuals->ClearScreenBufferWithColor(255, 255, 255, 255);
        else
            visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->Sprites[20].ScaleX = StaffNotWinBG1Zoom;
        visuals->Sprites[20].ScaleY = StaffNotWinBG1Zoom;
        if (game->NetBookMode == false)  visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->Sprites[320].ScreenX = 320;
        visuals->Sprites[320].ScreenY = 240;
        visuals->Sprites[320].Transparency = StaffNotWinSpr1Transparency;
        if (game->NetBookMode == false)  visuals->DrawSpriteOntoScreenBuffer(320);

        for (int index = 0; index < (visuals->TotalNumberOfLoadedTexts + 1); index++)
        {
            visuals->DrawLoadedTextOntoScreenBuffer(index, visuals->LoadedTexts[index].ScreenX
                                                    , visuals->LoadedTexts[index].ScreenY);
        }
    }

    for (int index = 0; index < (visuals->TotalNumberOfLoadedTexts + 1); index++)
    {
        visuals->LoadedTexts[index].ScreenY-=0.8f;
    }

    if (StaffNotWinBG1Zoom > 0.0f)    StaffNotWinBG1Zoom-=0.001f;

    if (StaffNotWinSpr1Transparency < 1.0f)    StaffNotWinSpr1Transparency+=0.0005f;

    if (visuals->LoadedTexts[visuals->TotalNumberOfLoadedTexts].ScreenY < -20)
        ScreenTransitionStatus = FadeOut;

    ScreenMustBeDrawn = true;
    ScreenIsDirty = true;

    if ( input->KeyOnKeyboardPressedByUser == '_'
        || input->KeyOnKeyboardPressedByUser == '/'
        || input->MouseButtonPressed[0] == true
        || input->JoystickButtonOne[Any] == ON)
    {
        ScreenTransitionStatus = FadeOut;
    }

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        ScreenToDisplay = TitleScreen;

        if (logic->CrisisWon == true)
        {
            data->CheckForNewHighScore();
            data->NameInputArayIndex = 0;

            if (data->PlayerRankOnGameOver < 10)
            {
                if (logic->PlayerData[data->PlayerWithHighestScore].PlayerInput == Keyboard)  ScreenToDisplay = NameInputKeyboardScreen;
                else  ScreenToDisplay = NameInputJoystickScreen;

                logic->CrisisWon = false;
            }
            else
            {
                ScreenToDisplay = HighScoresScreen;
            }
        }

        ScreenTransitionStatus = FadeAll;

        visuals->Sprites[20].ScaleX = 1.0f;
        visuals->Sprites[20].ScaleY = 1.0f;

        input->DelayAllUserInput = 20;

        interface->ButtonSelected = 0;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayHowToPlayScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(8, 0, 320, 458);

        interface->ButtonSelected = 0;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->DrawTextOntoScreenBuffer("HOW TO PLAY:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->Sprites[30].ScreenX = 320;
        visuals->Sprites[30].ScreenY = 240;
        visuals->DrawSpriteOntoScreenBuffer(30);


        if (input->UserDefinedRotateCounterClockwise != '^' && input->UserDefinedRotateClockwise != '^')
        {
            char temp[64];

            sprintf(temp, "%c", input->UserDefinedRotateCounterClockwise);
            strcpy(visuals->VariableText, "[");
            strcat(visuals->VariableText, temp);
            strcat(visuals->VariableText, "] = Rotate Counter-Clockwise");
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0]
                                              , 0, 340, JustifyCenter, 255, 255, 255, 0, 0, 0);

            sprintf(temp, "%c", input->UserDefinedRotateClockwise);
            strcpy(visuals->VariableText, "[");
            strcat(visuals->VariableText, temp);
            strcat(visuals->VariableText, "] = Rotate Clockwise");
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0]
                                              , 0, 370, JustifyCenter, 255, 255, 255, 0, 0, 0);
        }

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (interface->ButtonSelected == 0)
        {
            ScreenToDisplay = TitleScreen;
            input->DelayAllUserInput = 20;
        }

        interface->DestroyAllButtons();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayHighScoresScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(8, 0, 320, 458);
        interface->CreateArrowSet(0, 1, 42);

        interface->ButtonSelected = 0;
        interface->ArrowSetSelected = 0;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->DrawTextOntoScreenBuffer("HIGH SCORES:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllArrowSetsOntoScreenBuffer();

        if (logic->GameMode == OriginalMode)
            visuals->DrawTextOntoScreenBuffer("Original Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);
        else if (logic->GameMode == TimeAttack30Mode)
            visuals->DrawTextOntoScreenBuffer("Time Attack 30 Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);
        else if (logic->GameMode == TimeAttack60Mode)
            visuals->DrawTextOntoScreenBuffer("Time Attack 60 Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);
        else if (logic->GameMode == TimeAttack120Mode)
            visuals->DrawTextOntoScreenBuffer("Time Attack 120 Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);
        else if (logic->GameMode == TwentyLineChallengeMode)
            visuals->DrawTextOntoScreenBuffer("20 Line Challenge Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);
        else if (logic->GameMode == CrisisMode)
            visuals->DrawTextOntoScreenBuffer("Crisis+Mode", visuals->FontDefault[0]
                                              , 0, 42, JustifyCenter, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Name:", visuals->FontDefault[0],
                                          37, 81, JustifyLeft, 200, 200, 200, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Level:", visuals->FontDefault[0],
                                          420, 81, JustifyLeft, 200, 200, 200, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Score:", visuals->FontDefault[0],
                                          510, 81, JustifyLeft, 200, 200, 200, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("1", visuals->FontDefault[0], 1, 112, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("2", visuals->FontDefault[0], 1, 142, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("3", visuals->FontDefault[0], 1, 172, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("4", visuals->FontDefault[0], 1, 202, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("5", visuals->FontDefault[0], 1, 232, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("6", visuals->FontDefault[0], 1, 263, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("7", visuals->FontDefault[0], 1, 292, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("8", visuals->FontDefault[0], 1, 322, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("9", visuals->FontDefault[0], 1, 352, JustifyLeft, 200, 200, 200, 1, 1, 1);
        visuals->DrawTextOntoScreenBuffer("10", visuals->FontDefault[0], 1, 382, JustifyLeft, 200, 200, 200, 1, 1, 1);

        GLfloat rankY = 112;
        GLfloat offsetY = 30;

        for (Uint8 index = 0; index < 10; index++)
        {
            Uint8 greenBlueColorValue = 255;

            for (int playerIndex = 0; playerIndex < NumberOfPlayers; playerIndex++)
            {
                if (data->PlayerWithHighestScore == playerIndex
                    && data->HighScoresScore[logic->GameMode][index] == logic->PlayerData[playerIndex].Score
                    && data->HighScoresLevel[logic->GameMode][index] == logic->PlayerData[playerIndex].Level)
                {
                    greenBlueColorValue = 0;
                }
            }

            visuals->DrawTextOntoScreenBuffer(data->HighScoresName[logic->GameMode][index], visuals->FontDefault[0]
                                              , 37, rankY, JustifyLeft, 255, greenBlueColorValue, greenBlueColorValue, 1, 1, 1);

            if (logic->GameMode == CrisisMode && data->HighScoresLevel[logic->GameMode][index] == 10)
            {
                visuals->DrawTextOntoScreenBuffer("Won!", visuals->FontDefault[0], 420, rankY
                                                  , JustifyLeft, 255, greenBlueColorValue, greenBlueColorValue, 1, 1, 1);
            }
            else
            {
                sprintf(visuals->VariableText, "%d", data->HighScoresLevel[logic->GameMode][index]);
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], 420, rankY
                                                  , JustifyLeft, 255, greenBlueColorValue, greenBlueColorValue, 1, 1, 1);
            }

            sprintf(visuals->VariableText, "%d", data->HighScoresScore[logic->GameMode][index]);
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], 510, rankY, JustifyLeft
                                              , 255, greenBlueColorValue, greenBlueColorValue, 1, 1, 1);

            rankY += offsetY;
        }

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();
    interface->ArrowSelected = -1;
    interface->ProcessAllArrowSetsOnScreen();

    if (input->KeyOnKeyboardPressedByUser == 'c')
    {
        data->ClearHighScores();
        input->DelayAllUserInput = 10;
        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
    }

    if (game->Debug == 2 && input->JoyButton1[Any] == ON)
        ScreenTransitionStatus = FadeOut;

    if (interface->ArrowSelected == 0)
    {
        if (logic->GameMode > 0)  logic->GameMode--;
        else  logic->GameMode = 5;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
    }
    else if (interface->ArrowSelected == 1)
    {
        if (logic->GameMode < 5)  logic->GameMode++;
        else  logic->GameMode = 0;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
    }

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (interface->ButtonSelected == 0)
        {
            ScreenToDisplay = TitleScreen;
            input->DelayAllUserInput = 20;
        }

        if (logic->GameWasJustPlayed == true)
        {
            audio->PlayMusic(0, 0);
            logic->GameWasJustPlayed = false;
        }

        interface->DestroyAllButtons();
        interface->DestroyAllArrowSets();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayOptionsScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(8, 0, 320, 458);
        interface->CreateArrowSet(0, 1, 50);
        interface->CreateArrowSet(2, 3, 95);
        interface->CreateArrowSet(4, 5, 140);
        interface->CreateArrowSet(6, 7, 185);
        interface->CreateArrowSet(8, 9, 230);
        interface->CreateArrowSet(10, 11, 275);
        interface->ButtonSelected = 0;
        interface->ArrowSetSelected = 0;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->DrawTextOntoScreenBuffer("OPTIONS:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();
        interface->DisplayAllArrowSetsOntoScreenBuffer();

        visuals->DrawTextOntoScreenBuffer("Music Volume:", visuals->FontDefault[0]
                                          , 70, 50, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (audio->MusicVolume == 0)
            visuals->DrawTextOntoScreenBuffer("0%", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->MusicVolume == 32)
            visuals->DrawTextOntoScreenBuffer("25%", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->MusicVolume == 64)
            visuals->DrawTextOntoScreenBuffer("50%", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->MusicVolume == 96)
            visuals->DrawTextOntoScreenBuffer("75%", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->MusicVolume == 128)
            visuals->DrawTextOntoScreenBuffer("100%", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Sound Effects Volume:", visuals->FontDefault[0]
                                          , 70, 95, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (audio->SoundVolume == 0)
            visuals->DrawTextOntoScreenBuffer("0%", visuals->FontDefault[0]
                                              , 70, 95, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->SoundVolume == 32)
            visuals->DrawTextOntoScreenBuffer("25%", visuals->FontDefault[0]
                                              , 70, 95, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->SoundVolume == 64)
            visuals->DrawTextOntoScreenBuffer("50%", visuals->FontDefault[0]
                                              , 70, 95, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->SoundVolume == 96)
            visuals->DrawTextOntoScreenBuffer("75%", visuals->FontDefault[0]
                                              , 70, 95, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (audio->SoundVolume == 128)
            visuals->DrawTextOntoScreenBuffer("100%", visuals->FontDefault[0]
                                              , 70, 95, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Slow Game Performance Fix:", visuals->FontDefault[0]
                                          , 70, 140, JustifyLeft, 255, 255, 255, 0, 0, 0);
        if (game->NetBookMode == false)
            visuals->DrawTextOntoScreenBuffer("OFF", visuals->FontDefault[0]
                                              , 70, 140, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->NetBookMode == true)
            visuals->DrawTextOntoScreenBuffer("ON", visuals->FontDefault[0]
                                              , 70, 140, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("A.I. Players Speed:", visuals->FontDefault[0]
                                          , 70, 185, JustifyLeft, 255, 255, 255, 0, 0, 0);
        if (game->CPUPlayerEnabled == 0)
            visuals->DrawTextOntoScreenBuffer("OFF", visuals->FontDefault[0]
                                              , 70, 185, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->CPUPlayerEnabled == 1)
            visuals->DrawTextOntoScreenBuffer("Slow Speed", visuals->FontDefault[0]
                                              , 70, 185, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->CPUPlayerEnabled == 2)
            visuals->DrawTextOntoScreenBuffer("Medium Speed", visuals->FontDefault[0]
                                              , 70, 185, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->CPUPlayerEnabled == 3)
            visuals->DrawTextOntoScreenBuffer("Fast Speed", visuals->FontDefault[0]
                                              , 70, 185, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->CPUPlayerEnabled == 4)
            visuals->DrawTextOntoScreenBuffer("Very Fast Speed", visuals->FontDefault[0]
                                              , 70, 185, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("A.I. Logic:", visuals->FontDefault[0]
                                          , 70, 230, JustifyLeft, 255, 255, 255, 0, 0, 0);
        if (game->AIlogic == 0)
            visuals->DrawTextOntoScreenBuffer("''JeZ+Lee'' - Easy", visuals->FontDefault[0]
                                              , 70, 230, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->AIlogic == 1)
            visuals->DrawTextOntoScreenBuffer("''mattmatteh'' - Hard", visuals->FontDefault[0]
                                              , 70, 230, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Full Screen Mode:", visuals->FontDefault[0]
                                          , 70, 275, JustifyLeft, 255, 255, 255, 0, 0, 0);
        if (game->FullScreenMode == false)
            visuals->DrawTextOntoScreenBuffer("OFF", visuals->FontDefault[0]
                                              , 70, 275, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (game->FullScreenMode == true)
            visuals->DrawTextOntoScreenBuffer("ON", visuals->FontDefault[0]
                                              , 70, 275, JustifyRight, 255, 255, 255, 0, 0, 0);

        if (input->JoystickDeviceOne != NULL)
        {
            visuals->DrawTextOntoScreenBuffer("Press [F1] On Keyboard To Setup Joystick(s)", visuals->FontDefault[0]
                                              , 0, 350, JustifyCenter, 255, 255, 255, 0, 0, 0);
        }

        visuals->DrawTextOntoScreenBuffer("Press [F2] On Keyboard To Setup Keyboard", visuals->FontDefault[0]
                                          , 0, 385, JustifyCenter, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        if (input->RotateSetupProcess > RotateSetupNotStarted)
        {
            visuals->ClearScreenBufferWithColor(0.0f, 0.0f, 0.0f, 1.0f);

            if (input->RotateSetupProcess == RotateSetupPressCounterClockwise)
                visuals->DrawTextOntoScreenBuffer("Press key on keyboard for rotate one now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);

            else if (input->RotateSetupProcess == RotateSetupPressClockwise)
                visuals->DrawTextOntoScreenBuffer("Press key on keyboard for rotate two now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);

            visuals->DrawTextOntoScreenBuffer("Press [F2] key on keyboard to quit setup.", visuals->FontDefault[0]
                                              , 0, 380, JustifyCenter, 200, 200, 200, 1, 1, 1);

            visuals->DrawTextOntoScreenBuffer("(Will setup keyboard with default mappings.)", visuals->FontDefault[0]
                                              , 0, 410, JustifyCenter, 200, 200, 200, 1, 1, 1);
        }

        if (input->JoystickSetupProcess > JoySetupNotStarted)
        {
            visuals->ClearScreenBufferWithColor(0.0f, 0.0f, 0.0f, 1.0f);

            if (input->JoystickSetupProcess == Joy1SetupPressUP)
                visuals->DrawTextOntoScreenBuffer("Press [UP] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy1SetupPressDOWN)
                visuals->DrawTextOntoScreenBuffer("Press [DOWN] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy1SetupPressLEFT)
                visuals->DrawTextOntoScreenBuffer("Press [LEFT] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy1SetupPressRIGHT)
                visuals->DrawTextOntoScreenBuffer("Press [RIGHT] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy1SetupPressBUTTONOne)
                visuals->DrawTextOntoScreenBuffer("Press [BUTTON1] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy1SetupPressBUTTONTwo)
                visuals->DrawTextOntoScreenBuffer("Press [BUTTON2] on joystick #1 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressUP)
                visuals->DrawTextOntoScreenBuffer("Press [UP] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressDOWN)
                visuals->DrawTextOntoScreenBuffer("Press [DOWN] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressLEFT)
                visuals->DrawTextOntoScreenBuffer("Press [LEFT] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressRIGHT)
                visuals->DrawTextOntoScreenBuffer("Press [RIGHT] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressBUTTONOne)
                visuals->DrawTextOntoScreenBuffer("Press [BUTTON1] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);
            else if (input->JoystickSetupProcess == Joy2SetupPressBUTTONTwo)
                visuals->DrawTextOntoScreenBuffer("Press [BUTTON2] on joystick #2 now!", visuals->FontDefault[0]
                                                  , 0, 214, JustifyCenter, 0, 255, 0, 1, 1, 1);

            visuals->DrawTextOntoScreenBuffer("Press [F1] key on keyboard to quit setup.", visuals->FontDefault[0]
                                              , 0, 380, JustifyCenter, 200, 200, 200, 1, 1, 1);

            visuals->DrawTextOntoScreenBuffer("(Will setup joystick(s) with default mappings.)", visuals->FontDefault[0]
                                              , 0, 410, JustifyCenter, 200, 200, 200, 1, 1, 1);
        }

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    if (input->KeyOnKeyboardPressedByUser == '@' && input->JoystickSetupProcess == JoySetupNotStarted)
    {
        ScreenMustBeDrawn = true;
        audio->PlayDigitalSoundFX(0, 0);

        if (input->RotateSetupProcess == RotateSetupNotStarted)
        {
            input->RotateSetupProcess = RotateSetupPressCounterClockwise;
        }
        else
        {
            input->RotateSetupProcess = RotateSetupNotStarted;

            input->UserDefinedRotateCounterClockwise = '^';
            input->UserDefinedRotateClockwise = '^';
        }
    }

    if (input->RotateSetupProcess == RotateSetupPressCounterClockwise
        && input->KeyOnKeyboardPressedByUser > '`' && input->KeyOnKeyboardPressedByUser < '{'
        && input->KeyOnKeyboardPressedByUser != 'z' && input->KeyOnKeyboardPressedByUser != 'x')
    {
        input->UserDefinedRotateCounterClockwise = input->KeyOnKeyboardPressedByUser;
        input->RotateSetupProcess = RotateSetupPressClockwise;

        audio->PlayDigitalSoundFX(0, 0);
        input->DelayAllUserInput = 20;
        ScreenMustBeDrawn = true;
    }
    else if (input->RotateSetupProcess == RotateSetupPressClockwise
        && input->KeyOnKeyboardPressedByUser > '`' && input->KeyOnKeyboardPressedByUser < '{')
    {
        input->UserDefinedRotateClockwise = input->KeyOnKeyboardPressedByUser;
        input->RotateSetupProcess = RotateSetupNotStarted;

        audio->PlayDigitalSoundFX(0, 0);
        input->DelayAllUserInput = 20;
        ScreenMustBeDrawn = true;
    }

    if ( input->KeyOnKeyboardPressedByUser == '!' && SDL_JoystickOpened(0)
        && input->RotateSetupProcess == RotateSetupNotStarted )
    {
        ScreenMustBeDrawn = true;
        audio->PlayDigitalSoundFX(0, 0);

        if (input->JoystickSetupProcess == JoySetupNotStarted)
        {
            input->JoystickSetupProcess = Joy1SetupPressUP;
        }
        else
        {
            input->JoystickSetupProcess = JoySetupNotStarted;

            for (int joy = 0; joy < 2; joy++)
            {
                input->JoyUP[joy] = Axis1;
                input->JoyDOWN[joy] = Axis1;
                input->JoyLEFT[joy] = Axis0;
                input->JoyRIGHT[joy] = Axis0;
                input->JoyButton1[joy] = Button0;
                input->JoyButton2[joy] = Button1;
            }
        }
    }

    Uint8 joyAction = 0;
    if (input->JoystickSetupProcess > Joy1SetupPressBUTTONTwo)
        joyAction = input->QueryJoysticksForAction(1);
    else if (input->JoystickSetupProcess > JoySetupNotStarted)
        joyAction = input->QueryJoysticksForAction(0);

    if (joyAction > 0)
    {
        if (input->JoystickSetupProcess == Joy1SetupPressUP)
        {
            input->JoyUP[0] = joyAction;
            input->JoystickSetupProcess = Joy1SetupPressDOWN;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy1SetupPressDOWN)
        {
            input->JoyDOWN[0] = joyAction;
            input->JoystickSetupProcess = Joy1SetupPressLEFT;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy1SetupPressLEFT)
        {
            input->JoyLEFT[0] = joyAction;
            input->JoystickSetupProcess = Joy1SetupPressRIGHT;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy1SetupPressRIGHT)
        {
            input->JoyRIGHT[0] = joyAction;
            input->JoystickSetupProcess = Joy1SetupPressBUTTONOne;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy1SetupPressBUTTONOne)
        {
            input->JoyButton1[0] = joyAction;
            input->JoystickSetupProcess = Joy1SetupPressBUTTONTwo;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy1SetupPressBUTTONTwo)
        {
            input->JoyButton2[0] = joyAction;

            if (input->JoystickDeviceTwo)
                input->JoystickSetupProcess = Joy2SetupPressUP;
            else
                input->JoystickSetupProcess = JoySetupNotStarted;

            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;
        }
        else if (input->JoystickSetupProcess == Joy2SetupPressUP)
        {
            input->JoyUP[1] = joyAction;
            input->JoystickSetupProcess = Joy2SetupPressDOWN;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy2SetupPressDOWN)
        {
            input->JoyDOWN[1] = joyAction;
            input->JoystickSetupProcess = Joy2SetupPressLEFT;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy2SetupPressLEFT)
        {
            input->JoyLEFT[1] = joyAction;
            input->JoystickSetupProcess = Joy2SetupPressRIGHT;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy2SetupPressRIGHT)
        {
            input->JoyRIGHT[1] = joyAction;
            input->JoystickSetupProcess = Joy2SetupPressBUTTONOne;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy2SetupPressBUTTONOne)
        {
            input->JoyButton1[1] = joyAction;
            input->JoystickSetupProcess = Joy2SetupPressBUTTONTwo;
            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;

        }
        else if (input->JoystickSetupProcess == Joy2SetupPressBUTTONTwo)
        {
            input->JoyButton2[1] = joyAction;

            input->JoystickSetupProcess = JoySetupNotStarted;

            audio->PlayDigitalSoundFX(0, 0);
            input->DelayAllUserInput = 20;
            ScreenMustBeDrawn = true;
        }
    }

    if (input->JoystickSetupProcess == JoySetupNotStarted && input->RotateSetupProcess == RotateSetupNotStarted)
    {
        interface->ProcessAllButtonsOnScreen();
        interface->ArrowSelected = -1;
        interface->ProcessAllArrowSetsOnScreen();

        if (interface->ArrowSelected == 0)
        {
            if (audio->MusicVolume > 0)  audio->MusicVolume-=32;
            else  audio->MusicVolume = 128;

            Mix_VolumeMusic(audio->MusicVolume);
        }
        else if (interface->ArrowSelected == 1)
        {
            if (audio->MusicVolume < 128)  audio->MusicVolume+=32;
            else  audio->MusicVolume = 0;

            Mix_VolumeMusic(audio->MusicVolume);
        }
        else if (interface->ArrowSelected == 2)
        {
            if (audio->SoundVolume > 0)  audio->SoundVolume-=32;
            else  audio->SoundVolume = 128;

            Mix_HaltChannel(-1);
            audio->PlayDigitalSoundFX(0, 0);
        }
        else if (interface->ArrowSelected == 3)
        {
            if (audio->SoundVolume < 128)  audio->SoundVolume+=32;
            else  audio->SoundVolume = 0;

            Mix_HaltChannel(-1);
            audio->PlayDigitalSoundFX(0, 0);
        }
        else if (interface->ArrowSelected == 2)
        {
            if (audio->SoundVolume > 0)  audio->SoundVolume-=32;
            else  audio->SoundVolume = 128;

            Mix_HaltChannel(-1);
            audio->PlayDigitalSoundFX(0, 0);
        }
        else if (interface->ArrowSelected == 3)
        {
            if (audio->SoundVolume < 128)  audio->SoundVolume+=32;
            else  audio->SoundVolume = 0;

            Mix_HaltChannel(-1);
            audio->PlayDigitalSoundFX(0, 0);
        }
        else if (interface->ArrowSelected == 4)
        {
            if (game->NetBookMode < 1)  game->NetBookMode = 1;
            else  game->NetBookMode = 0;
        }
        else if (interface->ArrowSelected == 5)
        {
            if (game->NetBookMode < 1)  game->NetBookMode = 1;
            else  game->NetBookMode = 0;
        }
        else if (interface->ArrowSelected == 6)
        {
            if (game->CPUPlayerEnabled > 0)  game->CPUPlayerEnabled--;
            else  game->CPUPlayerEnabled = 4;
        }
        else if (interface->ArrowSelected == 7)
        {
            if (game->CPUPlayerEnabled < 4)  game->CPUPlayerEnabled++;
            else  game->CPUPlayerEnabled = 0;
        }
        else if (interface->ArrowSelected == 8)
        {
            if (game->AIlogic > 0)  game->AIlogic--;
            else  game->AIlogic = 1;
        }
        else if (interface->ArrowSelected == 9)
        {
            if (game->AIlogic < 1)  game->AIlogic++;
            else  game->AIlogic = 0;
        }
        else if (interface->ArrowSelected == 10)
        {
            game->FullScreenMode = !game->FullScreenMode;

            visuals->UnloadAllTextsFromMemory();
            visuals->UnloadSpritesFromMemory();
            interface->UnloadGUIFromMemory();

            if (game->FullScreenMode == false)
            {
                visuals->Screen = SDL_SetVideoMode(visuals->ScreenWidth, visuals->ScreenHeight, 0
                                                   , SDL_OPENGL | SDL_RESIZABLE);
            }
            else
            {
                visuals->Screen = SDL_SetVideoMode(visuals->ScreenWidth, visuals->ScreenHeight
                                                   , 0, SDL_OPENGL | SDL_FULLSCREEN );
            }

            if (!visuals->Screen)
            {
                printf("ERROR: SDL+OpenGL window resize failed\n");
                game->CoreFailure = true;
                return;
            }
            else fprintf(stdout, "SDL+OpenGL window resized\n");

            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            glOrtho(0, 640, 480, 0, -1, 1);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
            glDisable(GL_DEPTH_TEST);

            glViewport(0, 0, visuals->ScreenWidth, visuals->ScreenHeight);

            if ( interface->CreateAllGUIAndInitialize(false) == false )
            {
                game->CoreFailure = true;
                return;
            }

            if ( visuals->LoadSpritesIntoMemoryAndInitialize() == false )
            {
                game->CoreFailure = true;
                return;
            }

            if ( visuals->LoadAllTextsIntoMemory() == false )
            {
                game->CoreFailure = true;
                return;
            }

            ScreenMustBeDrawn = true;
        }
        else if (interface->ArrowSelected == 11)
        {
            game->FullScreenMode = !game->FullScreenMode;

            visuals->UnloadAllTextsFromMemory();
            visuals->UnloadSpritesFromMemory();
            interface->UnloadGUIFromMemory();

            if (game->FullScreenMode == false)
            {
                visuals->Screen = SDL_SetVideoMode(visuals->ScreenWidth, visuals->ScreenHeight, 0
                                                   , SDL_OPENGL | SDL_RESIZABLE);
            }
            else
            {
                visuals->Screen = SDL_SetVideoMode(visuals->ScreenWidth, visuals->ScreenHeight
                                                   , 0, SDL_OPENGL | SDL_FULLSCREEN );
            }

            if (!visuals->Screen)
            {
                printf("ERROR: SDL+OpenGL window resize failed\n");
                game->CoreFailure = true;
                return;
            }
            else fprintf(stdout, "SDL+OpenGL window resized\n");

            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            glOrtho(0, 640, 480, 0, -1, 1);
            glMatrixMode(GL_MODELVIEW);
            glLoadIdentity();
            glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
            glDisable(GL_DEPTH_TEST);

            glViewport(0, 0, visuals->ScreenWidth, visuals->ScreenHeight);

            if ( interface->CreateAllGUIAndInitialize(false) == false )
            {
                game->CoreFailure = true;
                return;
            }

            if ( visuals->LoadSpritesIntoMemoryAndInitialize() == false )
            {
                game->CoreFailure = true;
                return;
            }

            if ( visuals->LoadAllTextsIntoMemory() == false )
            {
                game->CoreFailure = true;
                return;
            }

            ScreenMustBeDrawn = true;

        }
    }

//    if (game->AIlogic > 0)  game->AIlogic = 0;

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (interface->ButtonSelected == 0)
        {
            ScreenToDisplay = TitleScreen;
            input->DelayAllUserInput = 20;
        }

        interface->DestroyAllButtons();
        interface->DestroyAllArrowSets();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayNewGameScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(2, 0, 320, 458);
        interface->CreateArrowSet(0, 1, 50);
        interface->CreateArrowSet(2, 3, 100);
        interface->CreateArrowSet(4, 5, 150);
        interface->CreateArrowSet(6, 7, 200);
        interface->CreateArrowSet(8, 9, 250);
        interface->CreateArrowSet(10, 11, 300);
        interface->CreateArrowSet(12, 13, 350);

        interface->ButtonSelected = 0;
        interface->ArrowSetSelected = 0;

        logic->PlayerData[0].PlayerInput = -1;
        logic->PlayerData[1].PlayerInput = -1;
        logic->PlayerData[2].PlayerInput = -1;

        audio->PlayMusic(4+logic->SelectedMusicTrack, 0);

        ExitToTitleScreen = false;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[40+logic->SelectedBackground].ScreenX = 320;
        visuals->Sprites[40+logic->SelectedBackground].ScreenY = 240;
        visuals->Sprites[40+logic->SelectedBackground].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(40+logic->SelectedBackground);

        visuals->DrawTextOntoScreenBuffer("NEW GAME:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllArrowSetsOntoScreenBuffer();

        visuals->DrawTextOntoScreenBuffer("Game Mode:", visuals->FontDefault[0]
                                          , 70, 50, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->GameMode == OriginalMode)
        {
            visuals->DrawTextOntoScreenBuffer("Original Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }
        else if (logic->GameMode == TimeAttack30Mode)
        {
            visuals->DrawTextOntoScreenBuffer("Time Attack 30 Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }
        else if (logic->GameMode == TimeAttack60Mode)
        {
            visuals->DrawTextOntoScreenBuffer("Time Attack 60 Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }
        else if (logic->GameMode == TimeAttack120Mode)
        {
            visuals->DrawTextOntoScreenBuffer("Time Attack 120 Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }
        else if (logic->GameMode == TwentyLineChallengeMode)
        {
            visuals->DrawTextOntoScreenBuffer("20 Line Challenge Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }
        else if (logic->GameMode == CrisisMode)
        {
            visuals->DrawTextOntoScreenBuffer("Crisis+Mode", visuals->FontDefault[0]
                                              , 70, 50, JustifyRight, 255, 255, 255, 0, 0, 0);
        }

        visuals->DrawTextOntoScreenBuffer("Music Track:", visuals->FontDefault[0]
                                          , 70, 100, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->SelectedMusicTrack == 0)
            visuals->DrawTextOntoScreenBuffer("Track-01", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 1)
            visuals->DrawTextOntoScreenBuffer("Track-02", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 2)
            visuals->DrawTextOntoScreenBuffer("Track-03", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 3)
            visuals->DrawTextOntoScreenBuffer("Track-04", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 4)
            visuals->DrawTextOntoScreenBuffer("Track-05", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 5)
            visuals->DrawTextOntoScreenBuffer("Track-06", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 6)
            visuals->DrawTextOntoScreenBuffer("Track-07", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 7)
            visuals->DrawTextOntoScreenBuffer("Track-08", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 8)
            visuals->DrawTextOntoScreenBuffer("Track-09", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 9)
            visuals->DrawTextOntoScreenBuffer("Track-10", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 10)
            visuals->DrawTextOntoScreenBuffer("Track-11", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 11)
            visuals->DrawTextOntoScreenBuffer("Track-12", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 12)
            visuals->DrawTextOntoScreenBuffer("Track-13", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 13)
            visuals->DrawTextOntoScreenBuffer("Track-14", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 14)
            visuals->DrawTextOntoScreenBuffer("Track-15", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 15)
            visuals->DrawTextOntoScreenBuffer("Bonus Track-16", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 16)
            visuals->DrawTextOntoScreenBuffer("Bonus Track-17", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 17)
            visuals->DrawTextOntoScreenBuffer("Bonus Track-18", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 18)
            visuals->DrawTextOntoScreenBuffer("Bonus Track-19", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedMusicTrack == 19)
            visuals->DrawTextOntoScreenBuffer("Bonus Track-20", visuals->FontDefault[0]
                                              , 70, 100, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Background:", visuals->FontDefault[0]
                                          , 70, 150, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->SelectedBackground == 0)
            visuals->DrawTextOntoScreenBuffer("Saint Basil's Cathedral", visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedBackground == 1)
            visuals->DrawTextOntoScreenBuffer("MIG-31 Russian Fighter", visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedBackground == 2)
            visuals->DrawTextOntoScreenBuffer("New York City", visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedBackground == 3)
            visuals->DrawTextOntoScreenBuffer("Van Gogh Painting", visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedBackground == 4)
            visuals->DrawTextOntoScreenBuffer("Beach Sunset", visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->SelectedBackground == 5)
        {
            char *reg = new char[2];
            sprintf(reg, "%c", 0xAE);
            strcpy(visuals->VariableText, "Nissan");
            strcat(visuals->VariableText, reg);
            strcat(visuals->VariableText, " GT-R");
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0]
                                              , 70, 150, JustifyRight, 255, 255, 255, 0, 0, 0);
        }

        visuals->DrawTextOntoScreenBuffer("Pressing [UP]:", visuals->FontDefault[0]
                                          , 70, 200, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->PressingUPAction == 0)
            visuals->DrawTextOntoScreenBuffer("Off", visuals->FontDefault[0]
                                              , 70, 200, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->PressingUPAction == 1)
            visuals->DrawTextOntoScreenBuffer("Fast Piece Drop", visuals->FontDefault[0]
                                              , 70, 200, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->PressingUPAction == 2)
            visuals->DrawTextOntoScreenBuffer("Rotate Piece", visuals->FontDefault[0]
                                              , 70, 200, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Display Next Piece:", visuals->FontDefault[0]
                                          , 70, 250, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->DisplayNextPiece == false)
            visuals->DrawTextOntoScreenBuffer("No", visuals->FontDefault[0]
                                              , 70, 250, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->DisplayNextPiece == true)
            visuals->DrawTextOntoScreenBuffer("Yes", visuals->FontDefault[0]
                                              , 70, 250, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Display Drop Shadow:", visuals->FontDefault[0]
                                          , 70, 300, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (logic->DisplayDropShadow == false)
            visuals->DrawTextOntoScreenBuffer("No", visuals->FontDefault[0]
                                              , 70, 300, JustifyRight, 255, 255, 255, 0, 0, 0);
        else if (logic->DisplayDropShadow == true)
            visuals->DrawTextOntoScreenBuffer("Yes", visuals->FontDefault[0]
                                              , 70, 300, JustifyRight, 255, 255, 255, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Box Tileset:", visuals->FontDefault[0]
                                          , 70, 350, JustifyLeft, 255, 255, 255, 0, 0, 0);

        if (game->NetBookMode == true)
            visuals->DrawTextOntoScreenBuffer("Not Available", visuals->FontDefault[0]
                                              , 70, 350, JustifyRight, 255, 255, 255, 0, 0, 0);
        else
        {
            GLfloat boxX = 853;
            GLfloat boxY = 483;
            for (int index = 2; index < 9; index++)
            {
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScreenX = boxX;
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScreenY = boxY;
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScaleX = 2.0;
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScaleY = 2.0;
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].Transparency = 1.0f;
                visuals->DrawSpriteOntoScreenBuffer(50 + (10*game->BoxTileSheet)+index);

                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScaleX = 1.0;
                visuals->Sprites[50 + (10*game->BoxTileSheet)+index].ScaleY = 1.0;

                boxX+=33;
            }
        }

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();
    interface->ArrowSelected = -1;
    interface->ProcessAllArrowSetsOnScreen();

    if (interface->ArrowSelected == 0)
    {
        if (logic->GameMode > 0)  logic->GameMode--;
        else  logic->GameMode = 5;
    }
    else if (interface->ArrowSelected == 1)
    {
        if (logic->GameMode < 5)  logic->GameMode++;
        else  logic->GameMode = 0;
    }
    else if (interface->ArrowSelected == 2)
    {
        if (logic->SelectedMusicTrack > 0)  logic->SelectedMusicTrack--;
        else  logic->SelectedMusicTrack = 18;

        audio->PlayMusic(4+logic->SelectedMusicTrack, 0);
    }
    else if (interface->ArrowSelected == 3)
    {
        if (logic->SelectedMusicTrack < 18)  logic->SelectedMusicTrack++;
        else  logic->SelectedMusicTrack = 0;

        audio->PlayMusic(4+logic->SelectedMusicTrack, 0);
    }
    else if (interface->ArrowSelected == 4)
    {
        if (logic->SelectedBackground > 0)  logic->SelectedBackground--;
        else  logic->SelectedBackground = 5;
    }
    else if (interface->ArrowSelected == 5)
    {
        if (logic->SelectedBackground < 5)  logic->SelectedBackground++;
        else  logic->SelectedBackground = 0;
    }
    else if (interface->ArrowSelected == 6)
    {
        if (logic->PressingUPAction > 0)  logic->PressingUPAction--;
        else  logic->PressingUPAction = 2;
    }
    else if (interface->ArrowSelected == 7)
    {
        if (logic->PressingUPAction < 2)  logic->PressingUPAction++;
        else  logic->PressingUPAction = 0;
    }
    else if (interface->ArrowSelected == 8)
    {
        if (logic->DisplayNextPiece == true)  logic->DisplayNextPiece = false;
        else  logic->DisplayNextPiece = true;
    }
    else if (interface->ArrowSelected == 9)
    {
        if (logic->DisplayNextPiece == true)  logic->DisplayNextPiece = false;
        else  logic->DisplayNextPiece = true;
    }
    else if (interface->ArrowSelected == 10)
    {
        if (logic->DisplayDropShadow == true)  logic->DisplayDropShadow = false;
        else  logic->DisplayDropShadow = true;
    }
    else if (interface->ArrowSelected == 11)
    {
        if (logic->DisplayDropShadow == true)  logic->DisplayDropShadow = false;
        else  logic->DisplayDropShadow = true;
    }
    else if (interface->ArrowSelected == 12 && game->NetBookMode == false)
    {
        if (game->BoxTileSheet > 0)  game->BoxTileSheet--;
        else  game->BoxTileSheet = 5;
    }
    else if (interface->ArrowSelected == 13 && game->NetBookMode == false)
    {
        if (game->BoxTileSheet < 5)  game->BoxTileSheet++;
        else  game->BoxTileSheet = 0;
    }

    if (input->JoystickButtonOne[Keyboard] == ON)
    {
        logic->PlayerData[0].PlayerInput = -1;
        logic->PlayerData[1].PlayerInput = Keyboard;
        logic->PlayerData[2].PlayerInput = -1;
    }
    else if (input->JoystickButtonOne[JoystickOne] == ON)
    {
        logic->PlayerData[0].PlayerInput = -1;
        logic->PlayerData[1].PlayerInput = JoystickOne;
        logic->PlayerData[2].PlayerInput = -1;
    }
    else if (input->JoystickButtonOne[JoystickTwo] == ON)
    {
        logic->PlayerData[0].PlayerInput = -1;
        logic->PlayerData[1].PlayerInput = JoystickTwo;
        logic->PlayerData[2].PlayerInput = -1;
    }

    if (game->Debug == 2 && input->JoyButton1[Any] == ON)
        ScreenTransitionStatus = FadeOut;

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (ExitToTitleScreen == false)
        {
            if (interface->ButtonSelected == 0)
            {
                ScreenToDisplay = PlayingGameScreen;

                if (logic->PlayerData[0].PlayerInput == -1 && logic->PlayerData[1].PlayerInput == -1 && logic->PlayerData[2].PlayerInput == -1)
                {
                    logic->PlayerData[0].PlayerInput = -1;
                    logic->PlayerData[1].PlayerInput = Keyboard;
                    logic->PlayerData[2].PlayerInput = -1;
                }

                logic->SetupForNewGame();

                if (logic->ThinkRussianTimer == 1)
                {
                    audio->PlayMusic(4+logic->SelectedMusicTrack, 0);
                    logic->ThinkRussianTimer--;
                }
                else  Mix_HaltMusic();

                if (game->Debug != 3)  game->FrameLock = 27;
                else  game->FrameLock = 7;

                input->DelayAllUserInput = 20;
            }
        }
        else
        {
            ScreenToDisplay = TitleScreen;
            audio->PlayMusic(0 ,0);
        }

        interface->DestroyAllButtons();
        interface->DestroyAllArrowSets();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayPlayingGameScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        ExitToTitleScreen = false;
    }

    if (logic->ThinkRussianTimer == 1)
    {
        audio->PlayMusic(4+logic->SelectedMusicTrack, 0);
        logic->ThinkRussianTimer--;
    }

    logic->RunTetriGameEngine();

    if (logic->GameDisplayChanged == true)
    {
        for (logic->Player = 0; logic->Player < NumberOfPlayers; logic->Player++)
        {
            if (logic->PlayerData[logic->Player].PlayerStatus != FlashingCompletedLines
                && logic->PlayerData[logic->Player].PlayerStatus != ClearingCompletedLines)
            {
                logic->DeletePieceFromPlayfieldMemory(Current);
                logic->AddPieceToPlayfieldMemory(DropShadow);
                logic->AddPieceToPlayfieldMemory(Current);

                ScreenMustBeDrawn = true;
            }
        }
    }

    if (ScreenMustBeDrawn == true && game->Debug != 2)
    {
        visuals->Sprites[40+logic->SelectedBackground].ScreenX = 320;
        visuals->Sprites[40+logic->SelectedBackground].ScreenY = 240;
        visuals->DrawSpriteOntoScreenBuffer(40+logic->SelectedBackground);

        for (int player = 0; player < NumberOfPlayers; player++)
        {
            sprintf(visuals->VariableText, "%d", logic->PlayerData[player].Score);
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 30,
                                              JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);

            sprintf(visuals->VariableText, "%d", logic->PlayerData[player].Lines);
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[5], logic->PlayerData[player].PlayersPlayfieldScreenX-75, 77,
                                              JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);

            sprintf(visuals->VariableText, "%d", logic->PlayerData[player].Level);
            visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[5], logic->PlayerData[player].PlayersPlayfieldScreenX+75, 77,
                                              JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);

            visuals->Sprites[300].ScreenX = logic->PlayerData[player].PlayersPlayfieldScreenX;
            visuals->Sprites[300].ScreenY = logic->PlayerData[player].PlayersPlayfieldScreenY;

            switch (player)
            {
                case 0:
                    visuals->Sprites[300].redHue = 1.0f;
                    visuals->Sprites[300].greenHue = 0.0f;
                    visuals->Sprites[300].blueHue = 0.0f;
                    break;

                case 1:
                    visuals->Sprites[300].redHue = 0.0f;
                    visuals->Sprites[300].greenHue = 1.0f;
                    visuals->Sprites[300].blueHue = 0.0f;
                    break;

                case 2:
                    visuals->Sprites[300].redHue = 1.0f;
                    visuals->Sprites[300].greenHue = 1.0f;
                    visuals->Sprites[300].blueHue = 0.0f;
                    break;

                default:
                    break;
            }

            if (game->NetBookMode == false)  visuals->Sprites[300].Transparency = 0.75f;
            else  visuals->Sprites[300].Transparency = 1.0f;

            visuals->DrawSpriteOntoScreenBuffer(300);
        }

        for (int player = 0; player < NumberOfPlayers; player++)
        {
            float boxScreenX = logic->PlayerData[player].PlayersPlayfieldScreenX+176;
            float boxScreenY = logic->PlayerData[player].PlayersPlayfieldScreenY-144;

            for (int y = 0; y < 26; y++)
            {
                for (int x = 2; x < 12; x++)
                {
                    if (logic->PlayerData[player].Playfield[x][y] == 1)
                    {
                        visuals->Sprites[51].ScreenX = boxScreenX;
                        visuals->Sprites[51].ScreenY = boxScreenY;
                        visuals->Sprites[51].Transparency = 0.50f;
                        visuals->DrawSpriteOntoScreenBuffer(51);
                    }
                    else if (logic->PlayerData[player].Playfield[x][y] > 10 && logic->PlayerData[player].Playfield[x][y] < 20)
                    {
                        int spriteIndex = 50 + (10*game->BoxTileSheet);
                        if (game->NetBookMode == true)  spriteIndex = 60;

                        visuals->Sprites[spriteIndex-9+logic->PlayerData[player].Playfield[x][y]].ScreenX = boxScreenX;
                        visuals->Sprites[spriteIndex-9+logic->PlayerData[player].Playfield[x][y]].ScreenY = boxScreenY;
                        visuals->Sprites[spriteIndex-9+logic->PlayerData[player].Playfield[x][y]].Transparency = 1.0f;
                        visuals->DrawSpriteOntoScreenBuffer(spriteIndex-9+logic->PlayerData[player].Playfield[x][y]);

                    }
                    else if (logic->PlayerData[player].Playfield[x][y] > 100 && logic->PlayerData[player].Playfield[x][y] < 110)
                    {
                        int offset = 51 + (10*game->BoxTileSheet);
                        if (game->NetBookMode == true)  offset = 61;

                        int spriteIndex = logic->PlayerData[player].Playfield[x][y] - 100 + offset;
                        visuals->Sprites[spriteIndex].ScreenX = boxScreenX;
                        visuals->Sprites[spriteIndex].ScreenY = boxScreenY;
                        visuals->Sprites[spriteIndex].Transparency = 1.0f;
                        visuals->DrawSpriteOntoScreenBuffer(spriteIndex);
                    }
                    else if (logic->PlayerData[player].Playfield[x][y] > 20 && logic->PlayerData[player].Playfield[x][y] < 30)
                    {
                        visuals->Sprites[51].ScreenX = boxScreenX;
                        visuals->Sprites[51].ScreenY = boxScreenY;
                        visuals->Sprites[51].Transparency = 1.0f;
                        visuals->DrawSpriteOntoScreenBuffer(51);
                    }

                    boxScreenX+=16;
                }

                boxScreenX = logic->PlayerData[player].PlayersPlayfieldScreenX+176;
                boxScreenY+=16;
            }
        }

        for (int player = 0; player < NumberOfPlayers; player++)
        {
            if (logic->PlayerData[player].PlayerStatus == GameOver)
            {
                visuals->DrawTextOntoScreenBuffer("Game Over", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 240,
                                                  JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);

                if (logic->PlayersCanJoin == true)
                {

                    if ( (logic->PlayerData[player].PlayerInput == JoystickOne && input->JoystickDeviceOne != NULL)
                        || (logic->PlayerData[player].PlayerInput == JoystickTwo && input->JoystickDeviceTwo != NULL)
                        || (logic->PlayerData[player].PlayerInput == Keyboard) )
                    {
                        visuals->DrawTextOntoScreenBuffer("JOIN IN!", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 270,
                                                          JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
                    }
                }
            }

            if (logic->GameMode == TimeAttack30Mode || logic->GameMode == TimeAttack60Mode || logic->GameMode == TimeAttack120Mode)
            {
                Uint32 taTimer = logic->TimeAttackTimer / 200;
                sprintf(visuals->VariableText, "%d", taTimer);
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[2], logic->PlayerData[player].PlayersPlayfieldScreenX
                                                  , 427, JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
            }
            else if (logic->GameMode == TwentyLineChallengeMode)
            {
                sprintf(visuals->VariableText, "%d", logic->PlayerData[player].TwentyLineCounter);
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[2], logic->PlayerData[player].PlayersPlayfieldScreenX
                                                  , 427, JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
            }

            if (game->NetBookMode == false)
            {
                if (logic->PlayerData[player].PlayerInput == JoystickOne && input->JoystickDeviceOne != NULL)
                    visuals->DrawTextOntoScreenBuffer("Joystick #1", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 448,
                                                      JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
                else if (logic->PlayerData[player].PlayerInput == Keyboard)
                    visuals->DrawTextOntoScreenBuffer("Keyboard", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 448,
                                                      JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
                else if (logic->PlayerData[player].PlayerInput == JoystickTwo && input->JoystickDeviceOne != NULL)
                    visuals->DrawTextOntoScreenBuffer("Joystick #2", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 448,
                                                      JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
                else if (logic->PlayerData[player].PlayerInput == CPU)
                    visuals->DrawTextOntoScreenBuffer("C.P.U.", visuals->FontDefault[4], logic->PlayerData[player].PlayersPlayfieldScreenX, 448,
                                                      JustifyCenterOnPoint, 255, 255, 255, 1, 1, 1);
            }
        }

        if (logic->GameMode == CrisisMode && logic->Crisis7BGMPlayed == true)
        {
            visuals->Sprites[310].ScreenX = 100;
            visuals->Sprites[310].ScreenY = 240;
            visuals->Sprites[310].Transparency = 0.50f;
            visuals->DrawSpriteOntoScreenBuffer(310);

            visuals->Sprites[310].ScreenX = 320;
            visuals->Sprites[310].ScreenY = 240;
            visuals->Sprites[310].Transparency = 0.50f;
            visuals->DrawSpriteOntoScreenBuffer(310);

            visuals->Sprites[310].ScreenX = 540;
            visuals->Sprites[310].ScreenY = 240;
            visuals->Sprites[310].Transparency = 0.50f;
            visuals->DrawSpriteOntoScreenBuffer(310);
        }

        bool humanStillAlive = false;
        if (logic->PlayerData[0].PlayerInput != CPU && logic->PlayerData[0].PlayerStatus != GameOver)  humanStillAlive = true;
        if (logic->PlayerData[1].PlayerInput != CPU && logic->PlayerData[1].PlayerStatus != GameOver)  humanStillAlive = true;
        if (logic->PlayerData[2].PlayerInput != CPU && logic->PlayerData[2].PlayerStatus != GameOver)  humanStillAlive = true;

        if (humanStillAlive == false && game->Debug == 0)
            visuals->DrawTextOntoScreenBuffer("Press [Esc] On Keyboard To Exit...", visuals->FontDefault[0], 320, 260,
                                              JustifyCenter, 255, 255, 255, 1, 1, 1);

        if (logic->PAUSEgame == true && game->Debug != 4)
        {
            for (GLfloat posY = -64; posY < 480+64; posY+=64)
            {
                for (GLfloat posX = -64; posX < 640+64; posX+=64)
                {
                    visuals->Sprites[1].ScreenX = posX;
                    visuals->Sprites[1].ScreenY = posY;
                    visuals->Sprites[1].Transparency = 0.90f;
                    visuals->DrawSpriteOntoScreenBuffer(1);
                }
            }

            visuals->DrawTextOntoScreenBuffer("P A U S E D", visuals->FontDefault[0], 320, 220,
                                              JustifyCenter, 255, 255, 255, 1, 1, 1);

            visuals->DrawTextOntoScreenBuffer("(Press [Spacebar] To Continue!)", visuals->FontDefault[0], 320, 260,
                                              JustifyCenter, 255, 255, 255, 1, 1, 1);
        }

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    for (int player = 0; player < NumberOfPlayers; player++)
    {
        if (input->JoystickButtonOne[logic->PlayerData[player].PlayerInput] == ON  && logic->PlayersCanJoin == true
            && logic->PlayerData[player].PlayerStatus == GameOver)  logic->PlayerData[player].PlayerStatus = NewPieceDropping;
    }

    if (game->Debug == 2)  game->FrameLock = 1;

    if (logic->PlayerData[0].PlayerStatus == GameOver
     && logic->PlayerData[1].PlayerStatus == GameOver
     && logic->PlayerData[2].PlayerStatus == GameOver)
    {
        game->FrameLock = 17;
        ScreenTransitionStatus = FadeOut;
    }

    if (logic->GameDisplayChanged == true)
    {
        for (logic->Player = 0; logic->Player < NumberOfPlayers; logic->Player++)
        {
            if (logic->PlayerData[logic->Player].PlayerStatus != FlashingCompletedLines
                && logic->PlayerData[logic->Player].PlayerStatus != ClearingCompletedLines)
            {
                logic->DeletePieceFromPlayfieldMemory(Current);
                logic->DeletePieceFromPlayfieldMemory(DropShadow);
            }
        }

        logic->GameDisplayChanged = false;
    }

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (ExitToTitleScreen == false)
        {
            if (logic->CrisisWon == true)
            {
                ScreenToDisplay = AboutScreen;
                return;
            }

            data->CheckForNewHighScore();
            data->NameInputArayIndex = 0;

            if (data->PlayerRankOnGameOver < 10)
            {
                if (logic->PlayerData[data->PlayerWithHighestScore].PlayerInput == Keyboard)  ScreenToDisplay = NameInputKeyboardScreen;
                else  ScreenToDisplay = NameInputJoystickScreen;
            }
            else
            {
                ScreenToDisplay = HighScoresScreen;
            }

            if (logic->CrisisWon == false)
            {
                audio->PlayMusic(2, 1);
            }
        }
        else
        {
            ScreenToDisplay = TitleScreen;
            audio->PlayMusic(0, 0);
        }

        ScreenMustBeDrawn = true;

        interface->DestroyAllButtons();
        ScreenTransitionStatus = FadeAll;

        if (logic->PlayerData[0].PlayerInput == CPU)
        {
            logic->TotalCPUPlayerLines += logic->PlayerData[0].Lines;
            logic->NumberofCPUGames++;
        }

        if (logic->PlayerData[1].PlayerInput == CPU)
        {
            logic->TotalCPUPlayerLines += logic->PlayerData[1].Lines;
            logic->NumberofCPUGames++;
        }

        if (logic->PlayerData[2].PlayerInput == CPU)
        {
            logic->TotalCPUPlayerLines += logic->PlayerData[2].Lines;
            logic->NumberofCPUGames++;
        }
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayNameInputKeyboardScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(8, 0, 320, 458);

        interface->ButtonSelected = 0;

        ExitToTitleScreen = false;

        if (logic->PlayerData[data->PlayerWithHighestScore].PlayerInput == CPU)  ScreenTransitionStatus = FadeOut;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->DrawTextOntoScreenBuffer("NEW HIGH SCORE:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("You have achieved a new high score!", visuals->FontDefault[0],
                                          0, 80, JustifyCenter, 255, 255, 255, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Please enter your name using the keyboard:", visuals->FontDefault[0],
                                          0, 125, JustifyCenter, 255, 255, 255, 1, 1, 1);

        if (data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] != (char)NULL)
            visuals->DrawTextOntoScreenBuffer(data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver],
                                              visuals->FontDefault[3], 0, 182, JustifyCenter, 255, 255, 0, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Press [Backspace] key on keyboard to erase.", visuals->FontDefault[0],
                                          0, 305, JustifyCenter, 255, 255, 255, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("Press [Enter] key on keyboard when finished.", visuals->FontDefault[0],
                                          0, 345, JustifyCenter, 255, 255, 255, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();

    if (input->KeyOnKeyboardPressedByUser != (char)NULL)
    {
        if (input->KeyOnKeyboardPressedByUser == '/')
        {

        }
        else if (input->KeyOnKeyboardPressedByUser == '<')
        {
            if (data->NameInputArayIndex == 19)
            {
                data->NameInputArayIndex--;
                data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
            }

            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
            if (data->NameInputArayIndex > 0)  data->NameInputArayIndex--;
            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
        }
        else if (input->KeyOnKeyboardPressedByUser == '_')
        {
            if (data->NameInputArayIndex < 18)
                data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex]
                    = ' ';

            if (data->NameInputArayIndex < 19)  data->NameInputArayIndex++;
        }
        else
        {
            if (data->NameInputArayIndex < 18)
                data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex]
                    = input->KeyOnKeyboardPressedByUser;

            if (data->NameInputArayIndex < 19)  data->NameInputArayIndex++;
        }

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
    }

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        if (interface->ButtonSelected == 0)
        {
            ScreenToDisplay = HighScoresScreen;
            input->DelayAllUserInput = 20;
        }

        if (ExitToTitleScreen == true)  ScreenToDisplay = TitleScreen;

        if (data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] == '\0')
            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] = ' ';

        interface->DestroyAllButtons();
        ScreenTransitionStatus = FadeAll;
    }
}

//-------------------------------------------------------------------------------------------------
void Screens::DisplayNameInputJoystickScreen(void)
{
    if (ScreenTransitionStatus == FadeAll)
    {
        ScreenTransitionStatus = FadeIn;
        ScreenFadeTransparency = 1.0f;

        interface->CreateButton(8, 0, 320, 458);

        interface->ButtonSelected = 0;

        ExitToTitleScreen = false;

        if (logic->PlayerData[data->PlayerWithHighestScore].PlayerInput == CPU)  ScreenTransitionStatus = FadeOut;
    }

    if (ScreenMustBeDrawn == true)
    {
        visuals->ClearScreenBufferWithColor(0, 0, 0, 255);

        visuals->Sprites[20].ScreenX = 320;
        visuals->Sprites[20].ScreenY = 240;
        visuals->Sprites[20].Transparency = 0.6f;
        visuals->DrawSpriteOntoScreenBuffer(20);

        visuals->DrawTextOntoScreenBuffer("NEW HIGH SCORE:", visuals->FontDefault[0]
                                          , 0, 1, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 7, JustifyCenter, 255, 255, 0, 0, 0, 0);

        visuals->DrawTextOntoScreenBuffer("Congratulations!", visuals->FontDefault[0],
                                          0, 50, JustifyCenter, 255, 255, 255, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("You have achieved a new high score!", visuals->FontDefault[0],
                                          0, 80, JustifyCenter, 255, 255, 255, 1, 1, 1);

        if (logic->PlayerData[data->PlayerWithHighestScore].PlayerInput == JoystickOne)
            visuals->DrawTextOntoScreenBuffer("Please enter your name using joystick #1:", visuals->FontDefault[0],
                                              0, 125, JustifyCenter, 255, 255, 255, 1, 1, 1);
        else
            visuals->DrawTextOntoScreenBuffer("Please enter your name using joystick #2:", visuals->FontDefault[0],
                                              0, 125, JustifyCenter, 255, 255, 255, 1, 1, 1);

        if (data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] != (char)NULL)
            visuals->DrawTextOntoScreenBuffer(data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver],
                                              visuals->FontDefault[3], 0, 182, JustifyCenter, 255, 255, 0, 1, 1, 1);

        GLfloat x = 45;
        for (char alphaNumeral = 'A'; alphaNumeral < 'N'; alphaNumeral++)
        {
            sprintf(visuals->VariableText, "%c", alphaNumeral);

            if (alphaNumeral == data->NameInputJoyChar)
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 239, JustifyCenterOnPoint,
                                                  255, 1, 1, 1, 1, 1);
            else
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 239, JustifyCenterOnPoint,
                                                  255, 255, 255, 1, 1, 1);
            x+=45;
        }

        x = 45;
        for (char alphaNumeral = 'N'; alphaNumeral < '['; alphaNumeral++)
        {
            sprintf(visuals->VariableText, "%c", alphaNumeral);

            if (alphaNumeral == data->NameInputJoyChar)
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 269, JustifyCenterOnPoint,
                                                  255, 1, 1, 1, 1, 1);
            else
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 269, JustifyCenterOnPoint,
                                                  255, 255, 255, 1, 1, 1);
            x+=45;
        }

        x = 45;
        for (char alphaNumeral = 'a'; alphaNumeral < 'n'; alphaNumeral++)
        {
            sprintf(visuals->VariableText, "%c", alphaNumeral);

            if (alphaNumeral == data->NameInputJoyChar)
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 299, JustifyCenterOnPoint,
                                                  255, 1, 1, 1, 1, 1);
            else
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 299, JustifyCenterOnPoint,
                                                  255, 255, 255, 1, 1, 1);
            x+=45;
        }

        x = 45;
        for (char alphaNumeral = 'n'; alphaNumeral < '{'; alphaNumeral++)
        {
            sprintf(visuals->VariableText, "%c", alphaNumeral);

            if (alphaNumeral == data->NameInputJoyChar)
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 329, JustifyCenterOnPoint,
                                                  255, 1, 1, 1, 1, 1);
            else
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 329, JustifyCenterOnPoint,
                                                  255, 255, 255, 1, 1, 1);
            x+=45;
        }

        x = 45;
        for (char alphaNumeral = '0'; alphaNumeral < ':'; alphaNumeral++)
        {
            sprintf(visuals->VariableText, "%c", alphaNumeral);

            if (alphaNumeral == data->NameInputJoyChar)
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                                  255, 1, 1, 1, 1, 1);
            else
                visuals->DrawTextOntoScreenBuffer(visuals->VariableText, visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                                  255, 255, 255, 1, 1, 1);
            x+=45;
        }

        if (data->NameInputJoyChar == '+')
            visuals->DrawTextOntoScreenBuffer("+", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 1, 1, 1, 1, 1);
        else
            visuals->DrawTextOntoScreenBuffer("+", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 255, 255, 1, 1, 1);
        x+=45;

        if (data->NameInputJoyChar == '_')
            visuals->DrawTextOntoScreenBuffer("_", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 1, 1, 1, 1, 1);
        else
            visuals->DrawTextOntoScreenBuffer("_", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 255, 255, 1, 1, 1);
        x+=45;

        if (data->NameInputJoyChar == '<')
            visuals->DrawTextOntoScreenBuffer("<", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 1, 1, 1, 1, 1);
        else
            visuals->DrawTextOntoScreenBuffer("<", visuals->FontDefault[0], x, 359, JustifyCenterOnPoint,
                                              255, 255, 255, 1, 1, 1);
        x+=45;

        if (data->NameInputJoyCharY == 5)
            visuals->DrawTextOntoScreenBuffer("END", visuals->FontDefault[0], 320, 389, JustifyCenterOnPoint,
                                              255, 1, 1, 1, 1, 1);
        else
            visuals->DrawTextOntoScreenBuffer("END", visuals->FontDefault[0], 320, 389, JustifyCenterOnPoint,
                                              255, 255, 255, 1, 1, 1);

        visuals->DrawTextOntoScreenBuffer("______________________________________", visuals->FontDefault[0]
                                          , 0, 400, JustifyCenter, 255, 255, 0, 0, 0, 0);

        interface->DisplayAllButtonsOntoScreenBuffer();

        ScreenMustBeDrawn = false;
        ScreenIsDirty = true;
    }

    interface->ProcessAllButtonsOnScreen();

    if (input->JoystickButtonOne[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == ON)
    {
        if (input->JoystickButton1Pressed[JoystickOne] == false)
        {
            if (data->NameInputJoyChar != (char)NULL)
            {
                if (data->NameInputJoyCharY == 5)
                {
                    ScreenTransitionStatus = FadeOut;
                }
                else if (data->NameInputJoyChar == '<')
                {
                    if (data->NameInputArayIndex == 19)
                    {
                        data->NameInputArayIndex--;
                        data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
                    }

                    data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
                    if (data->NameInputArayIndex > 0)  data->NameInputArayIndex--;
                    data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
                }
                else if (data->NameInputJoyChar == '_')
                {
                    if (data->NameInputArayIndex < 18)
                        data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex]
                            = ' ';

                    if (data->NameInputArayIndex < 19)  data->NameInputArayIndex++;
                }
                else if (data->NameInputJoyCharY != 5)
                {
                    if (data->NameInputArayIndex < 18)
                        data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex]
                            = data->NameInputJoyChar;

                    if (data->NameInputArayIndex < 19)  data->NameInputArayIndex++;
                }

                audio->PlayDigitalSoundFX(0, 0);
                ScreenMustBeDrawn = true;
                input->JoystickButton1Pressed[JoystickOne] = true;
            }
        }
    }
    else if (input->JoystickButtonTwo[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == ON)
    {
        if (input->JoystickButton2Pressed[JoystickOne] == false)
        {
            if (data->NameInputArayIndex == 19)
            {
                data->NameInputArayIndex--;
                data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
            }

            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';
            if (data->NameInputArayIndex > 0)  data->NameInputArayIndex--;
            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][data->NameInputArayIndex] = '\0';

            audio->PlayDigitalSoundFX(0, 0);
            ScreenMustBeDrawn = true;
            input->JoystickButton2Pressed[JoystickOne] = true;
        }
    }
    else
    {
        input->JoystickButton1Pressed[JoystickOne] = false;
        input->JoystickButton2Pressed[JoystickOne] = false;
    }

    if (input->JoystickDirection[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == UP)
    {
        if (data->NameInputJoyCharY > 0)  data->NameInputJoyCharY--;
        else  data->NameInputJoyCharY = 5;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
        input->DelayAllUserInput = 6;
    }
    else if (input->JoystickDirection[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == DOWN)
    {
        if (data->NameInputJoyCharY < 5)  data->NameInputJoyCharY++;
        else  data->NameInputJoyCharY = 0;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
        input->DelayAllUserInput = 6;
    }
    else if (input->JoystickDirection[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == LEFT)
    {
        if (data->NameInputJoyCharX > 0)  data->NameInputJoyCharX--;
        else  data->NameInputJoyCharX = 12;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
        input->DelayAllUserInput = 6;
    }
    else if (input->JoystickDirection[logic->PlayerData[data->PlayerWithHighestScore].PlayerInput] == RIGHT)
    {
        if (data->NameInputJoyCharX < 12)  data->NameInputJoyCharX++;
        else  data->NameInputJoyCharX = 0;

        audio->PlayDigitalSoundFX(0, 0);
        ScreenMustBeDrawn = true;
        input->DelayAllUserInput = 6;
    }

    if (data->NameInputJoyCharY == 0)
    {
        data->NameInputJoyChar = (65+data->NameInputJoyCharX);
    }
    else if (data->NameInputJoyCharY == 1)
    {
        data->NameInputJoyChar = (78+data->NameInputJoyCharX);
    }
    else if (data->NameInputJoyCharY == 2)
    {
        data->NameInputJoyChar = (97+data->NameInputJoyCharX);
    }
    else if (data->NameInputJoyCharY == 3)
    {
        data->NameInputJoyChar = (110+data->NameInputJoyCharX);
    }
    else if (data->NameInputJoyCharY == 4 && data->NameInputJoyCharX < 10)
    {
        data->NameInputJoyChar = (48+data->NameInputJoyCharX);
    }
    else if (data->NameInputJoyCharY == 4 && data->NameInputJoyCharX == 10)
    {
        data->NameInputJoyChar = '+';
    }
    else if (data->NameInputJoyCharY == 4 && data->NameInputJoyCharX == 11)
    {
        data->NameInputJoyChar = '_';
    }
    else if (data->NameInputJoyCharY == 4 && data->NameInputJoyCharX == 12)
    {
        data->NameInputJoyChar = '<';
    }
    else if (data->NameInputJoyCharY == 5)
    {
        data->NameInputJoyChar = '/';
    }

    if (ScreenTransitionStatus == FadeOut && ScreenFadeTransparency > 0.9f)
    {
        ScreenToDisplay = HighScoresScreen;
        input->DelayAllUserInput = 20;

        if (ExitToTitleScreen == true)  ScreenToDisplay = TitleScreen;

        if (data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] == '\0')
            data->HighScoresName[logic->GameMode][data->PlayerRankOnGameOver][0] = ' ';

        interface->DestroyAllButtons();
        ScreenTransitionStatus = FadeAll;
    }
}
