#include <math.h>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_opengl.h"

#include "visuals.h"

#include "game.h"
#include "interface.h"
#include "screens.h"

extern Game* game;
extern Interface* interface;
extern Screens* screens;

//-------------------------------------------------------------------------------------------------
Visuals::Visuals(char *datapath_dir)
{
    for (int index = 0; index < NumberOfSprites; index++)
    {
        Sprites[index].OGL_Texture = (GLuint)NULL;
    }

    for (int index = 0; index < NumberOfFonts; index++)
    {
        FontDefault[index] = NULL;
    }

    for (int index = 0; index < NumberOfLoadedTexts; index++)
    {
        LoadedTexts[index].OGL_Texture = (GLuint)NULL;
    }

    ColorIsRGBA = true;

    strcpy(datapath, datapath_dir);
    printf("Visuals: %s\n", datapath);
}

//-------------------------------------------------------------------------------------------------
Visuals::~Visuals(void)
{
    for (int index = 0; index < NumberOfLoadedTexts; index++)
        if (LoadedTexts[index].OGL_Texture)  glDeleteTextures( 1, &LoadedTexts[index].OGL_Texture );

        printf("Unloaded all texts from memory\n");

    for (int index = 0; index < NumberOfFonts; index++)
    {
        if (FontDefault[index] != NULL)  TTF_CloseFont(FontDefault[index]);
    }
    printf("Unloaded all TTF fonts from memory\n");

    TTF_Quit();
    printf("SDL_ttf closed\n");

    for (int index = 0; index < NumberOfSprites; index++)
    {
        if (Sprites[index].OGL_Texture)  glDeleteTextures( 1, &Sprites[index].OGL_Texture );
    }

    printf("Unloaded all sprite images from memory\n");

    SDL_Quit();
    printf("SDL+OpenGL successfully closed\n");
}

//-------------------------------------------------------------------------------------------------
bool Visuals::SetupGameWindow(int width, int height)
{
char filePath[256];

    putenv( strdup("SDL_VIDEO_CENTERED=1") );

    if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) != 0 )///SDL_INIT_EVERYTHING) != 0 )
    {
        printf("ERROR: Unable to initialize SDL: %s\n", SDL_GetError());
        return false;
    }
    else  printf("SDL (Everything) initialized\n");

    SDL_WM_SetCaption("T-Crisis 3 100% A.I.[TM] - (C)2011 By 16BitSoft(R)", NULL);
    strcpy(filePath, datapath);
    strcat(filePath, "graphics/Icon.bmp");
    SDL_WM_SetIcon(SDL_LoadBMP(filePath), NULL);

    if (width != 640 || height != 480)
    {
        Screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_FULLSCREEN );//SDL_RESIZABLE);
        if (!Screen)
        {
            printf("ERROR: SDL+OpenGL resizable window create failed\n");
            return false;
        }
        else fprintf(stdout, "SDL+OpenGL resizable window created\n");
    }
    else if (width == 640 && height == 480)
    {
        Screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_RESIZABLE);
        if (!Screen)
        {
            printf("ERROR: SDL+OpenGL resizable window create failed\n");
            return false;
        }
        else fprintf(stdout, "SDL+OpenGL resizable window created\n");
    }

    ScreenWidth = width;
    ScreenHeight = height;

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, 640, 480, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glDisable(GL_DEPTH_TEST);

    if ( TTF_Init() == -1 )
    {
        fprintf(stderr, "ERROR: TTF_Init: %s\n", TTF_GetError());
        return false;
    }
    else fprintf(stdout, "SDL_ttf initialized\n");

    #if SDL_BYTEORDER == SDL_BIG_ENDIAN
        R_Mask  = 0xff000000;
        G_Mask  = 0x00ff0000;
        B_Mask  = 0x0000ff00;
        A_Mask  = 0x000000ff;
        ColorIsRGBA = false;
    #else
        R_Mask  = 0x000000ff;
        G_Mask  = 0x0000ff00;
        B_Mask  = 0x00ff0000;
        A_Mask  = 0xff000000;
        ColorIsRGBA = true;
    #endif

    return true;
}

//-------------------------------------------------------------------------------------------------
bool Visuals::SetupGameWindowAgain(void)
{
    UnloadAllTextsFromMemory();
    UnloadSpritesFromMemory();
    interface->UnloadGUIFromMemory();

    if (game->FullScreenMode == false)
    {
        Screen = SDL_SetVideoMode(ScreenWidth, ScreenHeight, 0
                                  , SDL_OPENGL | SDL_RESIZABLE);
    }
    else
    {
        Screen = SDL_SetVideoMode(ScreenWidth, ScreenHeight
                                  , 0, SDL_OPENGL | SDL_FULLSCREEN );
    }

    if (!Screen)
    {
        printf("ERROR: SDL+OpenGL window resize failed\n");
        game->CoreFailure = true;
        return(false);
    }
    else fprintf(stdout, "SDL+OpenGL window resized\n");

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, 640, 480, 0, -1, 1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glDisable(GL_DEPTH_TEST);

    glViewport(0, 0, ScreenWidth, ScreenHeight);

    if ( interface->CreateAllGUIAndInitialize(false) == false )
    {
        game->CoreFailure = true;
        return(false);
    }

    if ( LoadSpritesIntoMemoryAndInitialize() == false )
    {
        game->CoreFailure = true;
        return(false);
    }

    if ( LoadAllTextsIntoMemory() == false )
    {
        game->CoreFailure = true;
        return(false);
    }

    screens->ScreenMustBeDrawn = true;

    return(true);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadSpritesIntoMemoryAndInitialize(void)
{
char filePath[256];
SDL_Surface *tempSurface;
bool loadedImageIsOK = true;
GLenum textureFormat = 0;
GLint  numberOfColors;
bool imageToTextureOK = true;
SDL_Surface *tempSurfaceFixed = NULL;
Uint32 saved_flags;
Uint8  saved_alpha;
SDL_Surface *boxTileSheetSurface = NULL;
int x = 1;
int y = 1;
SDL_Rect srcRect;
SDL_Rect dstRect;
int index = 0;

    for (index = 0; index < NumberOfSprites; index++)
    {
	strcpy(filePath, datapath);
        switch(index)
        {
            case 0:
                strcat(filePath, "graphics/Screen-Fade.png");
                break;

            case 1:
                strcat(filePath, "graphics/Screen-Fade.png");
                break;

            case 8:
                strcat(filePath, "graphics/cnet-Logo.png");
                break;

            case 10:
                strcat(filePath, "graphics/16BitSoft-Logo.png");
                break;

            case 20:
                strcat(filePath, "graphics/Title-BG.png");
                break;

            case 25:
                strcat(filePath, "graphics/T-Crisis3-Logo.png");
                break;

            case 30:
                strcat(filePath, "graphics/Keyboard-Controls.png");
                break;

            case 40:
                strcat(filePath, "graphics/Title-BG.png");
                break;

            case 41:
                strcat(filePath, "graphics/MIG31-BG.png");
                break;

            case 42:
                strcat(filePath, "graphics/City-BG.png");
                break;

            case 43:
                strcat(filePath, "graphics/Van-Gogh-BG.png");
                break;

            case 44:
                strcat(filePath, "graphics/Beach-BG.png");
                break;

            case 45:
                strcat(filePath, "graphics/Nissan-GTR-BG.png");
                break;

            case 50:
                strcat(filePath, "graphics/Box-Tiles.png");
                break;


            case 300:
                strcat(filePath, "graphics/PlayField.png");
                break;

            case 310:
                strcat(filePath, "graphics/Crack.png");
                break;

            case 320:
                strcat(filePath, "graphics/IMNEO.png");
                break;

            default:
                strcpy(filePath, "~");
                break;
        }

        if (index > 49 && index < 110) 
        {
	    strcpy(filePath, datapath);
	    strcat(filePath, "graphics/Box-Tiles.png");
        }

        strcat(filePath, "\0");

        if (filePath[0] != '~')
        {
            tempSurface = NULL;

            if ( ((tempSurface = IMG_Load(filePath)) != NULL) )
            {
                if ( (tempSurface->w & (tempSurface->w - 1)) != 0 )
                {
                    printf("ERROR: Image width is not a power of 2\n");
                    loadedImageIsOK = false;
                }

                if ( (tempSurface->h & (tempSurface->h - 1)) != 0 )
                {
                    printf("ERROR: Image height is not a power of 2\n");
                    loadedImageIsOK = false;
                }

                if (loadedImageIsOK == true)
                {
                    tempSurfaceFixed = SDL_CreateRGBSurface(SDL_SWSURFACE, tempSurface->w, tempSurface->h, 32,
                                                            R_Mask, G_Mask, B_Mask, A_Mask);

                    saved_flags = tempSurface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
                    saved_alpha = tempSurface->format->alpha;
                    if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
                    {
                        SDL_SetAlpha(tempSurface, 0, 0);
                    }

                    dstRect.x = 0;
                    dstRect.y = 0;
                    dstRect.w = 16;
                    dstRect.h = 16;

                    if (index > 49 && index < 59)
                    {
                        srcRect.x = x;
                        srcRect.y = y;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 50) x = 1;
                    }
                    else if (index > 59 && index < 69)
                    {
                        srcRect.x = x;
                        srcRect.y = 1+17;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 60)  x = 1;
                    }

                    else if (index > 69 && index < 79)
                    {
                        srcRect.x = x;
                        srcRect.y = 1+17*2;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 70)  x = 1;
                    }

                    else if (index > 79 && index < 89)
                    {
                        srcRect.x = x;
                        srcRect.y = 1+17*3;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 80)  x = 1;
                    }

                    else if (index > 89 && index < 99)
                    {
                        srcRect.x = x;
                        srcRect.y = 1+17*4;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 90)  x = 1;
                    }

                    else if (index > 99 && index < 109)
                    {
                        srcRect.x = x;
                        srcRect.y = 1+17*5;
                        srcRect.w = 16;
                        srcRect.h = 16;

                        if (index == 100)  x = 1;
                    }



                    if (index > 49 && index < 110)
                    {
                        SDL_BlitSurface(tempSurface, &srcRect, tempSurfaceFixed, NULL);
                        x+=17;
                    }
                    else
                    {
                        SDL_BlitSurface(tempSurface, NULL, tempSurfaceFixed, NULL);
                    }

                    if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
                    {
                        SDL_SetAlpha(tempSurfaceFixed, saved_flags, saved_alpha);
                    }

                    numberOfColors = tempSurfaceFixed->format->BytesPerPixel;
                    if (numberOfColors == 4)
                    {
                        textureFormat = GL_RGBA;
                    }
                    else if (numberOfColors == 3)
                    {
                        textureFormat = GL_RGB;
                    }
                    else
                    {
                        printf("ERROR: The image is not truecolor\n");
                        imageToTextureOK = false;
                    }

                    Sprites[index].OGL_Texture = (GLuint)NULL;
                    if (imageToTextureOK == true)
                    {
                        glPushMatrix();

                        glEnable(GL_TEXTURE_2D);

                        glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

                        glColor3f(1.0f, 1.0f, 1.0f);

                        glGenTextures(1, &Sprites[index].OGL_Texture);

                        glBindTexture(GL_TEXTURE_2D, Sprites[index].OGL_Texture);

                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
                        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

                        glTexImage2D(GL_TEXTURE_2D, 0, numberOfColors, tempSurface->w, tempSurface->h, 0
                                     , textureFormat, GL_UNSIGNED_BYTE, tempSurfaceFixed->pixels);

                        glPopMatrix();

                        Sprites[index].ScreenX = 320;
                        Sprites[index].ScreenY = 240;
                        Sprites[index].ScaleX = 1.0f;
                        Sprites[index].ScaleY = 1.0f;
                        Sprites[index].RotationDegree = 0.0f;
                        Sprites[index].Transparency = 1.0f;
                        Sprites[index].redHue = 1.0f;
                        Sprites[index].greenHue = 1.0f;
                        Sprites[index].blueHue = 1.0f;
                        Sprites[index].Smooth = true;
                        Sprites[index].FlipX = false;
                        Sprites[index].FlipY = false;
                        Sprites[index].Width = tempSurface->w;
                        Sprites[index].Height = tempSurface->h;
                        Sprites[index].CenterX = tempSurface->w / 2;
                        Sprites[index].CenterY = tempSurface->h / 2;
                        Sprites[index].AnimationTimer = -1;

                        if (tempSurface)
                        {
                            SDL_FreeSurface(tempSurface);
                            SDL_FreeSurface(tempSurfaceFixed);
                        }

                        if (boxTileSheetSurface)
                        {
                            SDL_FreeSurface(boxTileSheetSurface);
                        }
                    }
                }
            }
            else
            {
                printf("ERROR: SDL could not load image %s\n", SDL_GetError());
                loadedImageIsOK = false;
                imageToTextureOK = false;
            }
        }

        if (loadedImageIsOK == false || imageToTextureOK == false)  return false;

        strcpy(filePath, "~");

    }

    printf("Loaded and initialized all sprite images into memory\n");
    return true;
}

//-------------------------------------------------------------------------------------------------
void Visuals::UnloadSpritesFromMemory(void)
{
    for (int index = 0; index < NumberOfSprites; index++)
    {
        if (Sprites[index].OGL_Texture)  glDeleteTextures( 1, &Sprites[index].OGL_Texture );
    }
    printf("Unloaded all sprite images from memory\n");

}

//-------------------------------------------------------------------------------------------------
void Visuals::DrawSpriteOntoScreenBuffer(Uint16 index)
{
    glPushMatrix();

    glBindTexture(GL_TEXTURE_2D, Sprites[index].OGL_Texture);
    glEnable(GL_TEXTURE_2D);

    glTranslatef(Sprites[index].ScreenX
                 , Sprites[index].ScreenY, 0.0f);
    glScalef(Sprites[index].ScaleX, Sprites[index].ScaleY, 1.0f);
    glRotatef(Sprites[index].RotationDegree, 0.0f, 0.0f, 1.0f);

    glColor4f(Sprites[index].redHue, Sprites[index].greenHue, Sprites[index].blueHue
              , Sprites[index].Transparency);

    if (Sprites[index].Smooth == false)
    {

    }
    else
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    }

    GLfloat zeroWidth = 0, oneWidth = 0;
    GLfloat zeroHeight = 0, oneHeight = 0;

    if (Sprites[index].FlipX == false)
    {
        zeroWidth = -(Sprites[index].Width / 2); oneWidth = (Sprites[index].Width / 2);
    }
    else if (Sprites[index].FlipX == true)
    {
        zeroWidth = (Sprites[index].Width); oneWidth = 0;
    }

    if (Sprites[index].FlipY == false)
    {
        zeroHeight = -(Sprites[index].Height / 2); oneHeight = (Sprites[index].Height / 2);
    }
    else if (Sprites[index].FlipY == true)
    {
        zeroHeight = (Sprites[index].Height); oneHeight = 0;
    }

    int numvertices = 4;
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );

    GLfloat *vertices = new float[3*numvertices];
    GLfloat *textcoord = new float[2*numvertices];

    textcoord[0] = 0;
    textcoord[1] = 0;
    textcoord[2] = 1;
    textcoord[3] = 0;
    textcoord[4] = 1;
    textcoord[5] = 1;
    textcoord[6] = 0;
    textcoord[7] = 1;

    vertices[0] = zeroWidth;
    vertices[1] = zeroHeight;
    vertices[2] = 0;
    vertices[3] = oneWidth;
    vertices[4] = zeroHeight;
    vertices[5] = 0;
    vertices[6] = oneWidth;
    vertices[7] = oneHeight;
    vertices[8] = 0;
    vertices[9] = zeroWidth;
    vertices[10] = oneHeight;
    vertices[11] = 0;

    glVertexPointer( 3, GL_FLOAT, 0, vertices );
    glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

    glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
    delete textcoord;
    delete vertices;

    glPopMatrix();

    if ( index < 95 || (index > 125 && index < 200) )
        if (Sprites[index].AnimationTimer > 0)  Sprites[index].AnimationTimer--;
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadFontsIntoMemory(void)
{
    FontDefault[0] = NULL;
    char filePath[256];
    strcpy(filePath, datapath);
    strcat(filePath, "fonts/Default.ttf");
    if ( !(FontDefault[0] = TTF_OpenFont(filePath, 27)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    FontDefault[1] = NULL;
    if ( !(FontDefault[1] = TTF_OpenFont(filePath, 22)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    FontDefault[2] = NULL;
    if ( !(FontDefault[2] = TTF_OpenFont(filePath, 14)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    FontDefault[3] = NULL;
    if ( !(FontDefault[3] = TTF_OpenFont(filePath, 45)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    FontDefault[4] = NULL;
    strcpy(filePath, datapath);
    strcat(filePath, "fonts/Destructo.ttf");
    if ( !(FontDefault[4] = TTF_OpenFont(filePath, 27)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    FontDefault[5] = NULL;
    strcpy(filePath, datapath);
    strcat(filePath, "fonts/Destructo.ttf");
    if ( !(FontDefault[5] = TTF_OpenFont(filePath, 23)) )
    {
        printf("ERROR: Loading font: %s\n", TTF_GetError());
        return false;
    }

    printf("Loaded all TTF fonts into memory\n");

    return true;
}

//-------------------------------------------------------------------------------------------------
void Visuals::UnloadFontsFromMemory(void)
{
    for (int index = 0; index < NumberOfFonts; index++)
    {
        if (FontDefault[index] != NULL)  TTF_CloseFont(FontDefault[index]);
    }
    printf("Unloaded all TTF fonts from memory\n");
}

//-------------------------------------------------------------------------------------------------
int Visuals::NextPowerOfTwo(int x)
{
    double logbase2 = log(x) / log(2);
    return (int)round(pow(2,ceil(logbase2)));
}

//-------------------------------------------------------------------------------------------------
void Visuals::DrawTextOntoScreenBuffer(const char *textToDisplay, TTF_Font *font, GLfloat posX, GLfloat posY
                                       , Uint8 XJustification, Uint8 textRed, Uint8 textGreen, Uint8 textBlue
                                       , Uint8 outlineRed, Uint8 outlineGreen, Uint8 outlineBlue)
{
SDL_Color textColor;
SDL_Color outlineColor;
SDL_Surface *text;
SDL_Surface *textOutline;
SDL_Surface *textAndOutline;
int w,h;
SDL_Rect rect;
GLuint textTexture;

    textColor.r = textRed;
    textColor.g = textGreen;
    textColor.b = textBlue;

    outlineColor.r = outlineRed;
    outlineColor.g = outlineGreen;
    outlineColor.b = outlineBlue;

    text = TTF_RenderText_Blended(font, textToDisplay, textColor);
    textOutline = TTF_RenderText_Solid(font, textToDisplay, outlineColor);

    if (XJustification == JustifyLeft)
    {
    }
    else if (XJustification == JustifyCenter)
    {
        posX = (640 / 2) - (text->w / 2) - 3;
    }
    else if (XJustification == JustifyRight)
    {
        posX = (640 - posX) - text->w - 3;
    }
    else if (XJustification == JustifyCenterOnPoint)
    {
        posX = posX - (text->w / 2) - 3;
    }

    w = NextPowerOfTwo(5+text->w);
    h = NextPowerOfTwo(5+text->h);

    textAndOutline = SDL_CreateRGBSurface(0, w, h, 32, R_Mask, G_Mask, B_Mask, A_Mask);

    for (Sint16 y = 0; y < 5; y+=1)
    {
        for (Sint16 x = 0; x < 5; x+=1)
        {
            rect.x = x; rect.y = y;
            SDL_BlitSurface(textOutline, 0, textAndOutline, &rect);
        }
    }

    rect.x = 2;
    rect.y = 2;
    SDL_BlitSurface(text, 0, textAndOutline, &rect);

    glPushMatrix();

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

    glGenTextures(1, &textTexture);
    glBindTexture(GL_TEXTURE_2D, textTexture);

    glTexImage2D(GL_TEXTURE_2D, 0, 4, w, h, 0, GL_RGBA,
    GL_UNSIGNED_BYTE, textAndOutline->pixels );

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, textTexture);
    glColor3f(1.0f, 1.0f, 1.0f);

    glTranslatef(posX, posY, 0.0f);

    GLfloat zeroWidth = 0, oneWidth = 0;
    GLfloat zeroHeight = 0, oneHeight = 0;

    zeroWidth = 0; oneWidth = w;
    zeroHeight = 0; oneHeight = h;

    int numvertices = 4;
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );

    GLfloat *vertices = new float[3*numvertices];
    GLfloat *textcoord = new float[2*numvertices];

    textcoord[0] = 0;
    textcoord[1] = 0;
    textcoord[2] = 1;
    textcoord[3] = 0;
    textcoord[4] = 1;
    textcoord[5] = 1;
    textcoord[6] = 0;
    textcoord[7] = 1;

    vertices[0] = zeroWidth;
    vertices[1] = zeroHeight;
    vertices[2] = 0;
    vertices[3] = oneWidth;
    vertices[4] = zeroHeight;
    vertices[5] = 0;
    vertices[6] = oneWidth;
    vertices[7] = oneHeight;
    vertices[8] = 0;
    vertices[9] = zeroWidth;
    vertices[10] = oneHeight;
    vertices[11] = 0;

    glVertexPointer( 3, GL_FLOAT, 0, vertices );
    glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

    glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
    delete textcoord;
    delete vertices;

    glPopMatrix();

    SDL_FreeSurface(text);
    SDL_FreeSurface(textOutline);
    SDL_FreeSurface(textAndOutline);
    glDeleteTextures(1, &textTexture);
}

//-------------------------------------------------------------------------------------------------
bool Visuals::LoadAllTextsIntoMemory(void)
{
SDL_Color textColor;
SDL_Color outlineColor;
char textToDisplay[NumberOfLoadedTexts][128];
GLfloat blueColor[NumberOfLoadedTexts];
SDL_Surface *text;
SDL_Surface *textOutline;
SDL_Surface *textAndOutline;
int w,h;
SDL_Rect rect;
GLenum textureFormat = 0;

    textColor.r = 255; textColor.g = 255; textColor.b = 255; textColor.unused = 0;
    outlineColor.r = 25; outlineColor.g = 25; outlineColor.b = 25; outlineColor.unused = 0;

    char *copy = new char[2];
    char *reg = new char[2];
    sprintf(copy, "%c", 0xA9);
    sprintf(reg, "%c", 0xAE);

    for (int index = 0; index < NumberOfLoadedTexts; index++)
        LoadedTexts[index].OGL_Texture = (GLuint)NULL;

    int index = 0;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "TM");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "T-Crisis 3 100% A.I.");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], copy);
    strcat(textToDisplay[index], "opyright 2011, By 16BitSoft");
    strcat(textToDisplay[index], reg);

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "''GT-R Twin Turbo'' Game Engine Programmer:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Artificial Intelligence Engine Programmer:");

//    index++;
//    blueColor[index] = 1.0f;
//    strcpy(textToDisplay[index], "''mattmatteh''-''GrandMaster''-XXX Line Average!");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''-''Gift of Sight''-350 Line Average!");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Lead Game Designer:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Lead Game Programmer:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Lead Game Beta Tester:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Lead Graphic Artist:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Lead Music Composer / Sound Editor:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''DJ Fading Twilight''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Microsoft");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Windows");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Technical Advisor:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Linux Technical Advisor:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''JeZ+Lee'' & ''mattmatteh''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Apple");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Mac");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " OS X");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Technical Advisor:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Someone");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Support Designers/Programmers/Testers:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''mattmatteh''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''Daotheman''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Developed On A:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Dell");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " XPS Computer System");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "(Running Genuine Xubuntu 11.04 64bit Linux)");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Tested On A:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Falcon Northwest");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Mach V Computer System");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Gateway");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " NetBook Computer System");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "PowerSpec");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Computer System From 2002");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Technology Credits:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''SDL''1.2-Simple DirectMedia Layer");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "(SDL_Image / SDL_Mixer / SDL_TTF)");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''OpenGL");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], "''");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Software Credits:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Xubuntu 11.04 64bit Linux");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Microsoft");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Windows");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " XP Pro SP3 32bit");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Microsoft");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " Windows");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " 7 Home Premium 64bit");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Apple");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " OS X");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " 10.6.4 Snow Leopard");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Oracle");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " ''VM VirtualBox''-Virtualization Software");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "VMware");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " ''Workstation''-Virtualization Software");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''Code::Blocks''-The Cross-Platform C++ IDE");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "Apple");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], " ''Xcode''-C++ IDE");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''NeoPaint For Windows");
    strcat(textToDisplay[index], reg);
    strcat(textToDisplay[index], "''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''GIMP''-The GNU Image Manipulation Program");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''GoldWave''-Sound Editor for Windows");
    strcat(textToDisplay[index], reg);

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "Top Beta Testers:");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''EvanR''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''demonicmaniac''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''juanmabcirc''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "''Jupp3''");

    index++;
    blueColor[index] = 1.0f;
    strcpy(textToDisplay[index], "You!");

    index++;
    blueColor[index] = 0;
    strcpy(textToDisplay[index], "''A 100% By JeZ+Lee''...");

    TotalNumberOfLoadedTexts = index;

    for (int textIndex = 0; textIndex < (TotalNumberOfLoadedTexts+1); textIndex++)
    {
        LoadedTexts[textIndex].OGL_Texture = (GLuint)NULL;

        text = NULL;
        textAndOutline = NULL;

        LoadedTexts[textIndex].blueHue = blueColor[textIndex];

        text = TTF_RenderText_Blended(FontDefault[0], textToDisplay[textIndex], textColor);
        textOutline = TTF_RenderText_Solid(FontDefault[0], textToDisplay[textIndex], outlineColor);

        w = NextPowerOfTwo(7+text->w);
        h = NextPowerOfTwo(7+text->h);

        LoadedTexts[textIndex].TextWidth = 7+text->w;
        LoadedTexts[textIndex].TextHeight = 7+text->h;

        LoadedTexts[textIndex].Width = w;
        LoadedTexts[textIndex].Height = h;

        textAndOutline = SDL_CreateRGBSurface(0, w, h, 32, R_Mask, G_Mask, B_Mask, A_Mask);

        for (int yOffSet = 0; yOffSet < 7; yOffSet++)
            for (int xOffSet = 0; xOffSet < 7; xOffSet++)
                {
                    rect.x = xOffSet; rect.y = yOffSet;
                    SDL_BlitSurface(textOutline, 0, textAndOutline, &rect);
                }

        rect.x = 3;
        rect.y = 3;
        SDL_BlitSurface(text, 0, textAndOutline, &rect);

        glPushMatrix();

        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0.0f, 0.0f, 0.0f, 1.0f);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

        glColor3f(1.0f, 1.0f, 1.0f);

        glGenTextures(1, &LoadedTexts[textIndex].OGL_Texture);
        glBindTexture(GL_TEXTURE_2D, LoadedTexts[textIndex].OGL_Texture);

        if (ColorIsRGBA == true) textureFormat = GL_RGBA;
        else  textureFormat = GL_BGRA;

        glTexImage2D(GL_TEXTURE_2D, 0, 4, w, h, 0, textureFormat,
        GL_UNSIGNED_BYTE, textAndOutline->pixels );

        glPopMatrix();

        SDL_FreeSurface(text);
        SDL_FreeSurface(textOutline);
        SDL_FreeSurface(textAndOutline);
    }

    printf("Loaded all texts into memory\n");
    return(true);
}

//-------------------------------------------------------------------------------------------------
void Visuals::UnloadAllTextsFromMemory(void)
{
    for (int index = 0; index < NumberOfLoadedTexts; index++)
        if (LoadedTexts[index].OGL_Texture)  glDeleteTextures( 1, &LoadedTexts[index].OGL_Texture );

        printf("Unloaded all texts from memory\n");
}

//-------------------------------------------------------------------------------------------------
void Visuals::DrawLoadedTextOntoScreenBuffer(Uint16 index, GLfloat posX, GLfloat posY)
{
    posX = posX - (LoadedTexts[index].TextWidth / 2) - 2;

    glPushMatrix();

    glBindTexture(GL_TEXTURE_2D, LoadedTexts[index].OGL_Texture);
    glEnable(GL_TEXTURE_2D);

    glTranslatef(posX, posY, 0.0f);

    glColor4f(1.0f, 1.0f, LoadedTexts[index].blueHue, 1.0f);

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    GLfloat zeroWidth = 0, oneWidth = 0;
    GLfloat zeroHeight = 0, oneHeight = 0;

    zeroWidth = 0; oneWidth = (LoadedTexts[index].Width);
    zeroHeight = 0; oneHeight = (LoadedTexts[index].Height);

    int numvertices = 4;
    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );

    GLfloat *vertices = new float[3*numvertices];
    GLfloat *textcoord = new float[2*numvertices];

    textcoord[0] = 0;
    textcoord[1] = 0;
    textcoord[2] = 1;
    textcoord[3] = 0;
    textcoord[4] = 1;
    textcoord[5] = 1;
    textcoord[6] = 0;
    textcoord[7] = 1;

    vertices[0] = zeroWidth;
    vertices[1] = zeroHeight;
    vertices[2] = 0;
    vertices[3] = oneWidth;
    vertices[4] = zeroHeight;
    vertices[5] = 0;
    vertices[6] = oneWidth;
    vertices[7] = oneHeight;
    vertices[8] = 0;
    vertices[9] = zeroWidth;
    vertices[10] = oneHeight;
    vertices[11] = 0;

    glVertexPointer( 3, GL_FLOAT, 0, vertices );
    glTexCoordPointer( 2, GL_FLOAT, 0, textcoord );

    glDrawArrays( GL_TRIANGLE_FAN, 0, numvertices );
    delete textcoord;
    delete vertices;

    glPopMatrix();
}

//-------------------------------------------------------------------------------------------------
void Visuals::ClearScreenBufferWithColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
{
    glClearColor(red, green, blue, alpha);
    glClear(GL_COLOR_BUFFER_BIT);
}

//-------------------------------------------------------------------------------------------------
void Visuals::DisplayScreenBufferOntoDisplay(void)
{
    glFlush();
    SDL_GL_SwapBuffers();
}
